﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.ComponentModel;
using System.Data;
using System.Data.Common;
using System.Data.SqlClient;
using System.Diagnostics;
using System.Reflection;
using System.Text;
using System.Xml;

namespace BigfootSQL
{
    /// <summary>
    /// This is a simple SQL syntax builder helper class. It aids in the creation of a SQL statement
    /// by auto creting parameters, as well as preventing against injection attacks etc. It also uses 
    /// the DAAB class and ObjectHelper classes to execute the query and hydrate the Model objects.
    /// 
    /// It was designed with simplicity and speed in mind. It is a great replacement to writing
    /// directly against the ADO.NET providers. It is not meant to be a full ORM but rather a rapid query
    /// execution and object hydration helper.
    /// 
    /// It uses a fluid interface for simplicity of code. You must know how to write SQL as it is a light
    /// SQL code builder while automating the rest. It does not generate SQL for you, rather it makes the 
    /// writing, executing, and results mapping simple.
    /// 
    /// Examples (Assumes there is a DB variable of type SqlHelper):
    ///     Select a list of orders:
    ///         List<OrderListItem> obj;
    ///         obj = DB.SELECT("OrderID, OrderDate, ShipToCity").FROM("Orders").WHERE("ShipToState","FL").ExecuteCollection<OrderListItem>();
    ///     
    ///     Select a single value typed to the correct type:
    ///         datetime d;
    ///         d = DB.SELECT("OrderDate").FROM("Orders").WHERE("OrderID",OrderID).ExecuteScalar<datetime>()
    /// 
    /// It has several other Execute methods to retreive DataReaders, DataSets, and many others. Also has ExecuteNonQuery for executing 
    /// void queries.
    /// </summary>
    public class SqlHelper
    {

        #region "Constructors / Private variables"

        StringBuilder _sql = new StringBuilder();
        List<DbParameter> _params = new List<DbParameter>();
        public string ConnectionString;


        // See more constructors bellow for dotnetnuke integration
        public SqlHelper(){}
        public SqlHelper(string connectionString)
        {
            ConnectionString = connectionString;
        }

        #endregion


        #region "SQL Builder methods"

        public StringBuilder RawBuilder
        {
            get { return _sql; }
        }
        
        /// <summary>
        /// Add literal SQL statement to the query
        /// </summary>
        /// <param name="sql">SQL Fragment to add</param>
        public SqlHelper Add(string sql)
        {
            return Append(sql);
        }

        private SqlHelper Append(string sql)
        {
            if (_sql.Length > 0 && sql.EndsWith(" ") == false)
                _sql.Append(" ");
            _sql.Append(Translate(sql));
            return this;
        }

        /// <summary>
        /// Add a parameter to a WHERE statement. Will generate ColumnName = 'Value' (quotes only added if it is a string)
        /// </summary>
        /// <param name="wherecolumn">The name of the column to search</param>
        /// <param name="value">The value to search for</param>
        public SqlHelper Add(string wherecolumn, object value)
        {
            return Add(wherecolumn, "=", value, false);
        }

        /// <summary>
        /// Add a paramter to a WHERE statement. Will generate ColumnName {Operator} 'Value' (quotes only added if it is a string)
        /// </summary>
        /// <param name="wherecolumn">The of the column to search</param>
        /// <param name="value">The value to search for. If it is a string it is properly espaped etc.</param>
        /// /// <param name="isSet">Identifies this comparison as a set statement. Needed for setting null values</param>
        public SqlHelper Add(string wherecolumn, object value, bool isSet )
        {
            return Add(wherecolumn, "=", value, isSet);
        }

        /// <summary>
        /// Add a paramter to a WHERE statement. Will generate ColumnName {Operator} 'Value' (quotes only added if it is a string)
        /// </summary>
        /// <param name="wherecolumn">The of the column to search</param>
        /// <param name="operator">The operator for the search. e.g. = <= LIKE <> etc.</param>
        /// <param name="value">The value to search for. If it is a string it is properly espaped etc.</param>
        public SqlHelper Add(string wherecolumn, string @operator, object value)
        {
            return Add(wherecolumn, @operator, value, false);
        }

        /// <summary>
        /// Add a paramter to a WHERE statement. Will generate ColumnName {Operator} 'Value' (quotes only added if it is a string)
        /// </summary>
        /// <param name="wherecolumn">The of the column to search</param>
        /// <param name="operator">The operator for the search. e.g. = <= LIKE <> etc.</param>
        /// <param name="value">The value to search for. If it is a string it is properly espaped etc.</param>
        /// <param name="isSet">Identifies this comparison as a set statement. Needed for setting null values</param>
        public SqlHelper Add(string wherecolumn, string @operator, object value, bool isSet)
        {
            if (value == null)
            {
                Add(wherecolumn);
                return isSet ? Add("= NULL") : 
                               Add("IS NULL");
            }
            else
                return Add(wherecolumn).Add(@operator).Add(AddTempParam(value));
        }

        public SqlHelper LIKE(string wherecolumn, string value)
        {
            return Add(wherecolumn).Add("LIKE").Add("'%" + EscapeForLike(value,true) + "%'");
        }

        public SqlHelper LIKE(string wherecolumn, string value, bool fullTextSearch)
        {
            if (!string.IsNullOrEmpty(value) && fullTextSearch)
            {
                var firstWord = true;
                OpenParenthesis();
                foreach (var term in value.Split(' '))
                {
                    if (firstWord) firstWord = false; else AND();
                    LIKE(wherecolumn, term);
                }
                CloseParenthesis();
                return this;
            }
            else
            {
                return LIKE(wherecolumn, value);
            }
        }

        public SqlHelper SELECT(string sql)
        {
            return Add("SELECT " + sql);
        }

        public SqlHelper CASE()
        {
            return Add("CASE");
        }

        public SqlHelper CASE(string condition, string trueResult, string falseResult)
        {
            return CASE().WHEN(condition, trueResult).ELSE(falseResult).END();
        }

        public SqlHelper AS(string columnName)
        {
            return Add("AS " + columnName);
        }

        public SqlHelper WHEN(string condition, string trueResult)
        {
            return Add(string.Format("WHEN {0} THEN {1}", condition, trueResult));
        }

        public SqlHelper ELSE(string elseResult)
        {
            return Add("ELSE " + elseResult);
        }

        public SqlHelper END()
        {
            return Add("END");
        }

        /// <summary>
        /// Used to returned a paged result set
        /// </summary>
        /// <param name="columns">Columns to include in the paged result. When paging this you can specify columns only once</param>
        /// <param name="pageorderby">This is used to generate the RowNumber field resulting in: 
        /// ROW_NUMBER() OVER(ORDER BY Field1, Field2) AS RowNumber </param>
        /// <param name="pageSize">How many records per page</param>
        /// <returns></returns>
        public SqlHelper SELECTPAGED(string columns, string pageorderby, int pageSize)
        {
            _pageorderby = pageorderby;
            _pageSize = pageSize;
            return Add("SELECT " + columns + ", ROW_NUMBER() OVER(ORDER BY " + pageorderby + " ) AS RowNumber");
        }
        private string _pageorderby;
        private int _pageSize;

        /// <summary>
        /// Call this function to identify which page to get. Must be called at the end of the statement.
        /// </summary>
        /// <param name="currentpage">The current page (0 based)</param>
        public SqlHelper PAGE(int currentpage)
        {
            AddParam("StartRow", (_pageSize * currentpage));
            AddParam("EndRow", (_pageSize * currentpage) + _pageSize + 1);
            _sql.Insert(0, "SELECT * FROM (");
            _sql.AppendFormat(") PagedResult WHERE RowNumber > @StartRow AND RowNumber < @EndRow ORDER BY " + _pageorderby);
            return this;
        }
        
        public SqlHelper SELECT_ALL_FROM(string tablename)
        {
            return Add("SELECT * FROM").Add(Translate(tablename));
        }

        public SqlHelper SELECT_ALL_FROM(string tablename, int topCount)
        {
            return Add("SELECT TOP  " + topCount + " * FROM").Add(Translate(tablename));
        }

        public SqlHelper SELECT_COUNT_ALL_FROM(string tablename)
        {
            return Add("SELECT COUNT(*) FROM").Add(Translate(tablename));
        }
        
        public SqlHelper SELECT(params string[] columns)
        {
            var s = "SELECT ";
            var firstcolumn = true;
            foreach (string c in columns)
            {
                if (firstcolumn) {
                    s += columns;
                    firstcolumn = false;
                }
                else
                {
                    s += ", " + c;
                }
            }
            return Add(s);
        }

        public SqlHelper SELECT_IDENTITY()
        {
            return Add("SELECT @@IDENTITY");
        }

        public SqlHelper INNERJOIN(string sql)
        {
            return Add("INNER JOIN " + sql);
        }

        public SqlHelper LEFTJOIN(string sql)
        {
            return Add("LEFT JOIN " + sql);
        }

        public SqlHelper ON(string sql)
        {
            return Add("ON " + sql);
        }

        public SqlHelper ON(string leftcolumn, string rightcolumn)
        {
            return Add("ON " + leftcolumn + " = " + rightcolumn);
        }

        public SqlHelper FROM(string sql)
        {
            return Add("FROM " + sql);
        }

        public SqlHelper WHERE()
        {
            return Add("WHERE");
        }

        public SqlHelper WHERE(string sql)
        {
            return Add("WHERE " + sql);
        }

        public SqlHelper WHERE(string columnname, object value)
        {
            return Add("WHERE").Add(columnname, value);
        }

        public SqlHelper WHERE(string columnname, string @operator, object value)
        {
            return Add("WHERE").Add(columnname, @operator, value);
        }

        public SqlHelper IN(string columnname, string values)
        {
            return Add(columnname + " IN (" + EscapeApostrophe(values) + ")");
        }

        public SqlHelper ORDERBY(string sql)
        {
            return Add("ORDER BY " + sql);
        }

        public SqlHelper GROUPBY(string sql)
        {
            return Add("GROUP BY " + sql);
        }

        public SqlHelper HAVING(string sql)
        {
            return Add("HAVING " + sql);
        }

        public SqlHelper INSERTINTO(string tablename, string columns)
        {
            return Add("INSERT INTO " + tablename + "(" + columns + ")");
        }

        public SqlHelper OP()
        {
            return OpenParenthesis();
        }

        public SqlHelper OpenParenthesis()
        {
            return Add("(");
        }

        public SqlHelper OP(string wherecolumn, object value)
        {
            return OpenParenthesis(wherecolumn, value);
        }

        public SqlHelper OpenParenthesis(string wherecolumn, object value)
        {
            return Add("(").Add(wherecolumn,value);
        }

        public SqlHelper CP()
        {
            return CloseParenthesis();
        }

        public SqlHelper CloseParenthesis()
        {
            return Add(")");
        }

        public SqlHelper UPDATE(string sql)
        {
            return Add("UPDATE " + sql);
        }

        public SqlHelper SET()
        {
            return Add("SET");
        }

        public SqlHelper SET(string columnname, object value)
        {
            AddIfNotFound("SET");
            
            if (!_sql.ToString().TrimEnd().EndsWith(" SET") && 
                !_sql.ToString().TrimEnd().EndsWith(","))
                Add(",");

            Add(columnname, value, true);

            return this;
        }

        public SqlHelper DELETEFROM(string sql)
        {
            return Add("DELETE FROM " + sql);
        }

        public SqlHelper AND()
        {
            return Add("AND");
        }

        public SqlHelper AND(string sql)
        {
            return Add("AND " + sql);
        }

        public SqlHelper AND(string column, object value)
        {
            return Add("AND").Add(column, value);
        }

        public SqlHelper AND(string column, string @operator, object value)
        {
            return Add("AND").Add(column, @operator, value);
        }

        public SqlHelper OR() 
        {
            return Add("OR"); 
        }

        public SqlHelper OR(string sql)
        {
            return Add("OR " + sql);
        }

        public SqlHelper OR(string column, object value)
        {
            return Add("OR").Add(column, value);
        }

        public SqlHelper OR(string column, string @operator, object value )
        {
            return Add("OR").Add(column, @operator, value);
        }

        public SqlHelper VALUES(string sql)
        {
            return Add("VALUES ( " + sql + " ) ");
        }

        public SqlHelper VALUES(params object[] ps)
        {
            Add("VALUES (");
            var first = true;
            foreach (var p in ps)
            {
                if (first) 
                {
                    Add(AddTempParam(p));
                    first = false;
                }
                else
                {
                    Add(",").Add(AddTempParam(p));
                }
            }
            return Add(")");
        }

        public SqlHelper DECLARE(string varname, string vartype)
        {
            if (varname.StartsWith("@")==false ) varname = "@" + varname;
            return Add("DELCARE " + varname + ((vartype.ToLower() == "table") ? " AS " : " ") + vartype);
        }

        public SqlHelper DECLARE(string varname, string vartype, object value)
        {
            if (varname.StartsWith("@")==false ) varname = "@" + varname;
            return DECLARE(varname, vartype).Add("SET " + varname + " = " + AddTempParam(value));
        }

        public SqlHelper DECLARE_TABLE(string varname, string columns)
        {
            return DECLARE(varname, "TABLE").Add("( " + columns + " )");
        }

        /// <summary>
        /// Execute a StoredProcedure. By passing the SP name and a list values as parameters to the SP
        /// </summary>
        /// <param name="spname">SP Name</param>
        /// <param name="ps">Parameters for the Stored Procedure in the proper order</param>
        public SqlHelper EXECUTE(string spname, params object[] ps)
        {
            Add("EXECUTE " + spname);
            var first = true;
            foreach (var p in ps)
            {
                if (first)
                    first = false;
                else
                    Add(",");

                Add(AddTempParam(p));
            }
            return this;
        }

        /// <summary>
        /// Execute a function by passing the function anem of and a list of paremters. 
        /// e.g. SELECT_SCALARFUNCTION("GetOrderName", OrderDate, CustomerID)
        /// </summary>
        /// <param name="fname">Name of the function</param>
        /// <param name="ps">List of parameters values to pass into the function. Must be in the right order</param>
        public SqlHelper SELECT_SCALARFUNCTION(string fname, params object[] ps)
        {
            Add("SELECT " + fname + "(");
            var first = true;
            foreach (var p in ps)
            {
                if (first)
                    first = false;
                else
                    Add(",");

                Add(AddTempParam(p));
            }
            Add(")");
            return this;
        }

        private void AddIfNotFound(string statement)
        {
            if (_sql.ToString().IndexOf(statement) == -1)
            {
                Add(statement);
            }
        }

        #endregion

        
        #region "AddParam / AddTempParam"
        
        /// <summary>
        /// Adds a parameter with the provided value and returns the created parameter name
        /// Used when creating dynamic queries and the parameter is not important outside of the inmediate query
        /// Used internally to auto created parameters for the WHERE AND OR and other statements
        /// </summary>
        /// <param name="value">The value of the parameter</param>
        /// <returns>The generated name for the parameter</returns>
        public string AddTempParam(object value)
        {
            var name = "_tempParam" + _params.Count;
            AddParam(name, value);
            return "@" + name;
        }

        /// <summary>
        /// Adds a named parameter to the query
        /// </summary>
        /// <param name="name">The name of the parameter</param>
        /// <param name="value">The value of the paremter</param>
        public SqlHelper AddParam(string name, object value)
        {
            if (name.StartsWith("@")) name=name.Substring(1);
            var param = DAAB.ProviderFactory().CreateParameter();
            param.ParameterName = name;
            param.Value = value;
            _params.Add(param);
            return this;
        }

        private bool HasParams
        {
            get { return _params.Count > 0; }
        }

        #endregion


        #region "ToString / DebugSql / Clear"

        /// <summary>
        /// Clear the current query
        /// </summary>
        public void Clear()
        {
            _sql = new StringBuilder();
            _params=new List<DbParameter>();
        }

        /// <summary>
        /// Auto writes the finished statement as close as possible.
        /// </summary>
        public override string ToString()
        {
            Debug.WriteLine(DebugSql);
            return _sql.ToString();
        }
        
        /// <summary>
        /// Creates an executable SQL statement including declaration of SQL parameters for debugging purposes.
        /// </summary>
        public string DebugSql
        {
            get
            {
                var value = "====NEW QUERY====\r\n";
                foreach (DbParameter param in _params)
                {
                    var addQuotes = (param.DbType == DbType.String);
                    value += "DECLARE @" + param.ParameterName + " " + param.DbType;
                    if (param.Value == null)
                        value += " SET @" + param.ParameterName + " = NULL";
                    else
                        value += " SET @" + param.ParameterName + " = " + 
                                    ((addQuotes) ? "'" + EscapeApostrophe(param.Value.ToString()) + "'"
                                                 : EscapeApostrophe(param.Value.ToString()));
                    value += "\r\n";
                }
                value += _sql + "\r\n";
                
                //Add \r\n before each of these words
                string[] words = { "SELECT", "FROM", "WHERE", "INNER JOIN", "LEFT JOIN", "ORDER BY", "GROUP BY", "DECLARE", 
                                     "SET", "VALUES", "INSERT INTO", "DELETE FROM", "UPDATE" };
                foreach (string w in words)
                    value = value.Replace(w, "\r\n" + w);
                
                // Return the value
                return value;
            }
        }

        #endregion


        #region "DotNetNuke integration: Translate | GetName"

        public String Owner;
        public String Qualifier;
        public String ModuleQualifier;

        public SqlHelper(string owner, string qualifier)
        {
            Owner = owner;
            Qualifier = qualifier;
        }
        public SqlHelper(string owner, string qualifier, string moduleQualifier)
        {
            Owner = owner;
            Qualifier = qualifier;
            ModuleQualifier = moduleQualifier;
        }
        public SqlHelper(string owner, string qualifier, string modulequalifier, string connectionString)
        {
            Owner = owner;
            Qualifier = qualifier;
            ModuleQualifier = modulequalifier;
            ConnectionString = connectionString;
        }

        /// <summary>
        /// Used to support DotNetNuke queries where database object names
        /// are dynamic based on the prefix of the installation.
        /// By using {{{TableName}}} it will translate into a proper DNN core table name
        /// By using {{TableName}} it will translate into a proper custom DNN Module table name
        /// </summary>
        /// <param name="sql">The sql fragment to translate</param>
        /// <returns>The translated string</returns>
        public string Translate(string sql)
        {
            // Translate databaseOwner and objectQualifier tokens
            sql = sql.Replace("{databaseOwner}", Owner);
            sql = sql.Replace("{objectQualifier}", Qualifier);

            // Get the tokens for Core {{{tablename}}} and Module {{tablename}}
            var tokens = GetTokens(sql,"{{{", "}}}");
            GetTokens(sql,tokens,"{{", "}}");
            
            // Replace the tokens
            if (tokens.Count > 0)
                foreach (var t in tokens.Keys)
                    sql = sql.Replace(t, GetName(tokens[t]));
            
            // Return the translated sql
            return sql;
        }

        private string GetName(string name)
        {
            return name.StartsWith("{{{") ? GetName(name, false) : 
                                            GetName(name, true);
           
        }

        private string GetName(string name, bool addModuleQualifier)
        {
            var value = Owner + Qualifier;
            if (!string.IsNullOrEmpty(ModuleQualifier) &&  addModuleQualifier)
                value += ModuleQualifier;
            value += name;
            return value;
        }


        #endregion
        

        #region "Static Methods"
        /// <summary>
        /// Converts a collection of int into a value list to be used in and IN statement
        /// </summary>
        /// <param name="ps">List of int values</param>
        /// <returns>Properly formatted list for IN statement</returns>
        public static string ArrayToINint(params int[] ps)
        {
            var result = "";
            foreach (var p in ps)
            {
                if (result.Length > 0) result += ", ";
                result += p.ToString();
            }
            return result;
        }

        /// <summary>
        /// Escapes the apostrophe on strings
        /// </summary>
        /// <param name="sql">SQL statement fragment</param>
        /// <returns>The clean SQL fragment</returns>
        public static string EscapeApostrophe(string sql)
        {
            sql = sql.Replace("'", "''");
            return sql;
        }

        /// <summary>
        /// Properly excapes a string to be included in a LIKE statement
        /// </summary>
        /// <param name="value">Value to search for</param>
        /// <param name="escapeApostrophe">Weather to escape the apostrophe. Prevents double escaping of apostrophes</param>
        /// <returns>The transalted value ready to be used in a LIKE statement</returns>
        public static string EscapeForLike(string value, bool escapeApostrophe)
        {
            string[] specialChars = {"%", "_", "-", "^"};
            string newChars;

            // Escape the [ braket
            newChars = value.Replace("[", "[[]");

            // Replace the special chars
            foreach (string t in specialChars){
                newChars = newChars.Replace(t, "[" + t + "]");
            }

            // Escape the apostrophe if requested
            if (escapeApostrophe)
                newChars = EscapeApostrophe(newChars);

            return newChars;
        }

        /// <summary>
        /// Retreives a list of tokens included in a string
        /// </summary>
        /// <param name="s">The string object being exteded</param>
        /// <param name="startDelimiter">The start delimiter for the token</param>
        /// <param name="endDelimiter">The end delimiter for the token</param>
        /// <returns>A dictionary of tokens present in the string</returns>
        public static Dictionary<string, string> GetTokens(string s, string startDelimiter, string endDelimiter)
        {
            return GetTokens(s, new Dictionary<string, string>(), startDelimiter, endDelimiter);
        }

        /// <summary>
        /// Retreives a list of tokens included in a string
        /// </summary>
        /// <param name="s">The string object being exteded</param>
        /// <param name="startDelimiter">The start delimiter for the token</param>
        /// <param name="endDelimiter">The end delimiter for the token</param>
        /// <param name="tokens">The dictionary of tokens to add the new finds to</param>
        /// <returns>A dictionary of tokens present in the string</returns>
        public static Dictionary<string, string> GetTokens(string s, Dictionary<string, string> tokens, string startDelimiter, string endDelimiter)
        {
            var marker = 0;
            do
            {
                // Get the start and end of the tokens
                var startIndex = s.IndexOf(startDelimiter, marker);
                int endIndex;

                if (startIndex > -1)
                    endIndex = s.IndexOf(endDelimiter, startIndex);
                else
                    break;

                // Add the token and increase the marker
                if (endIndex > -1)
                {
                    var token = s.Substring(startIndex, (endIndex + endDelimiter.Length) - startIndex);
                    var tokenValue = token.Substring(startDelimiter.Length, token.Length - (startDelimiter.Length + endDelimiter.Length));
                    tokens.Add(token, tokenValue);
                    marker = endIndex + endDelimiter.Length;
                }

            } while (marker < s.Length);

            return tokens;
        }
        #endregion


        #region "Execute SQL"

        /// <summary>
        /// Executes the query and returns a Scalar value
        /// </summary>
        /// <returns>Object (null when dbnull value is returned)</returns>
        public object ExecuteScalar()
        {
            var rvalue = (HasParams)
                                ? DAAB.ExecuteScalar(ConnectionString, CommandType.Text, ToString(), _params.ToArray())
                                : DAAB.ExecuteScalar(ConnectionString, CommandType.Text, ToString());
            
            if (rvalue == DBNull.Value) rvalue = null;

            return rvalue;
        }

        /// <summary>
        /// Executes the query and returns a Scalar value for the specific generic value
        /// </summary>
        /// <returns>A typed object of T</returns>
        public T ExecuteScalar<T>()
        {
            var rvalue = ExecuteScalar();
            if (rvalue != null)
            {
                var tc = TypeDescriptor.GetConverter(typeof(T));
                return (T)tc.ConvertFromInvariantString(rvalue.ToString());
            }
            
            return default(T);
        }

        /// <summary>
        /// Appends a SELECT @@IDENTITY statement to the query and then executes
        /// </summary>
        /// <returns>The identity of the just inserted record</returns>
        public int ExecuteScalarIdentity()
        {
            SELECT_IDENTITY();
            return ExecuteScalarInt();
        }

        /// <summary>
        /// Executes the query and returns a sclar value of type int
        /// </summary>
        public int ExecuteScalarInt()
        {
            return ExecuteScalar<int>();
        }

        /// <summary>
        /// Executes the query and returns a sclar value of type string
        /// </summary>
        public string ExecuteScalarString()
        {
            return ExecuteScalar<string>();
        }

        /// <summary>
        /// Executes a query that does not return a value
        /// </summary>
        public int ExecuteNonquery()
        {
            return (HasParams)
                ? DAAB.ExecuteNonQuery(ConnectionString, CommandType.Text, ToString(), _params.ToArray())
                : DAAB.ExecuteNonQuery(ConnectionString, CommandType.Text, ToString());
        }

        /// <summary>
        /// Executes a query and hydrates an object with the result
        /// </summary>
        /// <typeparam name="T">The type of the object to hydrate and return</typeparam>
        /// <returns>I hydrated object of the type specified</returns>
        public T ExecuteObject<T>()
        {
            var reader = ExecuteReader();
            return FillObject<T>(reader);
        }

        /// <summary>
        /// Returns a dictionary of FieldName. Works like Scalar except it can 
        /// return multiple fields values for the first record of the resultset
        /// </summary>
        /// <returns>Dictionary with FieldName, Value</returns>
        public Dictionary<string, object> ExecuteValues()
        {
            var reader = ExecuteReader();
            try
            {
                Dictionary<string, object> values = null;
                if (reader.Read() == false) { return values; }
                values = new Dictionary<string, object>();
                for (int i = 0; i < 100; i++)
                {
                    try
                    {
                        var key = reader.GetName(i);
                        object value = null;
                        if (reader.IsDBNull(i) == false)
                        {
                            value = reader[i];
                        }
                        values.Add(key,value);
                    }
                    catch (IndexOutOfRangeException)
                    {
                        return values;
                    }
                }
                return values;
            } 
            finally 
            {
                reader.Close();
            }
        }

        /// <summary>
        /// Executes the query and returnes a collection of strings. 
        /// Useful when needing a quick lookup set of values
        /// </summary>
        /// <returns>A string collection</returns>
        public List<string> ExecuteStringCollection()
        {
            var reader = ExecuteReader();

            try
            {
                List<string> values = null;
                while (reader.Read())
                {   
                    if (reader.IsDBNull(0) == false)
                    {
                        if (values == null) values = new List<string>();
                        values.Add(reader[0].ToString());
                    }
                }
                return values;
            }
            finally
            {
                reader.Close();
            }
        }

        /// <summary>
        /// Executes the query and maps the results to a collection of objects 
        /// of the type specified through the generic argument
        /// </summary>
        /// <typeparam name="T">The of object for the collection</typeparam>
        /// <returns>A collection of T</returns>
        public List<T> ExecuteCollection<T>()
        {
            var reader = ExecuteReader();
            return FillCollection<T>(reader);
        }

        /// <summary>
        /// Executes the query and returns a DataReader
        /// </summary>
        public DbDataReader ExecuteReader()
        {
            return (HasParams)
                       ? DAAB.ExecuteReader(ConnectionString, CommandType.Text, ToString(), _params.ToArray())
                       : DAAB.ExecuteReader(ConnectionString, CommandType.Text, ToString());
        }

        /// <summary>
        /// Executes the query and returns a filled Dataset with the resultset
        /// </summary>
        /// <returns>A dataset</returns>
        public DataSet ExecuteDataset()
        {
           return (HasParams)
                   ? DAAB.ExecuteDataset(ConnectionString, CommandType.Text, ToString(), _params.ToArray())
                   : DAAB.ExecuteDataset(ConnectionString, CommandType.Text, ToString());

        }

        /// <summary>
        /// Executes ther query and adds the results to the provided dataset
        /// </summary>
        /// <param name="data">The dataset to fill with this query</param>
        /// <returns>The dataset passed in</returns>
        public DataSet ExecuteDataset(DataSet data)
        {
            DataSet localdata;
            localdata = (HasParams)
                       ? DAAB.ExecuteDataset(ConnectionString, CommandType.Text, ToString(), _params.ToArray())
                       : DAAB.ExecuteDataset(ConnectionString, CommandType.Text, ToString());
            // Make sure the table names match
            if (data.Tables.Count == 1 && localdata.Tables.Count == 1)
                localdata.Tables[0].TableName = data.Tables[0].TableName;

            data.Merge(localdata, false, MissingSchemaAction.Ignore);
            return data;
        }

        #endregion
        

        #region "Object Hydration"

        #region "Cache"

        /// <summary>
        /// Generic cache class to cache object graphs and type converters
        /// </summary>
        private static class Cache
        {
            static Dictionary<string, object> _cache = new Dictionary<string, object>();


            public static bool Contains(String key) { return _cache.ContainsKey(key); }
            public static object GetValue(String key)
            {
                return (_cache.ContainsKey(key)) ? _cache[key] : null;
            }
            public static void Add(String key, object data)
            {
                if (_cache.ContainsKey(key))
                    _cache[key] = data;
                else
                    _cache.Add(key, data);
            }
            public static void Remove(string key)
            {
                if (_cache.ContainsKey(key)) _cache.Remove(key);
            }
        }

        #endregion


        #region "GetValue / TypeConverters"

        public interface ITypeConverter { object GetValue(Type fieldType, object value); }

        /// <summary>
        /// Adds a custom type converter. When hydrating an object, certain complex types might have special needs 
        /// when it comes to deserializing from database. You can add a customer converter for the type, in order to 
        /// have the auto hydration use the this type converter instead of the Default type converter
        /// </summary>
        /// <param name="typeName">Full name of the type to use the converter for</param>
        /// <param name="converter">The converter oject to use. It must implement the ITypeConverter interface</param>
        public static void AddTypeConverter(string typeName, ITypeConverter converter)
        {
            typeName = typeName.ToLower();
            var list = GetAllConverters();
            if (list.ContainsKey(typeName))
                list[typeName] = converter;
            else
                list.Add(typeName, converter);
        }

        /// <summary>
        /// Retreives a list of ITypeConverters in the system
        /// </summary>
        private static Dictionary<String, ITypeConverter> GetAllConverters()
        {
            const string key = "TypeConverters";
            if (!Cache.Contains(key))
            {
                var types = new Dictionary<String, ITypeConverter>();
                Cache.Add(key, types);
            }
            return (Dictionary<String, ITypeConverter>)Cache.GetValue(key);
        }

        /// <summary>
        /// Gets the database value converter for a certain Type name
        /// </summary>
        /// <param name="typeName">The full name of the type e.g. System.String</param>
        /// <returns>A ITypeConverter object from the cache</returns>
        private static ITypeConverter GetTypeConverter(String typeName)
        {
            typeName = typeName.ToLower();
            var converters = GetAllConverters();
            return (converters.ContainsKey(typeName)) ? converters[typeName] : null;
        }

        /// <summary>
        /// Converts a database value to its object representation. Uses the TypeConverter cache to properly translate complex types
        /// </summary>
        /// <param name="fieldType">The Type of the field to convert to</param>
        /// <param name="value">The database value. e.g. the object from the data reader ordinal</param>
        /// <returns>A properly converted object</returns>
        public static object GetValue(Type fieldType, object value)
        {
            var typeName = fieldType.Name;
            object newValue = null;
            Type baseType = fieldType.BaseType;

            // Check if an empty value or an empty string
            if (value == null || value.ToString() == String.Empty)
                return newValue;

            if (fieldType.Equals(value.GetType()))
            {
                newValue = value;
            }
            else if (typeName == "Boolean")
            {
                newValue = (value.ToString() == "1" ||
                            value.ToString().ToLower() == "on" ||
                            value.ToString().ToLower() == "true" ||
                            value.ToString().ToLower() == "yes") ? true : false;
            }
            // Nullable types's name starts with nullable
            else if (typeName.StartsWith("Nullable"))
            {
                var typeFullName = fieldType.FullName;
                if (typeFullName.Contains("DateTime"))
                    newValue = Convert.ToDateTime(value);
                else if (typeFullName.Contains("Boolean"))
                    newValue = Convert.ToBoolean(value);
                else if (typeFullName.Contains("Int16"))
                    newValue = Convert.ToInt16(value);
                else if (typeFullName.Contains("Int32"))
                    newValue = Convert.ToInt32(value);
                else if (typeFullName.Contains("Integer"))
                    newValue = Convert.ToInt32(value);
                else if (fieldType.FullName.Contains("Int64"))
                    newValue = Convert.ToInt64(value);
                else if (fieldType.FullName.Contains("Decimal"))
                    newValue = Convert.ToDecimal(value);
                else if (typeFullName.Contains("Double"))
                    newValue = Convert.ToDouble(value);
                else if (typeFullName.Contains("Single"))
                    newValue = Convert.ToSingle(value);
                else if (typeFullName.Contains("UInt16"))
                    newValue = Convert.ToUInt16(value);
                else if (typeFullName.Contains("UInt32"))
                    newValue = Convert.ToUInt32(value);
                else if (typeFullName.Contains("UInt64"))
                    newValue = Convert.ToUInt64(value);
                else if (typeFullName.Contains("SByte"))
                    newValue = Convert.ToSByte(value);
            }
            else if (fieldType.FullName.Equals("System.Guid"))
            {
                newValue = new Guid(value.ToString());
            }
            else if (baseType != null && fieldType.BaseType == typeof(Enum))
            {
                int intEnum;
                if (int.TryParse(value.ToString(), out intEnum))
                    newValue = intEnum;
                else
                {
                    try
                    {
                        newValue = Enum.Parse(fieldType, value.ToString());
                    }
                    catch (Exception)
                    {
                        newValue = Enum.ToObject(fieldType, value);

                    }
                }
            }
            else if (fieldType.FullName.Equals("System.Guid"))
            {
                newValue = new Guid(value.ToString());
            }
            else
            {
                // Try to get a specific type converter
                //      when no type converter is foudn then do a brute convert and ignore any errors that come up                
                var converter = GetTypeConverter(fieldType.Name);
                if (converter != null)
                {
                    newValue = converter.GetValue(fieldType, value);
                }
                else
                {
                    try
                    {
                        newValue = Convert.ChangeType(value, fieldType);
                    }
                    catch (Exception) { }
                }
            }


            return newValue;

        }

        #endregion


        #region "FillObject / FillCollection | DataReader"

        /// <summary>
        /// Fills an object using generics from a data reader object
        /// </summary>
        /// <typeparam name="T">The type of the object to fill</typeparam>
        /// <param name="dr">The data reader object used to hydrate the object</param>
        /// <returns>A hydrated object of type T</returns>
        public static T FillObject<T>(DbDataReader dr)
        {
            return FillObject<T>(dr, true);
        }

        /// <summary>
        /// Fills a collection of objects using generics from a data reader object
        /// </summary>
        /// <typeparam name="T">The Type of the collection item object to fill</typeparam>
        /// <param name="reader">The reader object used to hydrate the collection</param>
        /// <returns>Collection of type of type T</returns>
        public static List<T> FillCollection<T>(DbDataReader reader)
        {
            var objCollection = new List<T>();

            try
            {
                while (reader.Read())
                {
                    objCollection.Add(CreateObject<T>(reader));
                }
            }
            finally
            {
                reader.Close();
            }

            return objCollection;
        }
        
        /// <summary>
        /// Fill a particular object
        /// </summary>
        /// <typeparam name="T">The type of object to fill</typeparam>
        /// <param name="dr">The reader object used to hydrate the object</param>
        /// <param name="manageDataReader">When set to true, closes the reader when finished</param>
        /// <returns>A hydrated object of type T</returns>
        private static T FillObject<T>(DbDataReader dr, bool manageDataReader)
        {
            var objFillObject = default(T);
            // Make sure the data reader has data
            if (manageDataReader && dr.Read() == false) return objFillObject;

            // Fill the object            
            objFillObject = CreateObject<T>(dr);

            // Close the reader when in charge
            if (manageDataReader) dr.Close();

            // Return the filled object
            return objFillObject;
        }

        private static T CreateObject<T>(DbDataReader dr)
        {
            // Create a new instance of the object
            var objObject = Activator.CreateInstance<T>();

            // Check weather the object is a value type
            if (objObject.GetType().IsValueType)
            {
                var value = GetValue(objObject.GetType(), dr.GetValue(0));
                if (value != null) objObject = (T)value;
                return objObject;
            }

            // Hydrate a complex type it
            //  Get the fields for the type
            List<object> fields = GetFields(objObject);

            //  Get the ordinals in the data reader
            int[] arrOrdinals = GetOrdinals(fields, dr);

            for (int i = 0; i < fields.Count; i++)
            {
                var field = fields[i];

                //   Make sure the value is found
                if (arrOrdinals[i] != -1)
                {
                    //  Get the value from the reader
                    var value = dr.GetValue(arrOrdinals[i]);

                    //  Make sure the value is not null before assigning
                    if (value == DBNull.Value) continue;

                    // Check weather the member is a property or a field and fill it accordingly
                    if (field is FieldInfo)
                        (field as FieldInfo).SetValue(objObject, GetValue(((FieldInfo)field).FieldType, value), BindingFlags.Default, null, null);
                    else if (field is PropertyInfo)
                        (field as PropertyInfo).SetValue(objObject, GetValue(((PropertyInfo)field).PropertyType, value), BindingFlags.Default, null, null, null);
                }
            }

            return objObject;
        }

        /// <summary>
        /// Match the bject fields against the ordinal of the DataReader. The ordinal of the DataReader is used 
        /// Rather than the string indexer to maximize performance
        /// </summary>
        /// <param name="fields">The list of members to hydrate for the object</param>
        /// <param name="dr">The data reader to map the ordinals from</param>
        /// <returns></returns>
        private static int[] GetOrdinals(List<object> fields, DbDataReader dr)
        {
            var arrOrdinals = new int[fields.Count];
            var columns = new Hashtable();

            // Get the column names
            for (var ci = 0; ci < dr.FieldCount; ci++)
                columns[dr.GetName(ci).ToUpperInvariant()] = "";

            // Match the fields to the column name ordinal
            for (var fi = 0; fi < fields.Count; fi++)
            {
                string fieldName = (fields[fi] is FieldInfo)
                                            ? ((FieldInfo)fields[fi]).Name.ToUpperInvariant()
                                            : ((PropertyInfo)fields[fi]).Name.ToUpperInvariant();

                if (columns.ContainsKey(fieldName))
                    arrOrdinals[fi] = dr.GetOrdinal(fieldName);
                else
                    arrOrdinals[fi] = -1;
            }

            return arrOrdinals;

        }

        /// <summary>
        /// Get the fields / properties to hydrate for an object. 
        /// Caches the object map in order to maximize performance. 
        /// So reflection is used only first time on n object type
        /// </summary>
        /// <param name="obj">Object to use to hydrate</param>
        /// <returns>A list of Fields and Properties</returns>
        private static List<object> GetFields(object obj)
        {
            var cacheKey = "reflectioncache_" + obj.GetType().FullName;
            List<object> fields;
            if (Cache.Contains(cacheKey))
                fields = Cache.GetValue(cacheKey) as List<object>;
            else
            {
                fields = new List<object>();
                foreach (PropertyInfo p in obj.GetType().GetProperties())
                {
                    if (p.CanWrite) fields.Add(p);
                }
                foreach (FieldInfo f in obj.GetType().GetFields())
                {
                    if (f.IsPublic && !f.IsStatic) fields.Add(f);
                }

                fields.AddRange(obj.GetType().GetFields());

                Cache.Add(cacheKey, fields);
            }
            return fields;
        }

        #endregion


        #region "FillObject / FillCollection : NameValueCollection"

        public static T FillObject<T>(NameValueCollection values)
        {
            return CreateObject<T>(values, "", "");
        }

        public static T FillObject<T>(NameValueCollection values, string prefix)
        {
            return CreateObject<T>(values, prefix, "");
        }

        public static T FillObject<T>(NameValueCollection values, string prefix, string suffix)
        {
            return CreateObject<T>(values, prefix, suffix);
        }

        public static object FillObject(object objectToFill, NameValueCollection values)
        {
            return CreateObject(objectToFill, values, "", "");
        }

        public static object FillObject(object objectToFill, NameValueCollection values, string prefix)
        {
            return CreateObject(objectToFill, values, prefix, "");
        }

        public static object FillObject(object objectToFill, NameValueCollection values, string prefix, string suffix)
        {
            return CreateObject(objectToFill, values, prefix, suffix);
        }

        private static T CreateObject<T>(NameValueCollection values, string prefix, string suffix)
        {
            // Create a new instance of the object
            var objObject = Activator.CreateInstance<T>();
            // fill it
            return (T)CreateObject(objObject, values, prefix, suffix);
        }

        private static object CreateObject(object objectToFill, NameValueCollection values, string prefix, string suffix)
        {
            // Make sure there are values to hydrate
            if (values == null || values.HasKeys() == false) return objectToFill;

            // Check weather the object is a value type
            if (objectToFill.GetType().IsValueType)
            {
                var value = GetValue(objectToFill.GetType(), values[0]);
                if (value != null) objectToFill = value;
                return objectToFill;
            }

            // Hydrate a complex type it
            //  Get the fields for the type
            List<object> fields = GetFields(objectToFill);
            foreach (var field in fields)
            {
                // Get the fieldname
                string fieldName;
                if (field is FieldInfo)
                    fieldName = (field as FieldInfo).Name.ToUpperInvariant();
                else
                    fieldName = (field as PropertyInfo).Name.ToUpperInvariant();

                // Loop through the values
                foreach (string formKey in values.Keys)
                {
                    if ((prefix + fieldName + suffix) == formKey.ToUpperInvariant())
                    {
                        // Check weather the member is a property or a field and fill it accordingly
                        if (field is FieldInfo)
                            (field as FieldInfo).SetValue(objectToFill, GetValue(((FieldInfo)field).FieldType, values[formKey]), BindingFlags.Default, null, null);
                        else if (field is PropertyInfo)
                            (field as PropertyInfo).SetValue(objectToFill, GetValue(((PropertyInfo)field).PropertyType, values[formKey]), BindingFlags.Default, null, null, null);

                        // Go to the next item
                        break;
                    }

                }

            }

            // Return the filled object
            return objectToFill;
        }

        #endregion

        #endregion

    }





    #region "DAAB"

    // ===============================================================================
    // Microsoft Data Access Application Block for .NET
    // http://msdn.microsoft.com/library/en-us/dnbda/html/daab-rm.asp
    //
    // SQLHelper.cs
    //
    // This file contains the implementations of the DAAB and SqlHelperParameterCache
    // classes.
    //
    // For more information see the Data Access Application Block Implementation Overview. 
    // ===============================================================================
    // Release history
    // VERSION	DESCRIPTION
    //   2.0	Added support for FillDataset, UpdateDataset and "Param" helper methods
    //   2.1    Mitch Labrador: Added support for using the DbProviderFactory abstracted all the
    //          SqlClient specific references to use the corresponding DbProviderFactory classes
    //          by defautl it uses the SqlClient provider factory. But can be changed by assigning
    //          the provider name.
    //
    // ===============================================================================
    // Copyright (C) 2000-2001 Microsoft Corporation
    // All rights reserved.
    // THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
    // OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
    // LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR
    // FITNESS FOR A PARTICULAR PURPOSE.
    // ==============================================================================


    /// <summary>
    /// The DAAB class is intended to encapsulate high performance, scalable best practices for 
    /// common uses of SqlClient
    /// </summary>
    public static class DAAB
    {

        /// <summary>
        /// Holds the provider name for the current DbFactory Provider
        /// </summary>
        public static string ProviderName = "System.Data.SqlClient";

        private static readonly Dictionary<string, MethodInfo> _cache = new Dictionary<string, MethodInfo>();

        #region private utility methods & constructors

        // Since this class provides only static methods, make the default constructor private to prevent 
        // instances from being created with "new DAAB()"

        /// <summary>
        /// This method is used to attach array of DbParameters to a DbCommand.
        /// 
        /// This method will assign a value of DbNull to any parameter with a direction of
        /// InputOutput and a value of null.  
        /// 
        /// This behavior will prevent default values from being used, but
        /// this will be the less common case than an intended pure output parameter (derived as InputOutput)
        /// where the user provided no input value.
        /// </summary>
        /// <param name="command">The command to which the parameters will be added</param>
        /// <param name="commandParameters">An array of DbParameters to be added to command</param>
        private static void AttachParameters(DbCommand command, IEnumerable<DbParameter> commandParameters)
        {
            if (command == null) throw new ArgumentNullException("command");
            if (commandParameters != null)
            {
                foreach (var p in commandParameters)
                {
                    if (p != null)
                    {
                        // Check for derived output value with no value assigned
                        if ((p.Direction == ParameterDirection.InputOutput ||
                            p.Direction == ParameterDirection.Input) &&
                            (p.Value == null))
                        {
                            p.Value = DBNull.Value;
                        }
                        command.Parameters.Add(p);
                    }
                }
            }
        }

        /// <summary>
        /// This method assigns dataRow column values to an array of DbParameters
        /// </summary>
        /// <param name="commandParameters">Array of DbParameters to be assigned values</param>
        /// <param name="dataRow">The dataRow used to hold the stored procedure's parameter values</param>
        private static void AssignParameterValues(IEnumerable<DbParameter> commandParameters, DataRow dataRow)
        {
            if ((commandParameters == null) || (dataRow == null))
            {
                // Do nothing if we get no data
                return;
            }

            int i = 0;
            // Set the parameters values
            foreach (var commandParameter in commandParameters)
            {
                // Check the parameter name
                if (commandParameter.ParameterName == null ||
                    commandParameter.ParameterName.Length <= 1)
                    throw new Exception(
                        string.Format(
                            "Please provide a valid parameter name on the parameter #{0}, the ParameterName property has the following value: '{1}'.",
                            i, commandParameter.ParameterName));
                if (dataRow.Table.Columns.IndexOf(commandParameter.ParameterName.Substring(1)) != -1)
                    commandParameter.Value = dataRow[commandParameter.ParameterName.Substring(1)];
                i++;
            }
        }

        /// <summary>
        /// This method assigns an array of values to an array of DbParameters
        /// </summary>
        /// <param name="commandParameters">Array of DbParameters to be assigned values</param>
        /// <param name="parameterValues">Array of objects holding the values to be assigned</param>
        private static void AssignParameterValues(DbParameter[] commandParameters, object[] parameterValues)
        {
            if ((commandParameters == null) || (parameterValues == null))
            {
                // Do nothing if we get no data
                return;
            }

            // We must have the same number of values as we pave parameters to put them in
            if (commandParameters.Length != parameterValues.Length)
            {
                throw new ArgumentException("Parameter count does not match Parameter Value count.");
            }

            // Iterate through the DbParameters, assigning the values from the corresponding position in the 
            // value array
            for (int i = 0, j = commandParameters.Length; i < j; i++)
            {
                // If the current array value derives from DbParameter, then assign its Value property
                if (parameterValues[i] is DbParameter)
                {
                    var paramInstance = (DbParameter)parameterValues[i];
                    commandParameters[i].Value = paramInstance.Value ?? DBNull.Value;
                }
                else if (parameterValues[i] == null)
                {
                    commandParameters[i].Value = DBNull.Value;
                }
                else
                {
                    commandParameters[i].Value = parameterValues[i];
                }
            }
        }

        /// <summary>
        /// This method opens (if necessary) and assigns a connection, transaction, command type and parameters 
        /// to the provided command
        /// </summary>
        /// <param name="command">The DbCommand to be prepared</param>
        /// <param name="connection">A valid DbConnection, on which to execute this command</param>
        /// <param name="transaction">A valid SqlTransaction, or 'null'</param>
        /// <param name="commandType">The CommandType (stored procedure, text, etc.)</param>
        /// <param name="commandText">The stored procedure name or T-SQL command</param>
        /// <param name="commandParameters">An array of DbParameters to be associated with the command or 'null' if no parameters are required</param>
        /// <param name="mustCloseConnection"><c>true</c> if the connection was opened by the method, otherwose is false.</param>
        private static void PrepareCommand(DbCommand command, DbConnection connection, DbTransaction transaction, CommandType commandType, string commandText, IEnumerable<DbParameter> commandParameters, out bool mustCloseConnection)
        {
            if (command == null) throw new ArgumentNullException("command");
            if (string.IsNullOrEmpty(commandText)) throw new ArgumentNullException("commandText");

            // If the provided connection is not open, we will open it
            if (connection.State != ConnectionState.Open)
            {
                mustCloseConnection = true;
                connection.Open();
            }
            else
            {
                mustCloseConnection = false;
            }

            // Associate the connection with the command
            command.Connection = connection;

            // Set the command text (stored procedure name or SQL statement)
            command.CommandText = commandText;

            // If we were provided a transaction, assign it
            if (transaction != null)
            {
                if (transaction.Connection == null) throw new ArgumentException("The transaction was rollbacked or commited, please provide an open transaction.", "transaction");
                command.Transaction = transaction;
            }

            // Set the command type
            command.CommandType = commandType;

            // Attach the command parameters if they are provided
            if (commandParameters != null)
            {
                AttachParameters(command, commandParameters);
            }
            return;
        }
        
        #endregion private utility methods & constructors


        #region "ObjectFactory"

        public static DbProviderFactory ProviderFactory()
        {
            return DbProviderFactories.GetFactory(ProviderName);
        }

        public static DbConnection CreateConnection(string connectionString)
        {
            var conn = ProviderFactory().CreateConnection();
            conn.ConnectionString = connectionString;
            return conn;
        }

        public static DbCommand CreateCommand()
        {
            return ProviderFactory().CreateCommand();
        }

        public static DbParameter CreateParameter()
        {
            return ProviderFactory().CreateParameter();
        }

        public static DbParameter[] CreateParameterCollection(int count)
        {
            var factory = ProviderFactory();
            var dbparams = new List<DbParameter>();
            for (var i = 0; i < count; i++)
            {
                dbparams.Add(factory.CreateParameter());
            }
            return dbparams.ToArray();
        }

        public static DataAdapter CreateDataAdapter(DbCommand cmd)
        {
            var da = ProviderFactory().CreateDataAdapter();
            da.SelectCommand=cmd;
            return da;
        }

        public static void DeriveParameters(DbCommand cmd)
        {
            // [Snip] code to and assign values to command
            var commandBuilderType = ProviderFactory().CreateCommandBuilder().GetType();

            var cacheKey = ProviderName + ".DeriveParametersMethodInfo";
            MethodInfo cachedMethodInfo;

            // Get the method from the cache or use reflection and add it
            if (_cache.ContainsKey(cacheKey))
            {
                cachedMethodInfo = _cache[cacheKey];
            }
            else
            {
                cachedMethodInfo = commandBuilderType.GetMethod("DeriveParameters");
                _cache.Add(cacheKey, cachedMethodInfo);
            }

            // Only retreive the commands if this provider supports the DeriveParameters method
            if (cachedMethodInfo != null)
            {
                cachedMethodInfo.Invoke(null, new object[] { cmd });
            }
        }
        
        #endregion


        #region ExecuteNonQuery

        /// <summary>
        /// Execute a DbCommand (that returns no resultset and takes no parameters) against the database specified in 
        /// the connection string
        /// </summary>
        /// <remarks>
        /// e.g.:  
        ///  int result = ExecuteNonQuery(connString, CommandType.StoredProcedure, "PublishOrders");
        /// </remarks>
        /// <param name="connectionString">A valid connection string for a DbConnection</param>
        /// <param name="commandType">The CommandType (stored procedure, text, etc.)</param>
        /// <param name="commandText">The stored procedure name or T-SQL command</param>
        /// <returns>An int representing the number of rows affected by the command</returns>
        public static int ExecuteNonQuery(string connectionString, CommandType commandType, string commandText)
        {
            // Pass through the call providing null for the set of DbParameters
            return ExecuteNonQuery(connectionString, commandType, commandText, null);
        }

        /// <summary>
        /// Execute a DbCommand (that returns no resultset) against the database specified in the connection string 
        /// using the provided parameters
        /// </summary>
        /// <remarks>
        /// e.g.:  
        ///  int result = ExecuteNonQuery(connString, CommandType.StoredProcedure, "PublishOrders", new DbParameter("@prodid", 24));
        /// </remarks>
        /// <param name="connectionString">A valid connection string for a DbConnection</param>
        /// <param name="commandType">The CommandType (stored procedure, text, etc.)</param>
        /// <param name="commandText">The stored procedure name or T-SQL command</param>
        /// <param name="commandParameters">An array of SqlParamters used to execute the command</param>
        /// <returns>An int representing the number of rows affected by the command</returns>
        public static int ExecuteNonQuery(string connectionString, CommandType commandType, string commandText, params DbParameter[] commandParameters)
        {
            if (string.IsNullOrEmpty(connectionString)) throw new ArgumentNullException("connectionString");

            // Create & open a DbConnection, and dispose of it after we are done
            using (var connection = CreateConnection(connectionString))
            {
                connection.Open();

                // Call the overload that takes a connection in place of the connection string
                return ExecuteNonQuery(connection, commandType, commandText, commandParameters);
            }
        }

        /// <summary>
        /// Execute a stored procedure via a DbCommand (that returns no resultset) against the database specified in 
        /// the connection string using the provided parameter values.  This method will query the database to discover the parameters for the 
        /// stored procedure (the first time each stored procedure is called), and assign the values based on parameter order.
        /// </summary>
        /// <remarks>
        /// This method provides no access to output parameters or the stored procedure's return value parameter.
        /// 
        /// e.g.:  
        ///  int result = ExecuteNonQuery(connString, "PublishOrders", 24, 36);
        /// </remarks>
        /// <param name="connectionString">A valid connection string for a DbConnection</param>
        /// <param name="spName">The name of the stored prcedure</param>
        /// <param name="parameterValues">An array of objects to be assigned as the input values of the stored procedure</param>
        /// <returns>An int representing the number of rows affected by the command</returns>
        public static int ExecuteNonQuery(string connectionString, string spName, params object[] parameterValues)
        {
            if (string.IsNullOrEmpty(connectionString)) throw new ArgumentNullException("connectionString");
            if (string.IsNullOrEmpty(spName)) throw new ArgumentNullException("spName");

            // If we receive parameter values, we need to figure out where they go
            if ((parameterValues != null) && (parameterValues.Length > 0))
            {
                // Pull the parameters for this stored procedure from the parameter cache (or discover them & populate the cache)
                var commandParameters = SqlHelperParameterCache.GetSpParameterSet(connectionString, spName);

                // Assign the provided values to these parameters based on parameter order
                AssignParameterValues(commandParameters, parameterValues);

                // Call the overload that takes an array of DbParameters
                return ExecuteNonQuery(connectionString, CommandType.StoredProcedure, spName, commandParameters);
            }
            else
            {
                // Otherwise we can just call the SP without params
                return ExecuteNonQuery(connectionString, CommandType.StoredProcedure, spName);
            }
        }

        /// <summary>
        /// Execute a DbCommand (that returns no resultset and takes no parameters) against the provided DbConnection. 
        /// </summary>
        /// <remarks>
        /// e.g.:  
        ///  int result = ExecuteNonQuery(conn, CommandType.StoredProcedure, "PublishOrders");
        /// </remarks>
        /// <param name="connection">A valid DbConnection</param>
        /// <param name="commandType">The CommandType (stored procedure, text, etc.)</param>
        /// <param name="commandText">The stored procedure name or T-SQL command</param>
        /// <returns>An int representing the number of rows affected by the command</returns>
        public static int ExecuteNonQuery(DbConnection connection, CommandType commandType, string commandText)
        {
            // Pass through the call providing null for the set of DbParameters
            return ExecuteNonQuery(connection, commandType, commandText, null);
        }

        /// <summary>
        /// Execute a DbCommand (that returns no resultset) against the specified DbConnection 
        /// using the provided parameters.
        /// </summary>
        /// <remarks>
        /// e.g.:  
        ///  int result = ExecuteNonQuery(conn, CommandType.StoredProcedure, "PublishOrders", new DbParameter("@prodid", 24));
        /// </remarks>
        /// <param name="connection">A valid DbConnection</param>
        /// <param name="commandType">The CommandType (stored procedure, text, etc.)</param>
        /// <param name="commandText">The stored procedure name or T-SQL command</param>
        /// <param name="commandParameters">An array of SqlParamters used to execute the command</param>
        /// <returns>An int representing the number of rows affected by the command</returns>
        public static int ExecuteNonQuery(DbConnection connection, CommandType commandType, string commandText, params DbParameter[] commandParameters)
        {
            if (connection == null) throw new ArgumentNullException("connection");

            // Create a command and prepare it for execution
            var cmd = CreateCommand();
            var mustCloseConnection = false;
            PrepareCommand(cmd, connection, null, commandType, commandText, commandParameters, out mustCloseConnection);

            // Finally, execute the command
            var retval = cmd.ExecuteNonQuery();

            // Detach the DbParameters from the command object, so they can be used again
            cmd.Parameters.Clear();
            if (mustCloseConnection)
                connection.Close();
            return retval;
        }

        /// <summary>
        /// Execute a stored procedure via a DbCommand (that returns no resultset) against the specified DbConnection 
        /// using the provided parameter values.  This method will query the database to discover the parameters for the 
        /// stored procedure (the first time each stored procedure is called), and assign the values based on parameter order.
        /// </summary>
        /// <remarks>
        /// This method provides no access to output parameters or the stored procedure's return value parameter.
        /// 
        /// e.g.:  
        ///  int result = ExecuteNonQuery(conn, "PublishOrders", 24, 36);
        /// </remarks>
        /// <param name="connection">A valid DbConnection</param>
        /// <param name="spName">The name of the stored procedure</param>
        /// <param name="parameterValues">An array of objects to be assigned as the input values of the stored procedure</param>
        /// <returns>An int representing the number of rows affected by the command</returns>
        public static int ExecuteNonQuery(DbConnection connection, string spName, params object[] parameterValues)
        {
            if (connection == null) throw new ArgumentNullException("connection");
            if (string.IsNullOrEmpty(spName)) throw new ArgumentNullException("spName");

            // If we receive parameter values, we need to figure out where they go
            if ((parameterValues != null) && (parameterValues.Length > 0))
            {
                // Pull the parameters for this stored procedure from the parameter cache (or discover them & populate the cache)
                var commandParameters = SqlHelperParameterCache.GetSpParameterSet(connection, spName);

                // Assign the provided values to these parameters based on parameter order
                AssignParameterValues(commandParameters, parameterValues);

                // Call the overload that takes an array of DbParameters
                return ExecuteNonQuery(connection, CommandType.StoredProcedure, spName, commandParameters);
            }
            else
            {
                // Otherwise we can just call the SP without params
                return ExecuteNonQuery(connection, CommandType.StoredProcedure, spName);
            }
        }

        /// <summary>
        /// Execute a DbCommand (that returns no resultset and takes no parameters) against the provided SqlTransaction. 
        /// </summary>
        /// <remarks>
        /// e.g.:  
        ///  int result = ExecuteNonQuery(trans, CommandType.StoredProcedure, "PublishOrders");
        /// </remarks>
        /// <param name="transaction">A valid SqlTransaction</param>
        /// <param name="commandType">The CommandType (stored procedure, text, etc.)</param>
        /// <param name="commandText">The stored procedure name or T-SQL command</param>
        /// <returns>An int representing the number of rows affected by the command</returns>
        public static int ExecuteNonQuery(SqlTransaction transaction, CommandType commandType, string commandText)
        {
            // Pass through the call providing null for the set of DbParameters
            return ExecuteNonQuery(transaction, commandType, commandText, null);
        }

        /// <summary>
        /// Execute a DbCommand (that returns no resultset) against the specified SqlTransaction
        /// using the provided parameters.
        /// </summary>
        /// <remarks>
        /// e.g.:  
        ///  int result = ExecuteNonQuery(trans, CommandType.StoredProcedure, "GetOrders", new DbParameter("@prodid", 24));
        /// </remarks>
        /// <param name="transaction">A valid SqlTransaction</param>
        /// <param name="commandType">The CommandType (stored procedure, text, etc.)</param>
        /// <param name="commandText">The stored procedure name or T-SQL command</param>
        /// <param name="commandParameters">An array of SqlParamters used to execute the command</param>
        /// <returns>An int representing the number of rows affected by the command</returns>
        public static int ExecuteNonQuery(DbTransaction transaction, CommandType commandType, string commandText, params DbParameter[] commandParameters)
        {
            if (transaction == null) throw new ArgumentNullException("transaction");
            if (transaction != null && transaction.Connection == null) throw new ArgumentException("The transaction was rollbacked or commited, please provide an open transaction.", "transaction");

            // Create a command and prepare it for execution
            var cmd = CreateCommand();
            var mustCloseConnection = false;
            PrepareCommand(cmd, transaction.Connection, transaction, commandType, commandText, commandParameters, out mustCloseConnection);

            // Finally, execute the command
            var retval = cmd.ExecuteNonQuery();

            // Detach the DbParameters from the command object, so they can be used again
            cmd.Parameters.Clear();
            return retval;
        }

        /// <summary>
        /// Execute a stored procedure via a DbCommand (that returns no resultset) against the specified 
        /// SqlTransaction using the provided parameter values.  This method will query the database to discover the parameters for the 
        /// stored procedure (the first time each stored procedure is called), and assign the values based on parameter order.
        /// </summary>
        /// <remarks>
        /// This method provides no access to output parameters or the stored procedure's return value parameter.
        /// 
        /// e.g.:  
        ///  int result = ExecuteNonQuery(conn, trans, "PublishOrders", 24, 36);
        /// </remarks>
        /// <param name="transaction">A valid SqlTransaction</param>
        /// <param name="spName">The name of the stored procedure</param>
        /// <param name="parameterValues">An array of objects to be assigned as the input values of the stored procedure</param>
        /// <returns>An int representing the number of rows affected by the command</returns>
        public static int ExecuteNonQuery(DbTransaction transaction, string spName, params object[] parameterValues)
        {
            if (transaction == null) throw new ArgumentNullException("transaction");
            if (transaction != null && transaction.Connection == null) throw new ArgumentException("The transaction was rollbacked or commited, please provide an open transaction.", "transaction");
            if (string.IsNullOrEmpty(spName)) throw new ArgumentNullException("spName");

            // If we receive parameter values, we need to figure out where they go
            if ((parameterValues != null) && (parameterValues.Length > 0))
            {
                // Pull the parameters for this stored procedure from the parameter cache (or discover them & populate the cache)
                var commandParameters = SqlHelperParameterCache.GetSpParameterSet(transaction.Connection, spName);

                // Assign the provided values to these parameters based on parameter order
                AssignParameterValues(commandParameters, parameterValues);

                // Call the overload that takes an array of DbParameters
                return ExecuteNonQuery(transaction, CommandType.StoredProcedure, spName, commandParameters);
            }
            else
            {
                // Otherwise we can just call the SP without params
                return ExecuteNonQuery(transaction, CommandType.StoredProcedure, spName);
            }
        }

        #endregion ExecuteNonQuery

        #region ExecuteDataset

        /// <summary>
        /// Execute a DbCommand (that returns a resultset and takes no parameters) against the database specified in 
        /// the connection string. 
        /// </summary>
        /// <remarks>
        /// e.g.:  
        ///  DataSet ds = ExecuteDataset(connString, CommandType.StoredProcedure, "GetOrders");
        /// </remarks>
        /// <param name="connectionString">A valid connection string for a DbConnection</param>
        /// <param name="commandType">The CommandType (stored procedure, text, etc.)</param>
        /// <param name="commandText">The stored procedure name or T-SQL command</param>
        /// <returns>A dataset containing the resultset generated by the command</returns>
        public static DataSet ExecuteDataset(string connectionString, CommandType commandType, string commandText)
        {
            // Pass through the call providing null for the set of DbParameters
            return ExecuteDataset(connectionString, commandType, commandText, null);
        }

        /// <summary>
        /// Execute a DbCommand (that returns a resultset) against the database specified in the connection string 
        /// using the provided parameters.
        /// </summary>
        /// <remarks>
        /// e.g.:  
        ///  DataSet ds = ExecuteDataset(connString, CommandType.StoredProcedure, "GetOrders", new DbParameter("@prodid", 24));
        /// </remarks>
        /// <param name="connectionString">A valid connection string for a DbConnection</param>
        /// <param name="commandType">The CommandType (stored procedure, text, etc.)</param>
        /// <param name="commandText">The stored procedure name or T-SQL command</param>
        /// <param name="commandParameters">An array of SqlParamters used to execute the command</param>
        /// <returns>A dataset containing the resultset generated by the command</returns>
        public static DataSet ExecuteDataset(string connectionString, CommandType commandType, string commandText, params DbParameter[] commandParameters)
        {
            if (string.IsNullOrEmpty(connectionString)) throw new ArgumentNullException("connectionString");

            // Create & open a DbConnection, and dispose of it after we are done
            using (var connection = CreateConnection(connectionString))
            {
                connection.Open();

                // Call the overload that takes a connection in place of the connection string
                return ExecuteDataset(connection, commandType, commandText, commandParameters);
            }
        }

        /// <summary>
        /// Execute a stored procedure via a DbCommand (that returns a resultset) against the database specified in 
        /// the connection string using the provided parameter values.  This method will query the database to discover the parameters for the 
        /// stored procedure (the first time each stored procedure is called), and assign the values based on parameter order.
        /// </summary>
        /// <remarks>
        /// This method provides no access to output parameters or the stored procedure's return value parameter.
        /// 
        /// e.g.:  
        ///  DataSet ds = ExecuteDataset(connString, "GetOrders", 24, 36);
        /// </remarks>
        /// <param name="connectionString">A valid connection string for a DbConnection</param>
        /// <param name="spName">The name of the stored procedure</param>
        /// <param name="parameterValues">An array of objects to be assigned as the input values of the stored procedure</param>
        /// <returns>A dataset containing the resultset generated by the command</returns>
        public static DataSet ExecuteDataset(string connectionString, string spName, params object[] parameterValues)
        {
            if (string.IsNullOrEmpty(connectionString)) throw new ArgumentNullException("connectionString");
            if (string.IsNullOrEmpty(spName)) throw new ArgumentNullException("spName");

            // If we receive parameter values, we need to figure out where they go
            if ((parameterValues != null) && (parameterValues.Length > 0))
            {
                // Pull the parameters for this stored procedure from the parameter cache (or discover them & populate the cache)
                var commandParameters = SqlHelperParameterCache.GetSpParameterSet(connectionString, spName);

                // Assign the provided values to these parameters based on parameter order
                AssignParameterValues(commandParameters, parameterValues);

                // Call the overload that takes an array of DbParameters
                return ExecuteDataset(connectionString, CommandType.StoredProcedure, spName, commandParameters);
            }
            else
            {
                // Otherwise we can just call the SP without params
                return ExecuteDataset(connectionString, CommandType.StoredProcedure, spName);
            }
        }

        /// <summary>
        /// Execute a DbCommand (that returns a resultset and takes no parameters) against the provided DbConnection. 
        /// </summary>
        /// <remarks>
        /// e.g.:  
        ///  DataSet ds = ExecuteDataset(conn, CommandType.StoredProcedure, "GetOrders");
        /// </remarks>
        /// <param name="connection">A valid DbConnection</param>
        /// <param name="commandType">The CommandType (stored procedure, text, etc.)</param>
        /// <param name="commandText">The stored procedure name or T-SQL command</param>
        /// <returns>A dataset containing the resultset generated by the command</returns>
        public static DataSet ExecuteDataset(DbConnection connection, CommandType commandType, string commandText)
        {
            // Pass through the call providing null for the set of DbParameters
            return ExecuteDataset(connection, commandType, commandText, null);
        }

        /// <summary>
        /// Execute a DbCommand (that returns a resultset) against the specified DbConnection 
        /// using the provided parameters.
        /// </summary>
        /// <remarks>
        /// e.g.:  
        ///  DataSet ds = ExecuteDataset(conn, CommandType.StoredProcedure, "GetOrders", new DbParameter("@prodid", 24));
        /// </remarks>
        /// <param name="connection">A valid DbConnection</param>
        /// <param name="commandType">The CommandType (stored procedure, text, etc.)</param>
        /// <param name="commandText">The stored procedure name or T-SQL command</param>
        /// <param name="commandParameters">An array of SqlParamters used to execute the command</param>
        /// <returns>A dataset containing the resultset generated by the command</returns>
        public static DataSet ExecuteDataset(DbConnection connection, CommandType commandType, string commandText, params DbParameter[] commandParameters)
        {
            if (connection == null) throw new ArgumentNullException("connection");

            // Create a command and prepare it for execution
            var cmd = CreateCommand();
            var mustCloseConnection = false;
            PrepareCommand(cmd, connection, null, commandType, commandText, commandParameters, out mustCloseConnection);

            // Create the DataAdapter & DataSet
            var da = CreateDataAdapter(cmd);
            try
            {
                var ds = new DataSet();

                // Fill the DataSet using default values for DataTable names, etc
                da.Fill(ds);

                // Detach the DbParameters from the command object, so they can be used again
                cmd.Parameters.Clear();

                if (mustCloseConnection)
                    connection.Close();

                // Return the dataset
                return ds;
            }
            finally 
            {
                cmd.Dispose();
            }

        }

        /// <summary>
        /// Execute a stored procedure via a DbCommand (that returns a resultset) against the specified DbConnection 
        /// using the provided parameter values.  This method will query the database to discover the parameters for the 
        /// stored procedure (the first time each stored procedure is called), and assign the values based on parameter order.
        /// </summary>
        /// <remarks>
        /// This method provides no access to output parameters or the stored procedure's return value parameter.
        /// 
        /// e.g.:  
        ///  DataSet ds = ExecuteDataset(conn, "GetOrders", 24, 36);
        /// </remarks>
        /// <param name="connection">A valid DbConnection</param>
        /// <param name="spName">The name of the stored procedure</param>
        /// <param name="parameterValues">An array of objects to be assigned as the input values of the stored procedure</param>
        /// <returns>A dataset containing the resultset generated by the command</returns>
        public static DataSet ExecuteDataset(DbConnection connection, string spName, params object[] parameterValues)
        {
            if (connection == null) throw new ArgumentNullException("connection");
            if (string.IsNullOrEmpty(spName)) throw new ArgumentNullException("spName");

            // If we receive parameter values, we need to figure out where they go
            if ((parameterValues != null) && (parameterValues.Length > 0))
            {
                // Pull the parameters for this stored procedure from the parameter cache (or discover them & populate the cache)
                var commandParameters = SqlHelperParameterCache.GetSpParameterSet(connection, spName);

                // Assign the provided values to these parameters based on parameter order
                AssignParameterValues(commandParameters, parameterValues);

                // Call the overload that takes an array of DbParameters
                return ExecuteDataset(connection, CommandType.StoredProcedure, spName, commandParameters);
            }
            else
            {
                // Otherwise we can just call the SP without params
                return ExecuteDataset(connection, CommandType.StoredProcedure, spName);
            }
        }

        /// <summary>
        /// Execute a DbCommand (that returns a resultset and takes no parameters) against the provided SqlTransaction. 
        /// </summary>
        /// <remarks>
        /// e.g.:  
        ///  DataSet ds = ExecuteDataset(trans, CommandType.StoredProcedure, "GetOrders");
        /// </remarks>
        /// <param name="transaction">A valid SqlTransaction</param>
        /// <param name="commandType">The CommandType (stored procedure, text, etc.)</param>
        /// <param name="commandText">The stored procedure name or T-SQL command</param>
        /// <returns>A dataset containing the resultset generated by the command</returns>
        public static DataSet ExecuteDataset(DbTransaction transaction, CommandType commandType, string commandText)
        {
            // Pass through the call providing null for the set of DbParameters
            return ExecuteDataset(transaction, commandType, commandText, null);
        }

        /// <summary>
        /// Execute a DbCommand (that returns a resultset) against the specified SqlTransaction
        /// using the provided parameters.
        /// </summary>
        /// <remarks>
        /// e.g.:  
        ///  DataSet ds = ExecuteDataset(trans, CommandType.StoredProcedure, "GetOrders", new DbParameter("@prodid", 24));
        /// </remarks>
        /// <param name="transaction">A valid SqlTransaction</param>
        /// <param name="commandType">The CommandType (stored procedure, text, etc.)</param>
        /// <param name="commandText">The stored procedure name or T-SQL command</param>
        /// <param name="commandParameters">An array of SqlParamters used to execute the command</param>
        /// <returns>A dataset containing the resultset generated by the command</returns>
        public static DataSet ExecuteDataset(DbTransaction transaction, CommandType commandType, string commandText, params DbParameter[] commandParameters)
        {
            if (transaction == null) throw new ArgumentNullException("transaction");
            if (transaction != null && transaction.Connection == null) throw new ArgumentException("The transaction was rollbacked or commited, please provide an open transaction.", "transaction");

            // Create a command and prepare it for execution
            var cmd = CreateCommand();
            var mustCloseConnection = false;
            PrepareCommand(cmd, transaction.Connection, transaction, commandType, commandText, commandParameters, out mustCloseConnection);

            // Create the DataAdapter & DataSet
            var da = CreateDataAdapter(cmd);
            try
            {
                var ds = new DataSet();

                // Fill the DataSet using default values for DataTable names, etc
                da.Fill(ds);

                // Detach the DbParameters from the command object, so they can be used again
                cmd.Parameters.Clear();

                // Return the dataset
                return ds;
            }
            finally
            {
                cmd.Dispose();
            }
        }

        /// <summary>
        /// Execute a stored procedure via a DbCommand (that returns a resultset) against the specified 
        /// SqlTransaction using the provided parameter values.  This method will query the database to discover the parameters for the 
        /// stored procedure (the first time each stored procedure is called), and assign the values based on parameter order.
        /// </summary>
        /// <remarks>
        /// This method provides no access to output parameters or the stored procedure's return value parameter.
        /// 
        /// e.g.:  
        ///  DataSet ds = ExecuteDataset(trans, "GetOrders", 24, 36);
        /// </remarks>
        /// <param name="transaction">A valid SqlTransaction</param>
        /// <param name="spName">The name of the stored procedure</param>
        /// <param name="parameterValues">An array of objects to be assigned as the input values of the stored procedure</param>
        /// <returns>A dataset containing the resultset generated by the command</returns>
        public static DataSet ExecuteDataset(DbTransaction transaction, string spName, params object[] parameterValues)
        {
            if (transaction == null) throw new ArgumentNullException("transaction");
            if (transaction != null && transaction.Connection == null) throw new ArgumentException("The transaction was rollbacked or commited, please provide an open transaction.", "transaction");
            if (string.IsNullOrEmpty(spName)) throw new ArgumentNullException("spName");

            // If we receive parameter values, we need to figure out where they go
            if ((parameterValues != null) && (parameterValues.Length > 0))
            {
                // Pull the parameters for this stored procedure from the parameter cache (or discover them & populate the cache)
                var commandParameters = SqlHelperParameterCache.GetSpParameterSet(transaction.Connection, spName);

                // Assign the provided values to these parameters based on parameter order
                AssignParameterValues(commandParameters, parameterValues);

                // Call the overload that takes an array of DbParameters
                return ExecuteDataset(transaction, CommandType.StoredProcedure, spName, commandParameters);
            }
            else
            {
                // Otherwise we can just call the SP without params
                return ExecuteDataset(transaction, CommandType.StoredProcedure, spName);
            }
        }

        #endregion ExecuteDataset

        #region ExecuteReader

        /// <summary>
        /// This enum is used to indicate whether the connection was provided by the caller, or created by DAAB, so that
        /// we can set the appropriate CommandBehavior when calling ExecuteReader()
        /// </summary>
        private enum SqlConnectionOwnership
        {
            /// <summary>Connection is owned and managed by DAAB</summary>
            Internal,
            /// <summary>Connection is owned and managed by the caller</summary>
            External
        }

        /// <summary>
        /// Create and prepare a DbCommand, and call ExecuteReader with the appropriate CommandBehavior.
        /// </summary>
        /// <remarks>
        /// If we created and opened the connection, we want the connection to be closed when the DataReader is closed.
        /// 
        /// If the caller provided the connection, we want to leave it to them to manage.
        /// </remarks>
        /// <param name="connection">A valid DbConnection, on which to execute this command</param>
        /// <param name="transaction">A valid SqlTransaction, or 'null'</param>
        /// <param name="commandType">The CommandType (stored procedure, text, etc.)</param>
        /// <param name="commandText">The stored procedure name or T-SQL command</param>
        /// <param name="commandParameters">An array of DbParameters to be associated with the command or 'null' if no parameters are required</param>
        /// <param name="connectionOwnership">Indicates whether the connection parameter was provided by the caller, or created by DAAB</param>
        /// <returns>SqlDataReader containing the results of the command</returns>
        private static DbDataReader ExecuteReader(DbConnection connection, DbTransaction transaction, CommandType commandType, string commandText, DbParameter[] commandParameters, SqlConnectionOwnership connectionOwnership)
        {
            if (connection == null) throw new ArgumentNullException("connection");

            var mustCloseConnection = false;
            // Create a command and prepare it for execution
            var cmd = CreateCommand();
            try
            {
                PrepareCommand(cmd, connection, transaction, commandType, commandText, commandParameters, out mustCloseConnection);

                // Create a reader
               DbDataReader dataReader;

                // Call ExecuteReader with the appropriate CommandBehavior
                if (connectionOwnership == SqlConnectionOwnership.External)
                {
                    dataReader = cmd.ExecuteReader();
                }
                else
                {
                    dataReader = cmd.ExecuteReader(CommandBehavior.CloseConnection);
                }

                // Detach the DbParameters from the command object, so they can be used again.
                // HACK: There is a problem here, the output parameter values are fletched 
                // when the reader is closed, so if the parameters are detached from the command
                // then the SqlReader can´t set its values. 
                // When this happen, the parameters can´t be used again in other command.
                var canClear = true;
                foreach (DbParameter commandParameter in cmd.Parameters)
                {
                    if (commandParameter.Direction != ParameterDirection.Input)
                        canClear = false;
                }

                if (canClear)
                {
                    cmd.Parameters.Clear();
                }

                return dataReader;
            }
            catch
            {
                if (mustCloseConnection)
                    connection.Close();
                throw;
            }
        }

        /// <summary>
        /// Execute a DbCommand (that returns a resultset and takes no parameters) against the database specified in 
        /// the connection string. 
        /// </summary>
        /// <remarks>
        /// e.g.:  
        ///  SqlDataReader dr = ExecuteReader(connString, CommandType.StoredProcedure, "GetOrders");
        /// </remarks>
        /// <param name="connectionString">A valid connection string for a DbConnection</param>
        /// <param name="commandType">The CommandType (stored procedure, text, etc.)</param>
        /// <param name="commandText">The stored procedure name or T-SQL command</param>
        /// <returns>A SqlDataReader containing the resultset generated by the command</returns>
        public static DbDataReader ExecuteReader(string connectionString, CommandType commandType, string commandText)
        {
            // Pass through the call providing null for the set of DbParameters
            return ExecuteReader(connectionString, commandType, commandText, null);
        }

        /// <summary>
        /// Execute a DbCommand (that returns a resultset) against the database specified in the connection string 
        /// using the provided parameters.
        /// </summary>
        /// <remarks>
        /// e.g.:  
        ///  SqlDataReader dr = ExecuteReader(connString, CommandType.StoredProcedure, "GetOrders", new DbParameter("@prodid", 24));
        /// </remarks>
        /// <param name="connectionString">A valid connection string for a DbConnection</param>
        /// <param name="commandType">The CommandType (stored procedure, text, etc.)</param>
        /// <param name="commandText">The stored procedure name or T-SQL command</param>
        /// <param name="commandParameters">An array of SqlParamters used to execute the command</param>
        /// <returns>A SqlDataReader containing the resultset generated by the command</returns>
        public static DbDataReader ExecuteReader(string connectionString, CommandType commandType, string commandText, params DbParameter[] commandParameters)
        {
            if (string.IsNullOrEmpty(connectionString)) throw new ArgumentNullException("connectionString");
            DbConnection connection = null;
            try
            {
                connection = CreateConnection(connectionString);
                connection.Open();

                // Call the private overload that takes an internally owned connection in place of the connection string
                return ExecuteReader(connection, null, commandType, commandText, commandParameters, SqlConnectionOwnership.Internal);
            }
            catch
            {
                // If we fail to return the SqlDatReader, we need to close the connection ourselves
                if (connection != null) connection.Close();
                throw;
            }

        }

        /// <summary>
        /// Execute a stored procedure via a DbCommand (that returns a resultset) against the database specified in 
        /// the connection string using the provided parameter values.  This method will query the database to discover the parameters for the 
        /// stored procedure (the first time each stored procedure is called), and assign the values based on parameter order.
        /// </summary>
        /// <remarks>
        /// This method provides no access to output parameters or the stored procedure's return value parameter.
        /// 
        /// e.g.:  
        ///  SqlDataReader dr = ExecuteReader(connString, "GetOrders", 24, 36);
        /// </remarks>
        /// <param name="connectionString">A valid connection string for a DbConnection</param>
        /// <param name="spName">The name of the stored procedure</param>
        /// <param name="parameterValues">An array of objects to be assigned as the input values of the stored procedure</param>
        /// <returns>A SqlDataReader containing the resultset generated by the command</returns>
        public static DbDataReader ExecuteReader(string connectionString, string spName, params object[] parameterValues)
        {
            if (string.IsNullOrEmpty(connectionString)) throw new ArgumentNullException("connectionString");
            if (string.IsNullOrEmpty(spName)) throw new ArgumentNullException("spName");

            // If we receive parameter values, we need to figure out where they go
            if ((parameterValues != null) && (parameterValues.Length > 0))
            {
                var commandParameters = SqlHelperParameterCache.GetSpParameterSet(connectionString, spName);

                AssignParameterValues(commandParameters, parameterValues);

                return ExecuteReader(connectionString, CommandType.StoredProcedure, spName, commandParameters);
            }
            else
            {
                // Otherwise we can just call the SP without params
                return ExecuteReader(connectionString, CommandType.StoredProcedure, spName);
            }
        }

        /// <summary>
        /// Execute a DbCommand (that returns a resultset and takes no parameters) against the provided DbConnection. 
        /// </summary>
        /// <remarks>
        /// e.g.:  
        ///  SqlDataReader dr = ExecuteReader(conn, CommandType.StoredProcedure, "GetOrders");
        /// </remarks>
        /// <param name="connection">A valid DbConnection</param>
        /// <param name="commandType">The CommandType (stored procedure, text, etc.)</param>
        /// <param name="commandText">The stored procedure name or T-SQL command</param>
        /// <returns>A SqlDataReader containing the resultset generated by the command</returns>
        public static DbDataReader ExecuteReader(DbConnection connection, CommandType commandType, string commandText)
        {
            // Pass through the call providing null for the set of DbParameters
            return ExecuteReader(connection, commandType, commandText, null);
        }

        /// <summary>
        /// Execute a DbCommand (that returns a resultset) against the specified DbConnection 
        /// using the provided parameters.
        /// </summary>
        /// <remarks>
        /// e.g.:  
        ///  SqlDataReader dr = ExecuteReader(conn, CommandType.StoredProcedure, "GetOrders", new DbParameter("@prodid", 24));
        /// </remarks>
        /// <param name="connection">A valid DbConnection</param>
        /// <param name="commandType">The CommandType (stored procedure, text, etc.)</param>
        /// <param name="commandText">The stored procedure name or T-SQL command</param>
        /// <param name="commandParameters">An array of SqlParamters used to execute the command</param>
        /// <returns>A SqlDataReader containing the resultset generated by the command</returns>
        public static DbDataReader ExecuteReader(DbConnection connection, CommandType commandType, string commandText, params DbParameter[] commandParameters)
        {
            // Pass through the call to the private overload using a null transaction value and an externally owned connection
            return ExecuteReader(connection, null, commandType, commandText, commandParameters, SqlConnectionOwnership.External);
        }

        /// <summary>
        /// Execute a stored procedure via a DbCommand (that returns a resultset) against the specified DbConnection 
        /// using the provided parameter values.  This method will query the database to discover the parameters for the 
        /// stored procedure (the first time each stored procedure is called), and assign the values based on parameter order.
        /// </summary>
        /// <remarks>
        /// This method provides no access to output parameters or the stored procedure's return value parameter.
        /// 
        /// e.g.:  
        ///  SqlDataReader dr = ExecuteReader(conn, "GetOrders", 24, 36);
        /// </remarks>
        /// <param name="connection">A valid DbConnection</param>
        /// <param name="spName">The name of the stored procedure</param>
        /// <param name="parameterValues">An array of objects to be assigned as the input values of the stored procedure</param>
        /// <returns>A SqlDataReader containing the resultset generated by the command</returns>
        public static DbDataReader ExecuteReader(DbConnection connection, string spName, params object[] parameterValues)
        {
            if (connection == null) throw new ArgumentNullException("connection");
            if (string.IsNullOrEmpty(spName)) throw new ArgumentNullException("spName");

            // If we receive parameter values, we need to figure out where they go
            if ((parameterValues != null) && (parameterValues.Length > 0))
            {
                var commandParameters = SqlHelperParameterCache.GetSpParameterSet(connection, spName);

                AssignParameterValues(commandParameters, parameterValues);

                return ExecuteReader(connection, CommandType.StoredProcedure, spName, commandParameters);
            }
            else
            {
                // Otherwise we can just call the SP without params
                return ExecuteReader(connection, CommandType.StoredProcedure, spName);
            }
        }

        /// <summary>
        /// Execute a DbCommand (that returns a resultset and takes no parameters) against the provided SqlTransaction. 
        /// </summary>
        /// <remarks>
        /// e.g.:  
        ///  SqlDataReader dr = ExecuteReader(trans, CommandType.StoredProcedure, "GetOrders");
        /// </remarks>
        /// <param name="transaction">A valid SqlTransaction</param>
        /// <param name="commandType">The CommandType (stored procedure, text, etc.)</param>
        /// <param name="commandText">The stored procedure name or T-SQL command</param>
        /// <returns>A SqlDataReader containing the resultset generated by the command</returns>
        public static DbDataReader ExecuteReader(DbTransaction transaction, CommandType commandType, string commandText)
        {
            // Pass through the call providing null for the set of DbParameters
            return ExecuteReader(transaction, commandType, commandText, null);
        }

        /// <summary>
        /// Execute a DbCommand (that returns a resultset) against the specified SqlTransaction
        /// using the provided parameters.
        /// </summary>
        /// <remarks>
        /// e.g.:  
        ///   SqlDataReader dr = ExecuteReader(trans, CommandType.StoredProcedure, "GetOrders", new DbParameter("@prodid", 24));
        /// </remarks>
        /// <param name="transaction">A valid SqlTransaction</param>
        /// <param name="commandType">The CommandType (stored procedure, text, etc.)</param>
        /// <param name="commandText">The stored procedure name or T-SQL command</param>
        /// <param name="commandParameters">An array of SqlParamters used to execute the command</param>
        /// <returns>A SqlDataReader containing the resultset generated by the command</returns>
        public static DbDataReader ExecuteReader(DbTransaction transaction, CommandType commandType, string commandText, params DbParameter[] commandParameters)
        {
            if (transaction == null) throw new ArgumentNullException("transaction");
            if (transaction != null && transaction.Connection == null) throw new ArgumentException("The transaction was rollbacked or commited, please provide an open transaction.", "transaction");

            // Pass through to private overload, indicating that the connection is owned by the caller
            return ExecuteReader(transaction.Connection, transaction, commandType, commandText, commandParameters, SqlConnectionOwnership.External);
        }

        /// <summary>
        /// Execute a stored procedure via a DbCommand (that returns a resultset) against the specified
        /// SqlTransaction using the provided parameter values.  This method will query the database to discover the parameters for the 
        /// stored procedure (the first time each stored procedure is called), and assign the values based on parameter order.
        /// </summary>
        /// <remarks>
        /// This method provides no access to output parameters or the stored procedure's return value parameter.
        /// 
        /// e.g.:  
        ///  SqlDataReader dr = ExecuteReader(trans, "GetOrders", 24, 36);
        /// </remarks>
        /// <param name="transaction">A valid SqlTransaction</param>
        /// <param name="spName">The name of the stored procedure</param>
        /// <param name="parameterValues">An array of objects to be assigned as the input values of the stored procedure</param>
        /// <returns>A SqlDataReader containing the resultset generated by the command</returns>
        public static DbDataReader ExecuteReader(DbTransaction transaction, string spName, params object[] parameterValues)
        {
            if (transaction == null) throw new ArgumentNullException("transaction");
            if (transaction != null && transaction.Connection == null) throw new ArgumentException("The transaction was rollbacked or commited, please provide an open transaction.", "transaction");
            if (string.IsNullOrEmpty(spName)) throw new ArgumentNullException("spName");

            // If we receive parameter values, we need to figure out where they go
            if ((parameterValues != null) && (parameterValues.Length > 0))
            {
                var commandParameters = SqlHelperParameterCache.GetSpParameterSet(transaction.Connection, spName);

                AssignParameterValues(commandParameters, parameterValues);

                return ExecuteReader(transaction, CommandType.StoredProcedure, spName, commandParameters);
            }
            else
            {
                // Otherwise we can just call the SP without params
                return ExecuteReader(transaction, CommandType.StoredProcedure, spName);
            }
        }

        #endregion ExecuteReader

        #region ExecuteScalar

        /// <summary>
        /// Execute a DbCommand (that returns a 1x1 resultset and takes no parameters) against the database specified in 
        /// the connection string. 
        /// </summary>
        /// <remarks>
        /// e.g.:  
        ///  int orderCount = (int)ExecuteScalar(connString, CommandType.StoredProcedure, "GetOrderCount");
        /// </remarks>
        /// <param name="connectionString">A valid connection string for a DbConnection</param>
        /// <param name="commandType">The CommandType (stored procedure, text, etc.)</param>
        /// <param name="commandText">The stored procedure name or T-SQL command</param>
        /// <returns>An object containing the value in the 1x1 resultset generated by the command</returns>
        public static object ExecuteScalar(string connectionString, CommandType commandType, string commandText)
        {
            // Pass through the call providing null for the set of DbParameters
            return ExecuteScalar(connectionString, commandType, commandText, null);
        }

        /// <summary>
        /// Execute a DbCommand (that returns a 1x1 resultset) against the database specified in the connection string 
        /// using the provided parameters.
        /// </summary>
        /// <remarks>
        /// e.g.:  
        ///  int orderCount = (int)ExecuteScalar(connString, CommandType.StoredProcedure, "GetOrderCount", new DbParameter("@prodid", 24));
        /// </remarks>
        /// <param name="connectionString">A valid connection string for a DbConnection</param>
        /// <param name="commandType">The CommandType (stored procedure, text, etc.)</param>
        /// <param name="commandText">The stored procedure name or T-SQL command</param>
        /// <param name="commandParameters">An array of SqlParamters used to execute the command</param>
        /// <returns>An object containing the value in the 1x1 resultset generated by the command</returns>
        public static object ExecuteScalar(string connectionString, CommandType commandType, string commandText, params DbParameter[] commandParameters)
        {
            if (string.IsNullOrEmpty(connectionString)) throw new ArgumentNullException("connectionString");
            // Create & open a DbConnection, and dispose of it after we are done
            using (var connection = CreateConnection(connectionString))
            {
                connection.Open();

                // Call the overload that takes a connection in place of the connection string
                return ExecuteScalar(connection, commandType, commandText, commandParameters);
            }
        }

        /// <summary>
        /// Execute a stored procedure via a DbCommand (that returns a 1x1 resultset) against the database specified in 
        /// the connection string using the provided parameter values.  This method will query the database to discover the parameters for the 
        /// stored procedure (the first time each stored procedure is called), and assign the values based on parameter order.
        /// </summary>
        /// <remarks>
        /// This method provides no access to output parameters or the stored procedure's return value parameter.
        /// 
        /// e.g.:  
        ///  int orderCount = (int)ExecuteScalar(connString, "GetOrderCount", 24, 36);
        /// </remarks>
        /// <param name="connectionString">A valid connection string for a DbConnection</param>
        /// <param name="spName">The name of the stored procedure</param>
        /// <param name="parameterValues">An array of objects to be assigned as the input values of the stored procedure</param>
        /// <returns>An object containing the value in the 1x1 resultset generated by the command</returns>
        public static object ExecuteScalar(string connectionString, string spName, params object[] parameterValues)
        {
            if (string.IsNullOrEmpty(connectionString)) throw new ArgumentNullException("connectionString");
            if (string.IsNullOrEmpty(spName)) throw new ArgumentNullException("spName");

            // If we receive parameter values, we need to figure out where they go
            if ((parameterValues != null) && (parameterValues.Length > 0))
            {
                // Pull the parameters for this stored procedure from the parameter cache (or discover them & populate the cache)
                var commandParameters = SqlHelperParameterCache.GetSpParameterSet(connectionString, spName);

                // Assign the provided values to these parameters based on parameter order
                AssignParameterValues(commandParameters, parameterValues);

                // Call the overload that takes an array of DbParameters
                return ExecuteScalar(connectionString, CommandType.StoredProcedure, spName, commandParameters);
            }
            else
            {
                // Otherwise we can just call the SP without params
                return ExecuteScalar(connectionString, CommandType.StoredProcedure, spName);
            }
        }

        /// <summary>
        /// Execute a DbCommand (that returns a 1x1 resultset and takes no parameters) against the provided DbConnection. 
        /// </summary>
        /// <remarks>
        /// e.g.:  
        ///  int orderCount = (int)ExecuteScalar(conn, CommandType.StoredProcedure, "GetOrderCount");
        /// </remarks>
        /// <param name="connection">A valid DbConnection</param>
        /// <param name="commandType">The CommandType (stored procedure, text, etc.)</param>
        /// <param name="commandText">The stored procedure name or T-SQL command</param>
        /// <returns>An object containing the value in the 1x1 resultset generated by the command</returns>
        public static object ExecuteScalar(DbConnection connection, CommandType commandType, string commandText)
        {
            // Pass through the call providing null for the set of DbParameters
            return ExecuteScalar(connection, commandType, commandText, null);
        }

        /// <summary>
        /// Execute a DbCommand (that returns a 1x1 resultset) against the specified DbConnection 
        /// using the provided parameters.
        /// </summary>
        /// <remarks>
        /// e.g.:  
        ///  int orderCount = (int)ExecuteScalar(conn, CommandType.StoredProcedure, "GetOrderCount", new DbParameter("@prodid", 24));
        /// </remarks>
        /// <param name="connection">A valid DbConnection</param>
        /// <param name="commandType">The CommandType (stored procedure, text, etc.)</param>
        /// <param name="commandText">The stored procedure name or T-SQL command</param>
        /// <param name="commandParameters">An array of SqlParamters used to execute the command</param>
        /// <returns>An object containing the value in the 1x1 resultset generated by the command</returns>
        public static object ExecuteScalar(DbConnection connection, CommandType commandType, string commandText, params DbParameter[] commandParameters)
        {
            if (connection == null) throw new ArgumentNullException("connection");

            // Create a command and prepare it for execution
            var cmd = CreateCommand();

            var mustCloseConnection = false;
            PrepareCommand(cmd, connection, null, commandType, commandText, commandParameters, out mustCloseConnection);

            // Execute the command & return the results
            var retval = cmd.ExecuteScalar();

            // Detach the DbParameters from the command object, so they can be used again
            cmd.Parameters.Clear();

            if (mustCloseConnection)
                connection.Close();

            return retval;
        }

        /// <summary>
        /// Execute a stored procedure via a DbCommand (that returns a 1x1 resultset) against the specified DbConnection 
        /// using the provided parameter values.  This method will query the database to discover the parameters for the 
        /// stored procedure (the first time each stored procedure is called), and assign the values based on parameter order.
        /// </summary>
        /// <remarks>
        /// This method provides no access to output parameters or the stored procedure's return value parameter.
        /// 
        /// e.g.:  
        ///  int orderCount = (int)ExecuteScalar(conn, "GetOrderCount", 24, 36);
        /// </remarks>
        /// <param name="connection">A valid DbConnection</param>
        /// <param name="spName">The name of the stored procedure</param>
        /// <param name="parameterValues">An array of objects to be assigned as the input values of the stored procedure</param>
        /// <returns>An object containing the value in the 1x1 resultset generated by the command</returns>
        public static object ExecuteScalar(DbConnection connection, string spName, params object[] parameterValues)
        {
            if (connection == null) throw new ArgumentNullException("connection");
            if (string.IsNullOrEmpty(spName)) throw new ArgumentNullException("spName");

            // If we receive parameter values, we need to figure out where they go
            if ((parameterValues != null) && (parameterValues.Length > 0))
            {
                // Pull the parameters for this stored procedure from the parameter cache (or discover them & populate the cache)
                var commandParameters = SqlHelperParameterCache.GetSpParameterSet(connection, spName);

                // Assign the provided values to these parameters based on parameter order
                AssignParameterValues(commandParameters, parameterValues);

                // Call the overload that takes an array of DbParameters
                return ExecuteScalar(connection, CommandType.StoredProcedure, spName, commandParameters);
            }
            else
            {
                // Otherwise we can just call the SP without params
                return ExecuteScalar(connection, CommandType.StoredProcedure, spName);
            }
        }

        /// <summary>
        /// Execute a DbCommand (that returns a 1x1 resultset and takes no parameters) against the provided SqlTransaction. 
        /// </summary>
        /// <remarks>
        /// e.g.:  
        ///  int orderCount = (int)ExecuteScalar(trans, CommandType.StoredProcedure, "GetOrderCount");
        /// </remarks>
        /// <param name="transaction">A valid SqlTransaction</param>
        /// <param name="commandType">The CommandType (stored procedure, text, etc.)</param>
        /// <param name="commandText">The stored procedure name or T-SQL command</param>
        /// <returns>An object containing the value in the 1x1 resultset generated by the command</returns>
        public static object ExecuteScalar(DbTransaction transaction, CommandType commandType, string commandText)
        {
            // Pass through the call providing null for the set of DbParameters
            return ExecuteScalar(transaction, commandType, commandText, null);
        }

        /// <summary>
        /// Execute a DbCommand (that returns a 1x1 resultset) against the specified SqlTransaction
        /// using the provided parameters.
        /// </summary>
        /// <remarks>
        /// e.g.:  
        ///  int orderCount = (int)ExecuteScalar(trans, CommandType.StoredProcedure, "GetOrderCount", new DbParameter("@prodid", 24));
        /// </remarks>
        /// <param name="transaction">A valid SqlTransaction</param>
        /// <param name="commandType">The CommandType (stored procedure, text, etc.)</param>
        /// <param name="commandText">The stored procedure name or T-SQL command</param>
        /// <param name="commandParameters">An array of SqlParamters used to execute the command</param>
        /// <returns>An object containing the value in the 1x1 resultset generated by the command</returns>
        public static object ExecuteScalar(DbTransaction transaction, CommandType commandType, string commandText, params DbParameter[] commandParameters)
        {
            if (transaction == null) throw new ArgumentNullException("transaction");
            if (transaction != null && transaction.Connection == null) throw new ArgumentException("The transaction was rollbacked or commited, please provide an open transaction.", "transaction");

            // Create a command and prepare it for execution
            var cmd = CreateCommand();
            var mustCloseConnection = false;
            PrepareCommand(cmd, transaction.Connection, transaction, commandType, commandText, commandParameters, out mustCloseConnection);

            // Execute the command & return the results
            var retval = cmd.ExecuteScalar();

            // Detach the DbParameters from the command object, so they can be used again
            cmd.Parameters.Clear();
            return retval;
        }

        /// <summary>
        /// Execute a stored procedure via a DbCommand (that returns a 1x1 resultset) against the specified
        /// SqlTransaction using the provided parameter values.  This method will query the database to discover the parameters for the 
        /// stored procedure (the first time each stored procedure is called), and assign the values based on parameter order.
        /// </summary>
        /// <remarks>
        /// This method provides no access to output parameters or the stored procedure's return value parameter.
        /// 
        /// e.g.:  
        ///  int orderCount = (int)ExecuteScalar(trans, "GetOrderCount", 24, 36);
        /// </remarks>
        /// <param name="transaction">A valid SqlTransaction</param>
        /// <param name="spName">The name of the stored procedure</param>
        /// <param name="parameterValues">An array of objects to be assigned as the input values of the stored procedure</param>
        /// <returns>An object containing the value in the 1x1 resultset generated by the command</returns>
        public static object ExecuteScalar(DbTransaction transaction, string spName, params object[] parameterValues)
        {
            if (transaction == null) throw new ArgumentNullException("transaction");
            if (transaction != null && transaction.Connection == null) throw new ArgumentException("The transaction was rollbacked or commited, please provide an open transaction.", "transaction");
            if (string.IsNullOrEmpty(spName)) throw new ArgumentNullException("spName");

            // If we receive parameter values, we need to figure out where they go
            if ((parameterValues != null) && (parameterValues.Length > 0))
            {
                // PPull the parameters for this stored procedure from the parameter cache (or discover them & populate the cache)
                var commandParameters = SqlHelperParameterCache.GetSpParameterSet(transaction.Connection, spName);

                // Assign the provided values to these parameters based on parameter order
                AssignParameterValues(commandParameters, parameterValues);

                // Call the overload that takes an array of DbParameters
                return ExecuteScalar(transaction, CommandType.StoredProcedure, spName, commandParameters);
            }
            else
            {
                // Otherwise we can just call the SP without params
                return ExecuteScalar(transaction, CommandType.StoredProcedure, spName);
            }
        }

        #endregion ExecuteScalar

        #region ExecuteXmlReader - Commented Out
        ///// <summary>
        ///// Execute a DbCommand (that returns a resultset and takes no parameters) against the provided DbConnection. 
        ///// </summary>
        ///// <remarks>
        ///// e.g.:  
        /////  XmlReader r = ExecuteXmlReader(conn, CommandType.StoredProcedure, "GetOrders");
        ///// </remarks>
        ///// <param name="connection">A valid DbConnection</param>
        ///// <param name="commandType">The CommandType (stored procedure, text, etc.)</param>
        ///// <param name="commandText">The stored procedure name or T-SQL command using "FOR XML AUTO"</param>
        ///// <returns>An XmlReader containing the resultset generated by the command</returns>
        //public static XmlReader ExecuteXmlReader(DbConnection connection, CommandType commandType, string commandText)
        //{
        //    // Pass through the call providing null for the set of DbParameters
        //    return ExecuteXmlReader(connection, commandType, commandText, (DbParameter[])null);
        //}

        ///// <summary>
        ///// Execute a DbCommand (that returns a resultset) against the specified DbConnection 
        ///// using the provided parameters.
        ///// </summary>
        ///// <remarks>
        ///// e.g.:  
        /////  XmlReader r = ExecuteXmlReader(conn, CommandType.StoredProcedure, "GetOrders", new DbParameter("@prodid", 24));
        ///// </remarks>
        ///// <param name="connection">A valid DbConnection</param>
        ///// <param name="commandType">The CommandType (stored procedure, text, etc.)</param>
        ///// <param name="commandText">The stored procedure name or T-SQL command using "FOR XML AUTO"</param>
        ///// <param name="commandParameters">An array of SqlParamters used to execute the command</param>
        ///// <returns>An XmlReader containing the resultset generated by the command</returns>
        //public static XmlReader ExecuteXmlReader(DbConnection connection, CommandType commandType, string commandText, params DbParameter[] commandParameters)
        //{
        //    if (connection == null) throw new ArgumentNullException("connection");

        //    var mustCloseConnection = false;
        //    // Create a command and prepare it for execution
        //    var cmd = CreateCommand();
        //    try
        //    {
        //        PrepareCommand(cmd, connection, null, commandType, commandText, commandParameters, out mustCloseConnection);

        //        // Create the DataAdapter & DataSet
        //        var retval = cmd.ExecuteXmlReader();

        //        // Detach the DbParameters from the command object, so they can be used again
        //        cmd.Parameters.Clear();

        //        return retval;
        //    }
        //    catch
        //    {
        //        if (mustCloseConnection)
        //            connection.Close();
        //        throw;
        //    }
        //}

        ///// <summary>
        ///// Execute a stored procedure via a DbCommand (that returns a resultset) against the specified DbConnection 
        ///// using the provided parameter values.  This method will query the database to discover the parameters for the 
        ///// stored procedure (the first time each stored procedure is called), and assign the values based on parameter order.
        ///// </summary>
        ///// <remarks>
        ///// This method provides no access to output parameters or the stored procedure's return value parameter.
        ///// 
        ///// e.g.:  
        /////  XmlReader r = ExecuteXmlReader(conn, "GetOrders", 24, 36);
        ///// </remarks>
        ///// <param name="connection">A valid DbConnection</param>
        ///// <param name="spName">The name of the stored procedure using "FOR XML AUTO"</param>
        ///// <param name="parameterValues">An array of objects to be assigned as the input values of the stored procedure</param>
        ///// <returns>An XmlReader containing the resultset generated by the command</returns>
        //public static XmlReader ExecuteXmlReader(DbConnection connection, string spName, params object[] parameterValues)
        //{
        //    if (connection == null) throw new ArgumentNullException("connection");
        //    if (string.IsNullOrEmpty(spName)) throw new ArgumentNullException("spName");

        //    // If we receive parameter values, we need to figure out where they go
        //    if ((parameterValues != null) && (parameterValues.Length > 0))
        //    {
        //        // Pull the parameters for this stored procedure from the parameter cache (or discover them & populate the cache)
        //        var commandParameters = SqlHelperParameterCache.GetSpParameterSet(connection, spName);

        //        // Assign the provided values to these parameters based on parameter order
        //        AssignParameterValues(commandParameters, parameterValues);

        //        // Call the overload that takes an array of DbParameters
        //        return ExecuteXmlReader(connection, CommandType.StoredProcedure, spName, commandParameters);
        //    }
        //    else
        //    {
        //        // Otherwise we can just call the SP without params
        //        return ExecuteXmlReader(connection, CommandType.StoredProcedure, spName);
        //    }
        //}

        ///// <summary>
        ///// Execute a DbCommand (that returns a resultset and takes no parameters) against the provided SqlTransaction. 
        ///// </summary>
        ///// <remarks>
        ///// e.g.:  
        /////  XmlReader r = ExecuteXmlReader(trans, CommandType.StoredProcedure, "GetOrders");
        ///// </remarks>
        ///// <param name="transaction">A valid SqlTransaction</param>
        ///// <param name="commandType">The CommandType (stored procedure, text, etc.)</param>
        ///// <param name="commandText">The stored procedure name or T-SQL command using "FOR XML AUTO"</param>
        ///// <returns>An XmlReader containing the resultset generated by the command</returns>
        //public static XmlReader ExecuteXmlReader(SqlTransaction transaction, CommandType commandType, string commandText)
        //{
        //    // Pass through the call providing null for the set of DbParameters
        //    return ExecuteXmlReader(transaction, commandType, commandText, (DbParameter[])null);
        //}

        ///// <summary>
        ///// Execute a DbCommand (that returns a resultset) against the specified SqlTransaction
        ///// using the provided parameters.
        ///// </summary>
        ///// <remarks>
        ///// e.g.:  
        /////  XmlReader r = ExecuteXmlReader(trans, CommandType.StoredProcedure, "GetOrders", new DbParameter("@prodid", 24));
        ///// </remarks>
        ///// <param name="transaction">A valid SqlTransaction</param>
        ///// <param name="commandType">The CommandType (stored procedure, text, etc.)</param>
        ///// <param name="commandText">The stored procedure name or T-SQL command using "FOR XML AUTO"</param>
        ///// <param name="commandParameters">An array of SqlParamters used to execute the command</param>
        ///// <returns>An XmlReader containing the resultset generated by the command</returns>
        //public static XmlReader ExecuteXmlReader(DbTransaction transaction, CommandType commandType, string commandText, params DbParameter[] commandParameters)
        //{
        //    if (transaction == null) throw new ArgumentNullException("transaction");
        //    if (transaction != null && transaction.Connection == null) throw new ArgumentException("The transaction was rollbacked or commited, please provide an open transaction.", "transaction");

        //    // Create a command and prepare it for execution
        //    var cmd = CreateCommand();
        //    var mustCloseConnection = false;
        //    PrepareCommand(cmd, transaction.Connection, transaction, commandType, commandText, commandParameters, out mustCloseConnection);

        //    // Create the DataAdapter & DataSet
        //    var retval = cmd.ExecuteXmlReader();

        //    // Detach the DbParameters from the command object, so they can be used again
        //    cmd.Parameters.Clear();
        //    return retval;
        //}

        ///// <summary>
        ///// Execute a stored procedure via a DbCommand (that returns a resultset) against the specified 
        ///// SqlTransaction using the provided parameter values.  This method will query the database to discover the parameters for the 
        ///// stored procedure (the first time each stored procedure is called), and assign the values based on parameter order.
        ///// </summary>
        ///// <remarks>
        ///// This method provides no access to output parameters or the stored procedure's return value parameter.
        ///// 
        ///// e.g.:  
        /////  XmlReader r = ExecuteXmlReader(trans, "GetOrders", 24, 36);
        ///// </remarks>
        ///// <param name="transaction">A valid SqlTransaction</param>
        ///// <param name="spName">The name of the stored procedure</param>
        ///// <param name="parameterValues">An array of objects to be assigned as the input values of the stored procedure</param>
        ///// <returns>A dataset containing the resultset generated by the command</returns>
        //public static XmlReader ExecuteXmlReader(DbTransaction transaction, string spName, params object[] parameterValues)
        //{
        //    if (transaction == null) throw new ArgumentNullException("transaction");
        //    if (transaction != null && transaction.Connection == null) throw new ArgumentException("The transaction was rollbacked or commited, please provide an open transaction.", "transaction");
        //    if (string.IsNullOrEmpty(spName)) throw new ArgumentNullException("spName");

        //    // If we receive parameter values, we need to figure out where they go
        //    if ((parameterValues != null) && (parameterValues.Length > 0))
        //    {
        //        // Pull the parameters for this stored procedure from the parameter cache (or discover them & populate the cache)
        //        var commandParameters = SqlHelperParameterCache.GetSpParameterSet(transaction.Connection, spName);

        //        // Assign the provided values to these parameters based on parameter order
        //        AssignParameterValues(commandParameters, parameterValues);

        //        // Call the overload that takes an array of DbParameters
        //        return ExecuteXmlReader(transaction, CommandType.StoredProcedure, spName, commandParameters);
        //    }
        //    else
        //    {
        //        // Otherwise we can just call the SP without params
        //        return ExecuteXmlReader(transaction, CommandType.StoredProcedure, spName);
        //    }
        //}

        #endregion ExecuteXmlReader

        #region FillDataset
        /// <summary>
        /// Execute a DbCommand (that returns a resultset and takes no parameters) against the database specified in 
        /// the connection string. 
        /// </summary>
        /// <remarks>
        /// e.g.:  
        ///  FillDataset(connString, CommandType.StoredProcedure, "GetOrders", ds, new string[] {"orders"});
        /// </remarks>
        /// <param name="connectionString">A valid connection string for a DbConnection</param>
        /// <param name="commandType">The CommandType (stored procedure, text, etc.)</param>
        /// <param name="commandText">The stored procedure name or T-SQL command</param>
        /// <param name="dataSet">A dataset wich will contain the resultset generated by the command</param>
        /// <param name="tableNames">This array will be used to create table mappings allowing the DataTables to be referenced
        /// by a user defined name (probably the actual table name)</param>
        public static void FillDataset(string connectionString, CommandType commandType, string commandText, DataSet dataSet, string[] tableNames)
        {
            if (string.IsNullOrEmpty(connectionString)) throw new ArgumentNullException("connectionString");
            if (dataSet == null) throw new ArgumentNullException("dataSet");

            // Create & open a DbConnection, and dispose of it after we are done
            using (var connection = CreateConnection(connectionString))
            {
                connection.Open();

                // Call the overload that takes a connection in place of the connection string
                FillDataset(connection, commandType, commandText, dataSet, tableNames);
            }
        }

        /// <summary>
        /// Execute a DbCommand (that returns a resultset) against the database specified in the connection string 
        /// using the provided parameters.
        /// </summary>
        /// <remarks>
        /// e.g.:  
        ///  FillDataset(connString, CommandType.StoredProcedure, "GetOrders", ds, new string[] {"orders"}, new DbParameter("@prodid", 24));
        /// </remarks>
        /// <param name="connectionString">A valid connection string for a DbConnection</param>
        /// <param name="commandType">The CommandType (stored procedure, text, etc.)</param>
        /// <param name="commandText">The stored procedure name or T-SQL command</param>
        /// <param name="commandParameters">An array of SqlParamters used to execute the command</param>
        /// <param name="dataSet">A dataset wich will contain the resultset generated by the command</param>
        /// <param name="tableNames">This array will be used to create table mappings allowing the DataTables to be referenced
        /// by a user defined name (probably the actual table name)
        /// </param>
        public static void FillDataset(string connectionString, CommandType commandType, string commandText, DataSet dataSet, string[] tableNames, params DbParameter[] commandParameters)
        {
            if (string.IsNullOrEmpty(connectionString)) throw new ArgumentNullException("connectionString");
            if (dataSet == null) throw new ArgumentNullException("dataSet");
            // Create & open a DbConnection, and dispose of it after we are done
            using (var connection = CreateConnection(connectionString))
            {
                connection.Open();

                // Call the overload that takes a connection in place of the connection string
                FillDataset(connection, commandType, commandText, dataSet, tableNames, commandParameters);
            }
        }

        /// <summary>
        /// Execute a stored procedure via a DbCommand (that returns a resultset) against the database specified in 
        /// the connection string using the provided parameter values.  This method will query the database to discover the parameters for the 
        /// stored procedure (the first time each stored procedure is called), and assign the values based on parameter order.
        /// </summary>
        /// <remarks>
        /// This method provides no access to output parameters or the stored procedure's return value parameter.
        /// 
        /// e.g.:  
        ///  FillDataset(connString, CommandType.StoredProcedure, "GetOrders", ds, new string[] {"orders"}, 24);
        /// </remarks>
        /// <param name="connectionString">A valid connection string for a DbConnection</param>
        /// <param name="spName">The name of the stored procedure</param>
        /// <param name="dataSet">A dataset wich will contain the resultset generated by the command</param>
        /// <param name="tableNames">This array will be used to create table mappings allowing the DataTables to be referenced
        /// by a user defined name (probably the actual table name)
        /// </param>    
        /// <param name="parameterValues">An array of objects to be assigned as the input values of the stored procedure</param>
        public static void FillDataset(string connectionString, string spName, DataSet dataSet, string[] tableNames, params object[] parameterValues)
        {
            if (string.IsNullOrEmpty(connectionString)) throw new ArgumentNullException("connectionString");
            if (dataSet == null) throw new ArgumentNullException("dataSet");
            // Create & open a DbConnection, and dispose of it after we are done
            using (var connection = CreateConnection(connectionString))
            {
                connection.Open();

                // Call the overload that takes a connection in place of the connection string
                FillDataset(connection, spName, dataSet, tableNames, parameterValues);
            }
        }

        /// <summary>
        /// Execute a DbCommand (that returns a resultset and takes no parameters) against the provided DbConnection. 
        /// </summary>
        /// <remarks>
        /// e.g.:  
        ///  FillDataset(conn, CommandType.StoredProcedure, "GetOrders", ds, new string[] {"orders"});
        /// </remarks>
        /// <param name="connection">A valid DbConnection</param>
        /// <param name="commandType">The CommandType (stored procedure, text, etc.)</param>
        /// <param name="commandText">The stored procedure name or T-SQL command</param>
        /// <param name="dataSet">A dataset wich will contain the resultset generated by the command</param>
        /// <param name="tableNames">This array will be used to create table mappings allowing the DataTables to be referenced
        /// by a user defined name (probably the actual table name)
        /// </param>    
        public static void FillDataset(DbConnection connection, CommandType commandType, string commandText, DataSet dataSet, string[] tableNames)
        {
            FillDataset(connection, commandType, commandText, dataSet, tableNames, null);
        }

        /// <summary>
        /// Execute a DbCommand (that returns a resultset) against the specified DbConnection 
        /// using the provided parameters.
        /// </summary>
        /// <remarks>
        /// e.g.:  
        ///  FillDataset(conn, CommandType.StoredProcedure, "GetOrders", ds, new string[] {"orders"}, new DbParameter("@prodid", 24));
        /// </remarks>
        /// <param name="connection">A valid DbConnection</param>
        /// <param name="commandType">The CommandType (stored procedure, text, etc.)</param>
        /// <param name="commandText">The stored procedure name or T-SQL command</param>
        /// <param name="dataSet">A dataset wich will contain the resultset generated by the command</param>
        /// <param name="tableNames">This array will be used to create table mappings allowing the DataTables to be referenced
        /// by a user defined name (probably the actual table name)
        /// </param>
        /// <param name="commandParameters">An array of SqlParamters used to execute the command</param>
        public static void FillDataset(DbConnection connection, CommandType commandType, string commandText, DataSet dataSet, string[] tableNames, params DbParameter[] commandParameters)
        {
            FillDataset(connection, null, commandType, commandText, dataSet, tableNames, commandParameters);
        }

        /// <summary>
        /// Execute a stored procedure via a DbCommand (that returns a resultset) against the specified DbConnection 
        /// using the provided parameter values.  This method will query the database to discover the parameters for the 
        /// stored procedure (the first time each stored procedure is called), and assign the values based on parameter order.
        /// </summary>
        /// <remarks>
        /// This method provides no access to output parameters or the stored procedure's return value parameter.
        /// 
        /// e.g.:  
        ///  FillDataset(conn, "GetOrders", ds, new string[] {"orders"}, 24, 36);
        /// </remarks>
        /// <param name="connection">A valid DbConnection</param>
        /// <param name="spName">The name of the stored procedure</param>
        /// <param name="dataSet">A dataset wich will contain the resultset generated by the command</param>
        /// <param name="tableNames">This array will be used to create table mappings allowing the DataTables to be referenced
        /// by a user defined name (probably the actual table name)
        /// </param>
        /// <param name="parameterValues">An array of objects to be assigned as the input values of the stored procedure</param>
        public static void FillDataset(DbConnection connection, string spName, DataSet dataSet, string[] tableNames, params object[] parameterValues)
        {
            if (connection == null) throw new ArgumentNullException("connection");
            if (dataSet == null) throw new ArgumentNullException("dataSet");
            if (string.IsNullOrEmpty(spName)) throw new ArgumentNullException("spName");

            // If we receive parameter values, we need to figure out where they go
            if ((parameterValues != null) && (parameterValues.Length > 0))
            {
                // Pull the parameters for this stored procedure from the parameter cache (or discover them & populate the cache)
                var commandParameters = SqlHelperParameterCache.GetSpParameterSet(connection, spName);

                // Assign the provided values to these parameters based on parameter order
                AssignParameterValues(commandParameters, parameterValues);

                // Call the overload that takes an array of DbParameters
                FillDataset(connection, CommandType.StoredProcedure, spName, dataSet, tableNames, commandParameters);
            }
            else
            {
                // Otherwise we can just call the SP without params
                FillDataset(connection, CommandType.StoredProcedure, spName, dataSet, tableNames);
            }
        }

        /// <summary>
        /// Execute a DbCommand (that returns a resultset and takes no parameters) against the provided SqlTransaction. 
        /// </summary>
        /// <remarks>
        /// e.g.:  
        ///  FillDataset(trans, CommandType.StoredProcedure, "GetOrders", ds, new string[] {"orders"});
        /// </remarks>
        /// <param name="transaction">A valid SqlTransaction</param>
        /// <param name="commandType">The CommandType (stored procedure, text, etc.)</param>
        /// <param name="commandText">The stored procedure name or T-SQL command</param>
        /// <param name="dataSet">A dataset wich will contain the resultset generated by the command</param>
        /// <param name="tableNames">This array will be used to create table mappings allowing the DataTables to be referenced
        /// by a user defined name (probably the actual table name)
        /// </param>
        public static void FillDataset(DbTransaction transaction, CommandType commandType, string commandText, DataSet dataSet, string[] tableNames)
        {
            FillDataset(transaction, commandType, commandText, dataSet, tableNames, null);
        }

        /// <summary>
        /// Execute a DbCommand (that returns a resultset) against the specified SqlTransaction
        /// using the provided parameters.
        /// </summary>
        /// <remarks>
        /// e.g.:  
        ///  FillDataset(trans, CommandType.StoredProcedure, "GetOrders", ds, new string[] {"orders"}, new DbParameter("@prodid", 24));
        /// </remarks>
        /// <param name="transaction">A valid SqlTransaction</param>
        /// <param name="commandType">The CommandType (stored procedure, text, etc.)</param>
        /// <param name="commandText">The stored procedure name or T-SQL command</param>
        /// <param name="dataSet">A dataset wich will contain the resultset generated by the command</param>
        /// <param name="tableNames">This array will be used to create table mappings allowing the DataTables to be referenced
        /// by a user defined name (probably the actual table name)
        /// </param>
        /// <param name="commandParameters">An array of SqlParamters used to execute the command</param>
        public static void FillDataset(DbTransaction transaction, CommandType commandType, string commandText, DataSet dataSet, string[] tableNames, params DbParameter[] commandParameters)
        {
            FillDataset(transaction.Connection, transaction, commandType, commandText, dataSet, tableNames, commandParameters);
        }

        /// <summary>
        /// Execute a stored procedure via a DbCommand (that returns a resultset) against the specified 
        /// SqlTransaction using the provided parameter values.  This method will query the database to discover the parameters for the 
        /// stored procedure (the first time each stored procedure is called), and assign the values based on parameter order.
        /// </summary>
        /// <remarks>
        /// This method provides no access to output parameters or the stored procedure's return value parameter.
        /// 
        /// e.g.:  
        ///  FillDataset(trans, "GetOrders", ds, new string[]{"orders"}, 24, 36);
        /// </remarks>
        /// <param name="transaction">A valid SqlTransaction</param>
        /// <param name="spName">The name of the stored procedure</param>
        /// <param name="dataSet">A dataset wich will contain the resultset generated by the command</param>
        /// <param name="tableNames">This array will be used to create table mappings allowing the DataTables to be referenced
        /// by a user defined name (probably the actual table name)
        /// </param>
        /// <param name="parameterValues">An array of objects to be assigned as the input values of the stored procedure</param>
        public static void FillDataset(DbTransaction transaction, string spName, DataSet dataSet, string[] tableNames, params object[] parameterValues)
        {
            if (transaction == null) throw new ArgumentNullException("transaction");
            if (transaction != null && transaction.Connection == null) throw new ArgumentException("The transaction was rollbacked or commited, please provide an open transaction.", "transaction");
            if (dataSet == null) throw new ArgumentNullException("dataSet");
            if (string.IsNullOrEmpty(spName)) throw new ArgumentNullException("spName");

            // If we receive parameter values, we need to figure out where they go
            if ((parameterValues != null) && (parameterValues.Length > 0))
            {
                // Pull the parameters for this stored procedure from the parameter cache (or discover them & populate the cache)
                var commandParameters = SqlHelperParameterCache.GetSpParameterSet(transaction.Connection, spName);

                // Assign the provided values to these parameters based on parameter order
                AssignParameterValues(commandParameters, parameterValues);

                // Call the overload that takes an array of DbParameters
                FillDataset(transaction, CommandType.StoredProcedure, spName, dataSet, tableNames, commandParameters);
            }
            else
            {
                // Otherwise we can just call the SP without params
                FillDataset(transaction, CommandType.StoredProcedure, spName, dataSet, tableNames);
            }
        }

        /// <summary>
        /// Private helper method that execute a DbCommand (that returns a resultset) against the specified SqlTransaction and DbConnection
        /// using the provided parameters.
        /// </summary>
        /// <remarks>
        /// e.g.:  
        ///  FillDataset(conn, trans, CommandType.StoredProcedure, "GetOrders", ds, new string[] {"orders"}, new DbParameter("@prodid", 24));
        /// </remarks>
        /// <param name="connection">A valid DbConnection</param>
        /// <param name="transaction">A valid SqlTransaction</param>
        /// <param name="commandType">The CommandType (stored procedure, text, etc.)</param>
        /// <param name="commandText">The stored procedure name or T-SQL command</param>
        /// <param name="dataSet">A dataset wich will contain the resultset generated by the command</param>
        /// <param name="tableNames">This array will be used to create table mappings allowing the DataTables to be referenced
        /// by a user defined name (probably the actual table name)
        /// </param>
        /// <param name="commandParameters">An array of SqlParamters used to execute the command</param>
        private static void FillDataset(DbConnection connection, DbTransaction transaction, CommandType commandType, string commandText, DataSet dataSet, string[] tableNames, params DbParameter[] commandParameters)
        {
            if (connection == null) throw new ArgumentNullException("connection");
            if (dataSet == null) throw new ArgumentNullException("dataSet");

            // Create a command and prepare it for execution
            var command = CreateCommand();
            var mustCloseConnection = false;
            PrepareCommand(command, connection, transaction, commandType, commandText, commandParameters, out mustCloseConnection);

            // Create the DataAdapter & DataSet
            var dataAdapter = CreateDataAdapter(command);
            try
            {
                // Add the table mappings specified by the user
                if (tableNames != null && tableNames.Length > 0)
                {
                    string tableName = "Table";
                    for (int index = 0; index < tableNames.Length; index++)
                    {
                        if (tableNames[index] == null || tableNames[index].Length == 0) throw new ArgumentException("The tableNames parameter must contain a list of tables, a value was provided as null or empty string.", "tableNames");
                        dataAdapter.TableMappings.Add(tableName, tableNames[index]);
                        tableName += (index + 1).ToString();
                    }
                }

                // Fill the DataSet using default values for DataTable names, etc
                dataAdapter.Fill(dataSet);

                // Detach the DbParameters from the command object, so they can be used again
                command.Parameters.Clear();
            }
            finally
            {
                command.Dispose();
            }
            
            if (mustCloseConnection) connection.Close();
        }
        #endregion

        //#region UpdateDataset
        ///// <summary>
        ///// Executes the respective command for each inserted, updated, or deleted row in the DataSet.
        ///// </summary>
        ///// <remarks>
        ///// e.g.:  
        /////  UpdateDataset(conn, insertCommand, deleteCommand, updateCommand, dataSet, "Order");
        ///// </remarks>
        ///// <param name="insertCommand">A valid transact-SQL statement or stored procedure to insert new records into the data source</param>
        ///// <param name="deleteCommand">A valid transact-SQL statement or stored procedure to delete records from the data source</param>
        ///// <param name="updateCommand">A valid transact-SQL statement or stored procedure used to update records in the data source</param>
        ///// <param name="dataSet">The DataSet used to update the data source</param>
        ///// <param name="tableName">The DataTable used to update the data source.</param>
        //public static void UpdateDataset(DbCommand insertCommand, DbCommand deleteCommand, DbCommand updateCommand, DataSet dataSet, string tableName)
        //{
        //    if (insertCommand == null) throw new ArgumentNullException("insertCommand");
        //    if (deleteCommand == null) throw new ArgumentNullException("deleteCommand");
        //    if (updateCommand == null) throw new ArgumentNullException("updateCommand");
        //    if (string.IsNullOrEmpty(tableName)) throw new ArgumentNullException("tableName");

        //    // Create a IDbDataAdapter, and dispose of it after we are done
        //    using (var dataAdapter = new IDbDataAdapter())
        //    {
        //        // Set the data adapter commands
        //        dataAdapter.UpdateCommand = updateCommand as DbCommand;
        //        dataAdapter.InsertCommand = insertCommand as DbCommand;
        //        dataAdapter.DeleteCommand = deleteCommand as DbCommand;

        //        // Update the dataset changes in the data source
        //        dataAdapter.Update(dataSet, tableName);

        //        // Commit all the changes made to the DataSet
        //        dataSet.AcceptChanges();
        //    }
        //}
        //#endregion

        //#region CreateCommand
        ///// <summary>
        ///// Simplify the creation of a Sql command object by allowing
        ///// a stored procedure and optional parameters to be provided
        ///// </summary>
        ///// <remarks>
        ///// e.g.:  
        /////  DbCommand command = CreateCommand(conn, "AddCustomer", "CustomerID", "CustomerName");
        ///// </remarks>
        ///// <param name="connection">A valid DbConnection object</param>
        ///// <param name="spName">The name of the stored procedure</param>
        ///// <param name="sourceColumns">An array of string to be assigned as the source columns of the stored procedure parameters</param>
        ///// <returns>A valid DbCommand object</returns>
        //public static DbCommand CreateCommand(DbConnection connection, string spName, params string[] sourceColumns)
        //{
        //    if (connection == null) throw new ArgumentNullException("connection");
        //    if (string.IsNullOrEmpty(spName)) throw new ArgumentNullException("spName");

        //    // Create a DbCommand
        //    var cmd = new DbCommand(spName, connection) {CommandType = CommandType.StoredProcedure};

        //    // If we receive parameter values, we need to figure out where they go
        //    if ((sourceColumns != null) && (sourceColumns.Length > 0))
        //    {
        //        // Pull the parameters for this stored procedure from the parameter cache (or discover them & populate the cache)
        //        var commandParameters = SqlHelperParameterCache.GetSpParameterSet(connection, spName);

        //        // Assign the provided source columns to these parameters based on parameter order
        //        for (var index = 0; index < sourceColumns.Length; index++)
        //            commandParameters[index].SourceColumn = sourceColumns[index];

        //        // Attach the discovered parameters to the DbCommand object
        //        AttachParameters(cmd, commandParameters);
        //    }

        //    return cmd;
        //}
        //#endregion

        #region ExecuteNonQueryTypedParams
        /// <summary>
        /// Execute a stored procedure via a DbCommand (that returns no resultset) against the database specified in 
        /// the connection string using the dataRow column values as the stored procedure's parameters values.
        /// This method will query the database to discover the parameters for the 
        /// stored procedure (the first time each stored procedure is called), and assign the values based on row values.
        /// </summary>
        /// <param name="connectionString">A valid connection string for a DbConnection</param>
        /// <param name="spName">The name of the stored procedure</param>
        /// <param name="dataRow">The dataRow used to hold the stored procedure's parameter values.</param>
        /// <returns>An int representing the number of rows affected by the command</returns>
        public static int ExecuteNonQueryTypedParams(String connectionString, String spName, DataRow dataRow)
        {
            if (string.IsNullOrEmpty(connectionString)) throw new ArgumentNullException("connectionString");
            if (string.IsNullOrEmpty(spName)) throw new ArgumentNullException("spName");

            // If the row has values, the store procedure parameters must be initialized
            if (dataRow != null && dataRow.ItemArray.Length > 0)
            {
                // Pull the parameters for this stored procedure from the parameter cache (or discover them & populate the cache)
                var commandParameters = SqlHelperParameterCache.GetSpParameterSet(connectionString, spName);

                // Set the parameters values
                AssignParameterValues(commandParameters, dataRow);

                return DAAB.ExecuteNonQuery(connectionString, CommandType.StoredProcedure, spName, commandParameters);
            }
            else
            {
                return DAAB.ExecuteNonQuery(connectionString, CommandType.StoredProcedure, spName);
            }
        }

        /// <summary>
        /// Execute a stored procedure via a DbCommand (that returns no resultset) against the specified DbConnection 
        /// using the dataRow column values as the stored procedure's parameters values.  
        /// This method will query the database to discover the parameters for the 
        /// stored procedure (the first time each stored procedure is called), and assign the values based on row values.
        /// </summary>
        /// <param name="connection">A valid DbConnection object</param>
        /// <param name="spName">The name of the stored procedure</param>
        /// <param name="dataRow">The dataRow used to hold the stored procedure's parameter values.</param>
        /// <returns>An int representing the number of rows affected by the command</returns>
        public static int ExecuteNonQueryTypedParams(DbConnection connection, String spName, DataRow dataRow)
        {
            if (connection == null) throw new ArgumentNullException("connection");
            if (string.IsNullOrEmpty(spName)) throw new ArgumentNullException("spName");

            // If the row has values, the store procedure parameters must be initialized
            if (dataRow != null && dataRow.ItemArray.Length > 0)
            {
                // Pull the parameters for this stored procedure from the parameter cache (or discover them & populate the cache)
                var commandParameters = SqlHelperParameterCache.GetSpParameterSet(connection, spName);

                // Set the parameters values
                AssignParameterValues(commandParameters, dataRow);

                return DAAB.ExecuteNonQuery(connection, CommandType.StoredProcedure, spName, commandParameters);
            }
            else
            {
                return DAAB.ExecuteNonQuery(connection, CommandType.StoredProcedure, spName);
            }
        }

        /// <summary>
        /// Execute a stored procedure via a DbCommand (that returns no resultset) against the specified
        /// SqlTransaction using the dataRow column values as the stored procedure's parameters values.
        /// This method will query the database to discover the parameters for the 
        /// stored procedure (the first time each stored procedure is called), and assign the values based on row values.
        /// </summary>
        /// <param name="transaction">A valid SqlTransaction object</param>
        /// <param name="spName">The name of the stored procedure</param>
        /// <param name="dataRow">The dataRow used to hold the stored procedure's parameter values.</param>
        /// <returns>An int representing the number of rows affected by the command</returns>
        public static int ExecuteNonQueryTypedParams(DbTransaction transaction, String spName, DataRow dataRow)
        {
            if (transaction == null) throw new ArgumentNullException("transaction");
            if (transaction != null && transaction.Connection == null) throw new ArgumentException("The transaction was rollbacked or commited, please provide an open transaction.", "transaction");
            if (string.IsNullOrEmpty(spName)) throw new ArgumentNullException("spName");

            // Sf the row has values, the store procedure parameters must be initialized
            if (dataRow != null && dataRow.ItemArray.Length > 0)
            {
                // Pull the parameters for this stored procedure from the parameter cache (or discover them & populate the cache)
                var commandParameters = SqlHelperParameterCache.GetSpParameterSet(transaction.Connection, spName);

                // Set the parameters values
                AssignParameterValues(commandParameters, dataRow);

                return DAAB.ExecuteNonQuery(transaction, CommandType.StoredProcedure, spName, commandParameters);
            }
            else
            {
                return DAAB.ExecuteNonQuery(transaction, CommandType.StoredProcedure, spName);
            }
        }
        #endregion

        #region ExecuteDatasetTypedParams
        /// <summary>
        /// Execute a stored procedure via a DbCommand (that returns a resultset) against the database specified in 
        /// the connection string using the dataRow column values as the stored procedure's parameters values.
        /// This method will query the database to discover the parameters for the 
        /// stored procedure (the first time each stored procedure is called), and assign the values based on row values.
        /// </summary>
        /// <param name="connectionString">A valid connection string for a DbConnection</param>
        /// <param name="spName">The name of the stored procedure</param>
        /// <param name="dataRow">The dataRow used to hold the stored procedure's parameter values.</param>
        /// <returns>A dataset containing the resultset generated by the command</returns>
        public static DataSet ExecuteDatasetTypedParams(string connectionString, String spName, DataRow dataRow)
        {
            if (string.IsNullOrEmpty(connectionString)) throw new ArgumentNullException("connectionString");
            if (string.IsNullOrEmpty(spName)) throw new ArgumentNullException("spName");

            //If the row has values, the store procedure parameters must be initialized
            if (dataRow != null && dataRow.ItemArray.Length > 0)
            {
                // Pull the parameters for this stored procedure from the parameter cache (or discover them & populate the cache)
                var commandParameters = SqlHelperParameterCache.GetSpParameterSet(connectionString, spName);

                // Set the parameters values
                AssignParameterValues(commandParameters, dataRow);

                return DAAB.ExecuteDataset(connectionString, CommandType.StoredProcedure, spName, commandParameters);
            }
            else
            {
                return DAAB.ExecuteDataset(connectionString, CommandType.StoredProcedure, spName);
            }
        }

        /// <summary>
        /// Execute a stored procedure via a DbCommand (that returns a resultset) against the specified DbConnection 
        /// using the dataRow column values as the store procedure's parameters values.
        /// This method will query the database to discover the parameters for the 
        /// stored procedure (the first time each stored procedure is called), and assign the values based on row values.
        /// </summary>
        /// <param name="connection">A valid DbConnection object</param>
        /// <param name="spName">The name of the stored procedure</param>
        /// <param name="dataRow">The dataRow used to hold the stored procedure's parameter values.</param>
        /// <returns>A dataset containing the resultset generated by the command</returns>
        public static DataSet ExecuteDatasetTypedParams(DbConnection connection, String spName, DataRow dataRow)
        {
            if (connection == null) throw new ArgumentNullException("connection");
            if (string.IsNullOrEmpty(spName)) throw new ArgumentNullException("spName");

            // If the row has values, the store procedure parameters must be initialized
            if (dataRow != null && dataRow.ItemArray.Length > 0)
            {
                // Pull the parameters for this stored procedure from the parameter cache (or discover them & populate the cache)
                var commandParameters = SqlHelperParameterCache.GetSpParameterSet(connection, spName);

                // Set the parameters values
                AssignParameterValues(commandParameters, dataRow);

                return DAAB.ExecuteDataset(connection, CommandType.StoredProcedure, spName, commandParameters);
            }
            else
            {
                return DAAB.ExecuteDataset(connection, CommandType.StoredProcedure, spName);
            }
        }

        /// <summary>
        /// Execute a stored procedure via a DbCommand (that returns a resultset) against the specified SqlTransaction 
        /// using the dataRow column values as the stored procedure's parameters values.
        /// This method will query the database to discover the parameters for the 
        /// stored procedure (the first time each stored procedure is called), and assign the values based on row values.
        /// </summary>
        /// <param name="transaction">A valid SqlTransaction object</param>
        /// <param name="spName">The name of the stored procedure</param>
        /// <param name="dataRow">The dataRow used to hold the stored procedure's parameter values.</param>
        /// <returns>A dataset containing the resultset generated by the command</returns>
        public static DataSet ExecuteDatasetTypedParams(SqlTransaction transaction, String spName, DataRow dataRow)
        {
            if (transaction == null) throw new ArgumentNullException("transaction");
            if (transaction != null && transaction.Connection == null) throw new ArgumentException("The transaction was rollbacked or commited, please provide an open transaction.", "transaction");
            if (string.IsNullOrEmpty(spName)) throw new ArgumentNullException("spName");

            // If the row has values, the store procedure parameters must be initialized
            if (dataRow != null && dataRow.ItemArray.Length > 0)
            {
                // Pull the parameters for this stored procedure from the parameter cache (or discover them & populate the cache)
                var commandParameters = SqlHelperParameterCache.GetSpParameterSet(transaction.Connection, spName);

                // Set the parameters values
                AssignParameterValues(commandParameters, dataRow);

                return DAAB.ExecuteDataset(transaction, CommandType.StoredProcedure, spName, commandParameters);
            }
            else
            {
                return DAAB.ExecuteDataset(transaction, CommandType.StoredProcedure, spName);
            }
        }

        #endregion

        #region ExecuteReaderTypedParams
        /// <summary>
        /// Execute a stored procedure via a DbCommand (that returns a resultset) against the database specified in 
        /// the connection string using the dataRow column values as the stored procedure's parameters values.
        /// This method will query the database to discover the parameters for the 
        /// stored procedure (the first time each stored procedure is called), and assign the values based on parameter order.
        /// </summary>
        /// <param name="connectionString">A valid connection string for a DbConnection</param>
        /// <param name="spName">The name of the stored procedure</param>
        /// <param name="dataRow">The dataRow used to hold the stored procedure's parameter values.</param>
        /// <returns>A SqlDataReader containing the resultset generated by the command</returns>
        public static DbDataReader ExecuteReaderTypedParams(String connectionString, String spName, DataRow dataRow)
        {
            if (string.IsNullOrEmpty(connectionString)) throw new ArgumentNullException("connectionString");
            if (string.IsNullOrEmpty(spName)) throw new ArgumentNullException("spName");

            // If the row has values, the store procedure parameters must be initialized
            if (dataRow != null && dataRow.ItemArray.Length > 0)
            {
                // Pull the parameters for this stored procedure from the parameter cache (or discover them & populate the cache)
                var commandParameters = SqlHelperParameterCache.GetSpParameterSet(connectionString, spName);

                // Set the parameters values
                AssignParameterValues(commandParameters, dataRow);

                return DAAB.ExecuteReader(connectionString, CommandType.StoredProcedure, spName, commandParameters);
            }
            else
            {
                return DAAB.ExecuteReader(connectionString, CommandType.StoredProcedure, spName);
            }
        }


        /// <summary>
        /// Execute a stored procedure via a DbCommand (that returns a resultset) against the specified DbConnection 
        /// using the dataRow column values as the stored procedure's parameters values.
        /// This method will query the database to discover the parameters for the 
        /// stored procedure (the first time each stored procedure is called), and assign the values based on parameter order.
        /// </summary>
        /// <param name="connection">A valid DbConnection object</param>
        /// <param name="spName">The name of the stored procedure</param>
        /// <param name="dataRow">The dataRow used to hold the stored procedure's parameter values.</param>
        /// <returns>A SqlDataReader containing the resultset generated by the command</returns>
        public static DbDataReader ExecuteReaderTypedParams(DbConnection connection, String spName, DataRow dataRow)
        {
            if (connection == null) throw new ArgumentNullException("connection");
            if (string.IsNullOrEmpty(spName)) throw new ArgumentNullException("spName");

            // If the row has values, the store procedure parameters must be initialized
            if (dataRow != null && dataRow.ItemArray.Length > 0)
            {
                // Pull the parameters for this stored procedure from the parameter cache (or discover them & populate the cache)
                var commandParameters = SqlHelperParameterCache.GetSpParameterSet(connection, spName);

                // Set the parameters values
                AssignParameterValues(commandParameters, dataRow);

                return DAAB.ExecuteReader(connection, CommandType.StoredProcedure, spName, commandParameters);
            }
            else
            {
                return DAAB.ExecuteReader(connection, CommandType.StoredProcedure, spName);
            }
        }

        /// <summary>
        /// Execute a stored procedure via a DbCommand (that returns a resultset) against the specified SqlTransaction 
        /// using the dataRow column values as the stored procedure's parameters values.
        /// This method will query the database to discover the parameters for the 
        /// stored procedure (the first time each stored procedure is called), and assign the values based on parameter order.
        /// </summary>
        /// <param name="transaction">A valid SqlTransaction object</param>
        /// <param name="spName">The name of the stored procedure</param>
        /// <param name="dataRow">The dataRow used to hold the stored procedure's parameter values.</param>
        /// <returns>A SqlDataReader containing the resultset generated by the command</returns>
        public static DbDataReader ExecuteReaderTypedParams(DbTransaction transaction, String spName, DataRow dataRow)
        {
            if (transaction == null) throw new ArgumentNullException("transaction");
            if (transaction != null && transaction.Connection == null) throw new ArgumentException("The transaction was rollbacked or commited, please provide an open transaction.", "transaction");
            if (string.IsNullOrEmpty(spName)) throw new ArgumentNullException("spName");

            // If the row has values, the store procedure parameters must be initialized
            if (dataRow != null && dataRow.ItemArray.Length > 0)
            {
                // Pull the parameters for this stored procedure from the parameter cache (or discover them & populate the cache)
                var commandParameters = SqlHelperParameterCache.GetSpParameterSet(transaction.Connection, spName);

                // Set the parameters values
                AssignParameterValues(commandParameters, dataRow);

                return DAAB.ExecuteReader(transaction, CommandType.StoredProcedure, spName, commandParameters);
            }
            else
            {
                return DAAB.ExecuteReader(transaction, CommandType.StoredProcedure, spName);
            }
        }
        #endregion

        #region ExecuteScalarTypedParams
        /// <summary>
        /// Execute a stored procedure via a DbCommand (that returns a 1x1 resultset) against the database specified in 
        /// the connection string using the dataRow column values as the stored procedure's parameters values.
        /// This method will query the database to discover the parameters for the 
        /// stored procedure (the first time each stored procedure is called), and assign the values based on parameter order.
        /// </summary>
        /// <param name="connectionString">A valid connection string for a DbConnection</param>
        /// <param name="spName">The name of the stored procedure</param>
        /// <param name="dataRow">The dataRow used to hold the stored procedure's parameter values.</param>
        /// <returns>An object containing the value in the 1x1 resultset generated by the command</returns>
        public static object ExecuteScalarTypedParams(String connectionString, String spName, DataRow dataRow)
        {
            if (string.IsNullOrEmpty(connectionString)) throw new ArgumentNullException("connectionString");
            if (string.IsNullOrEmpty(spName)) throw new ArgumentNullException("spName");

            // If the row has values, the store procedure parameters must be initialized
            if (dataRow != null && dataRow.ItemArray.Length > 0)
            {
                // Pull the parameters for this stored procedure from the parameter cache (or discover them & populate the cache)
                var commandParameters = SqlHelperParameterCache.GetSpParameterSet(connectionString, spName);

                // Set the parameters values
                AssignParameterValues(commandParameters, dataRow);

                return DAAB.ExecuteScalar(connectionString, CommandType.StoredProcedure, spName, commandParameters);
            }
            else
            {
                return DAAB.ExecuteScalar(connectionString, CommandType.StoredProcedure, spName);
            }
        }

        /// <summary>
        /// Execute a stored procedure via a DbCommand (that returns a 1x1 resultset) against the specified DbConnection 
        /// using the dataRow column values as the stored procedure's parameters values.
        /// This method will query the database to discover the parameters for the 
        /// stored procedure (the first time each stored procedure is called), and assign the values based on parameter order.
        /// </summary>
        /// <param name="connection">A valid DbConnection object</param>
        /// <param name="spName">The name of the stored procedure</param>
        /// <param name="dataRow">The dataRow used to hold the stored procedure's parameter values.</param>
        /// <returns>An object containing the value in the 1x1 resultset generated by the command</returns>
        public static object ExecuteScalarTypedParams(DbConnection connection, String spName, DataRow dataRow)
        {
            if (connection == null) throw new ArgumentNullException("connection");
            if (string.IsNullOrEmpty(spName)) throw new ArgumentNullException("spName");

            // If the row has values, the store procedure parameters must be initialized
            if (dataRow != null && dataRow.ItemArray.Length > 0)
            {
                // Pull the parameters for this stored procedure from the parameter cache (or discover them & populate the cache)
                var commandParameters = SqlHelperParameterCache.GetSpParameterSet(connection, spName);

                // Set the parameters values
                AssignParameterValues(commandParameters, dataRow);

                return DAAB.ExecuteScalar(connection, CommandType.StoredProcedure, spName, commandParameters);
            }
            else
            {
                return DAAB.ExecuteScalar(connection, CommandType.StoredProcedure, spName);
            }
        }

        /// <summary>
        /// Execute a stored procedure via a DbCommand (that returns a 1x1 resultset) against the specified SqlTransaction
        /// using the dataRow column values as the stored procedure's parameters values.
        /// This method will query the database to discover the parameters for the 
        /// stored procedure (the first time each stored procedure is called), and assign the values based on parameter order.
        /// </summary>
        /// <param name="transaction">A valid SqlTransaction object</param>
        /// <param name="spName">The name of the stored procedure</param>
        /// <param name="dataRow">The dataRow used to hold the stored procedure's parameter values.</param>
        /// <returns>An object containing the value in the 1x1 resultset generated by the command</returns>
        public static object ExecuteScalarTypedParams(SqlTransaction transaction, String spName, DataRow dataRow)
        {
            if (transaction == null) throw new ArgumentNullException("transaction");
            if (transaction != null && transaction.Connection == null) throw new ArgumentException("The transaction was rollbacked or commited, please provide an open transaction.", "transaction");
            if (string.IsNullOrEmpty(spName)) throw new ArgumentNullException("spName");

            // If the row has values, the store procedure parameters must be initialized
            if (dataRow != null && dataRow.ItemArray.Length > 0)
            {
                // Pull the parameters for this stored procedure from the parameter cache (or discover them & populate the cache)
                var commandParameters = SqlHelperParameterCache.GetSpParameterSet(transaction.Connection, spName);

                // Set the parameters values
                AssignParameterValues(commandParameters, dataRow);

                return DAAB.ExecuteScalar(transaction, CommandType.StoredProcedure, spName, commandParameters);
            }
            else
            {
                return DAAB.ExecuteScalar(transaction, CommandType.StoredProcedure, spName);
            }
        }
        #endregion

        #region ExecuteXmlReaderTypedParams - Commented Out
        ///// <summary>
        ///// Execute a stored procedure via a DbCommand (that returns a resultset) against the specified DbConnection 
        ///// using the dataRow column values as the stored procedure's parameters values.
        ///// This method will query the database to discover the parameters for the 
        ///// stored procedure (the first time each stored procedure is called), and assign the values based on parameter order.
        ///// </summary>
        ///// <param name="connection">A valid DbConnection object</param>
        ///// <param name="spName">The name of the stored procedure</param>
        ///// <param name="dataRow">The dataRow used to hold the stored procedure's parameter values.</param>
        ///// <returns>An XmlReader containing the resultset generated by the command</returns>
        //public static XmlReader ExecuteXmlReaderTypedParams(DbConnection connection, String spName, DataRow dataRow)
        //{
        //    if (connection == null) throw new ArgumentNullException("connection");
        //    if (string.IsNullOrEmpty(spName)) throw new ArgumentNullException("spName");

        //    // If the row has values, the store procedure parameters must be initialized
        //    if (dataRow != null && dataRow.ItemArray.Length > 0)
        //    {
        //        // Pull the parameters for this stored procedure from the parameter cache (or discover them & populate the cache)
        //        var commandParameters = SqlHelperParameterCache.GetSpParameterSet(connection, spName);

        //        // Set the parameters values
        //        AssignParameterValues(commandParameters, dataRow);

        //        return DAAB.ExecuteXmlReader(connection, CommandType.StoredProcedure, spName, commandParameters);
        //    }
        //    else
        //    {
        //        return DAAB.ExecuteXmlReader(connection, CommandType.StoredProcedure, spName);
        //    }
        //}

        ///// <summary>
        ///// Execute a stored procedure via a DbCommand (that returns a resultset) against the specified SqlTransaction 
        ///// using the dataRow column values as the stored procedure's parameters values.
        ///// This method will query the database to discover the parameters for the 
        ///// stored procedure (the first time each stored procedure is called), and assign the values based on parameter order.
        ///// </summary>
        ///// <param name="transaction">A valid SqlTransaction object</param>
        ///// <param name="spName">The name of the stored procedure</param>
        ///// <param name="dataRow">The dataRow used to hold the stored procedure's parameter values.</param>
        ///// <returns>An XmlReader containing the resultset generated by the command</returns>
        //public static XmlReader ExecuteXmlReaderTypedParams(SqlTransaction transaction, String spName, DataRow dataRow)
        //{
        //    if (transaction == null) throw new ArgumentNullException("transaction");
        //    if (transaction != null && transaction.Connection == null) throw new ArgumentException("The transaction was rollbacked or commited, please provide an open transaction.", "transaction");
        //    if (string.IsNullOrEmpty(spName)) throw new ArgumentNullException("spName");

        //    // If the row has values, the store procedure parameters must be initialized
        //    if (dataRow != null && dataRow.ItemArray.Length > 0)
        //    {
        //        // Pull the parameters for this stored procedure from the parameter cache (or discover them & populate the cache)
        //        var commandParameters = SqlHelperParameterCache.GetSpParameterSet(transaction.Connection, spName);

        //        // Set the parameters values
        //        AssignParameterValues(commandParameters, dataRow);

        //        return DAAB.ExecuteXmlReader(transaction, CommandType.StoredProcedure, spName, commandParameters);
        //    }
        //    else
        //    {
        //        return DAAB.ExecuteXmlReader(transaction, CommandType.StoredProcedure, spName);
        //    }
        //}
        #endregion

    }

    /// <summary>
    /// SqlHelperParameterCache provides functions to leverage a static cache of procedure parameters, and the
    /// ability to discover parameters for stored procedures at run-time.
    /// </summary>
    public static class SqlHelperParameterCache
    {
        #region private methods, variables, and constructors

        //Since this class provides only static methods, make the default constructor private to prevent 
        //instances from being created with "new SqlHelperParameterCache()"

        private static readonly Hashtable paramCache = Hashtable.Synchronized(new Hashtable());

        /// <summary>
        /// Resolve at run time the appropriate set of DbParameters for a stored procedure
        /// </summary>
        /// <param name="connection">A valid DbConnection object</param>
        /// <param name="spName">The name of the stored procedure</param>
        /// <param name="includeReturnValueParameter">Whether or not to include their return value parameter</param>
        /// <returns>The parameter array discovered.</returns>
        private static DbParameter[] DiscoverSpParameterSet(DbConnection connection, string spName, bool includeReturnValueParameter)
        {
            if (connection == null) throw new ArgumentNullException("connection");
            if (string.IsNullOrEmpty(spName)) throw new ArgumentNullException("spName");

            var cmd = DAAB.CreateCommand();
            cmd.CommandType = CommandType.StoredProcedure;
            cmd.Connection = connection;
            cmd.CommandText = spName;

            //DbCommand cmd = new DbCommand(spName, connection as DbConnection)
            //                     {
            //                         CommandType = CommandType.StoredProcedure
            //                     };

            connection.Open();

            DAAB.DeriveParameters(cmd);
            
            connection.Close();

            if (!includeReturnValueParameter)
            {
                cmd.Parameters.RemoveAt(0);
            }

            var discoveredParameters = DAAB.CreateParameterCollection(cmd.Parameters.Count);

            cmd.Parameters.CopyTo(discoveredParameters, 0);

            // Init the parameters with a DBNull value
            foreach (var discoveredParameter in discoveredParameters)
            {
                discoveredParameter.Value = DBNull.Value;
            }
            return discoveredParameters;
        }

        /// <summary>
        /// Deep copy of cached DbParameter array
        /// </summary>
        /// <param name="originalParameters"></param>
        /// <returns></returns>
        private static DbParameter[] CloneParameters(DbParameter[] originalParameters)
        {
            var clonedParameters = new DbParameter[originalParameters.Length];

            for (int i = 0, j = originalParameters.Length; i < j; i++)
            {
                clonedParameters[i] = (DbParameter)((ICloneable)originalParameters[i]).Clone();
            }

            return clonedParameters;
        }

        #endregion private methods, variables, and constructors

        #region caching functions

        /// <summary>
        /// Add parameter array to the cache
        /// </summary>
        /// <param name="connectionString">A valid connection string for a DbConnection</param>
        /// <param name="commandText">The stored procedure name or T-SQL command</param>
        /// <param name="commandParameters">An array of SqlParamters to be cached</param>
        public static void CacheParameterSet(string connectionString, string commandText, params DbParameter[] commandParameters)
        {
            if (string.IsNullOrEmpty(connectionString)) throw new ArgumentNullException("connectionString");
            if (string.IsNullOrEmpty(commandText)) throw new ArgumentNullException("commandText");

            var hashKey = connectionString + ":" + commandText;

            paramCache[hashKey] = commandParameters;
        }

        /// <summary>
        /// Retrieve a parameter array from the cache
        /// </summary>
        /// <param name="connectionString">A valid connection string for a DbConnection</param>
        /// <param name="commandText">The stored procedure name or T-SQL command</param>
        /// <returns>An array of SqlParamters</returns>
        public static DbParameter[] GetCachedParameterSet(string connectionString, string commandText)
        {
            if (string.IsNullOrEmpty(connectionString)) throw new ArgumentNullException("connectionString");
            if (string.IsNullOrEmpty(commandText)) throw new ArgumentNullException("commandText");

            var hashKey = connectionString + ":" + commandText;

            var cachedParameters = paramCache[hashKey] as DbParameter[];
            return cachedParameters == null ? null : CloneParameters(cachedParameters);
        }

        #endregion caching functions

        #region Parameter Discovery Functions

        /// <summary>
        /// Retrieves the set of DbParameters appropriate for the stored procedure
        /// </summary>
        /// <remarks>
        /// This method will query the database for this information, and then store it in a cache for future requests.
        /// </remarks>
        /// <param name="connectionString">A valid connection string for a DbConnection</param>
        /// <param name="spName">The name of the stored procedure</param>
        /// <returns>An array of DbParameters</returns>
        public static DbParameter[] GetSpParameterSet(string connectionString, string spName)
        {
            return GetSpParameterSet(connectionString, spName, false);
        }

        /// <summary>
        /// Retrieves the set of DbParameters appropriate for the stored procedure
        /// </summary>
        /// <remarks>
        /// This method will query the database for this information, and then store it in a cache for future requests.
        /// </remarks>
        /// <param name="connectionString">A valid connection string for a DbConnection</param>
        /// <param name="spName">The name of the stored procedure</param>
        /// <param name="includeReturnValueParameter">A bool value indicating whether the return value parameter should be included in the results</param>
        /// <returns>An array of DbParameters</returns>
        public static DbParameter[] GetSpParameterSet(string connectionString, string spName, bool includeReturnValueParameter)
        {
            if (string.IsNullOrEmpty(connectionString)) throw new ArgumentNullException("connectionString");
            if (string.IsNullOrEmpty(spName)) throw new ArgumentNullException("spName");

            using (var connection = DAAB.CreateConnection(connectionString))
            {
                return GetSpParameterSetInternal(connection, spName, includeReturnValueParameter);
            }
        }

        /// <summary>
        /// Retrieves the set of DbParameters appropriate for the stored procedure
        /// </summary>
        /// <remarks>
        /// This method will query the database for this information, and then store it in a cache for future requests.
        /// </remarks>
        /// <param name="connection">A valid DbConnection object</param>
        /// <param name="spName">The name of the stored procedure</param>
        /// <returns>An array of DbParameters</returns>
        internal static DbParameter[] GetSpParameterSet(DbConnection connection, string spName)
        {
            return GetSpParameterSet(connection, spName, false);
        }

        /// <summary>
        /// Retrieves the set of DbParameters appropriate for the stored procedure
        /// </summary>
        /// <remarks>
        /// This method will query the database for this information, and then store it in a cache for future requests.
        /// </remarks>
        /// <param name="connection">A valid DbConnection object</param>
        /// <param name="spName">The name of the stored procedure</param>
        /// <param name="includeReturnValueParameter">A bool value indicating whether the return value parameter should be included in the results</param>
        /// <returns>An array of DbParameters</returns>
        internal static DbParameter[] GetSpParameterSet(DbConnection connection, string spName, bool includeReturnValueParameter)
        {
            if (connection == null) throw new ArgumentNullException("connection");
            using (var clonedConnection = (DbConnection)((ICloneable)connection).Clone())
            {
                return GetSpParameterSetInternal(clonedConnection, spName, includeReturnValueParameter);
            }
        }

        /// <summary>
        /// Retrieves the set of DbParameters appropriate for the stored procedure
        /// </summary>
        /// <param name="connection">A valid DbConnection object</param>
        /// <param name="spName">The name of the stored procedure</param>
        /// <param name="includeReturnValueParameter">A bool value indicating whether the return value parameter should be included in the results</param>
        /// <returns>An array of DbParameters</returns>
        private static DbParameter[] GetSpParameterSetInternal(DbConnection connection, string spName, bool includeReturnValueParameter)
        {
            if (connection == null) throw new ArgumentNullException("connection");
            if (string.IsNullOrEmpty(spName)) throw new ArgumentNullException("spName");

            var hashKey = connection.ConnectionString + ":" + spName + (includeReturnValueParameter ? ":include ReturnValue Parameter" : "");

            var cachedParameters = paramCache[hashKey] as DbParameter[];
            if (cachedParameters == null)
            {
                var spParameters = DiscoverSpParameterSet(connection, spName, includeReturnValueParameter);
                paramCache[hashKey] = spParameters;
                cachedParameters = spParameters;
            }

            return CloneParameters(cachedParameters);
        }

        #endregion Parameter Discovery Functions

    }

    #endregion




}
