﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Web;

namespace BigfootWeb.Helpers
{

    /// <summary>
    /// This class contains a list of well known html tag attributes
    /// </summary>
    public class Attributes
    {
        public const string @class = "class";
        public const string id = "id";
        public const string name = "name";
        public const string alt = "alt";
        public const string @for = "for";
        public const string href = "href";
        public const string src = "src";
        public const string style = "style";
        public const string type = "type";
        public const string value = "value";
        public const string onclick = "onclick";
        public const string onblur = "onblur";
        public const string onfocus = "onfocus";
        public const string onmouseover = "onmouseover";
        public const string onmouseout = "onmouseout";
        public const string cols = "cols";
        public const string rows = "rows";
    }

    /// <summary>
    /// List of input types
    /// </summary>
    public class InputTypes
    {
        public const string button = "button";
        public const string checkbox = "checkbox";
        public const string file = "file";
        public const string hidden = "hidden";
        public const string image = "image";
        public const string password = "password";
        public const string radio = "radio";
        public const string reset = "reset";
        public const string submit = "submit";
        public const string text = "text";
    }

    /// <summary>
    /// Html tag render mode enumeration
    /// </summary>
    public enum RenderMode
    {
        /// <summary>
        /// Renders using a corresponding </tag>
        /// </summary>
        Normal,
        /// <summary>
        /// It is a selfclosing tag. Like for inputs
        /// </summary>
        SelfClose,
        /// <summary>
        /// Does not ouput any tag closing. Useful when ouputing the beginning tag of a form but you don't want to close it.
        /// </summary>
        openTagOnly
    }

    /// <summary>
    /// The tag builder class is the heart of the html builder. It is used to create any html tag
    /// using a fluid interface. It has a ton of convinience methods to aid in the bulding of tags.
    /// It goes beyond the mere building of tags though and also adds javscript here and there to create
    /// interactivity like rollover effects, rollover effects with sprites etc.
    /// 
    /// It also relies on the metadata method for adding jquery.validation to input elements.
    /// 
    /// The tagbuilder class also understands the concept of input and label combinations to aid
    /// in minimizing the amount of code you have to write to create the very common label input validation 
    /// combination
    /// </summary>
    public class TagBuilder
    {

        string TagName;
        Dictionary<string, string> TagAttributes = new Dictionary<string, string>();
        public RenderMode? Mode = new RenderMode?();

        public TagBuilder Parent;
        /// <summary>
        /// Returns the last added child. If you want to access all children use the children collection instead
        /// </summary>
        public TagBuilder Child;

        public List<TagBuilder> Children = new List<TagBuilder>();
        private string _innerText = "";
        private string _innerHtml = "";
        private string _label = "";
        private string _labelclass;

        private string _labelWidth;

        /// <summary>
        /// Creates a new TagBuilder by specifying the name of the tag you want to create i.e. h1
        /// </summary>
        /// <param name="tagName">The html tag name to use i.e. h1</param>
        public TagBuilder(string tagName)
        {
            TagName = tagName;
        }


        /// <summary>
        /// Creates a new TagBuilder by specifying the name of the tag you want to create i.e. h1
        /// </summary>
        /// <param name="tagName">The html tag name to use i.e. h1</param>
        /// <param name="parent">Create a tag as an embeded child of another tag for hiarchical tag ouput</param>
        public TagBuilder(string tagName, TagBuilder parent)
        {
            TagName = tagName;
            Parent = parent;
        }

        /// <summary>
        /// Creates a new TagBuilder by specifying the name of the tag you want to create i.e. h1
        /// </summary>
        /// <param name="tagName">The html tag name to use i.e. h1</param>
        /// <param name="mode">Specifies the render mode for the. SelfClosing etc.</param>
        public TagBuilder(string tagName, RenderMode mode)
        {
            TagName = tagName;
            Mode = mode;
        }

        /// <summary>
        /// Creates a new TagBuilder by specifying the name of the tag you want to create, id and name to use and the class name to assign to it
        /// </summary>
        /// <param name="tagName">The html tag name to use i.e. h1</param>
        /// <param name="idAndName">The ID and Name to assign</param>
        /// <param name="className">The className to assign to the task</param>
        public TagBuilder(string tagName, string idAndName, string className)
        {
            TagName = tagName;
            AddAttribute(Attributes.id, idAndName);
            AddAttribute(Attributes.name, idAndName);
            AddAttribute("class", className);
        }

        /// <summary>
        /// Adds a child to the children tags collection and returns the newaly added child
        /// </summary>
        /// <param name="tagname">The tag name for the child to add</param>
        /// <returns>The added child. Use the Parent property to get back to parent builder</returns>
        public TagBuilder AddChild(string tagname)
        {
            return AddChild(new TagBuilder(tagname));
        }

        /// <summary>
        /// Adds a child TagBuilder to the children tags collection and returns the newaly added child. 
        /// This is useful when calling render on the parent to make sure that it renders all of it's 
        /// children as well
        /// </summary>
        /// <param name="child">The TagBuilder object to add</param>
        /// <returns>The added child. Use the Parent property to get back to parent builder</returns>
        public TagBuilder AddChild(TagBuilder child)
        {
            child.Parent = this;
            Children.Add(child);
            return child;
        }

        /// <summary>
        /// Adds a class to the element only if a certain condition is met
        /// </summary>
        /// <param name="condition">Condition to use</param>
        /// <param name="classname">Class name to add</param>
        public TagBuilder AddClassIf(bool condition, string classname)
        {
            return AddClassIf(condition, classname, string.Empty);
        }

        /// <summary>
        /// Adds a class to the element only if a certain condition is met. If it adds another class
        /// </summary>
        /// <param name="condition">Condition to use</param>
        /// <param name="classname">Class name to add when condition is true</param>
        /// <param name="otherclassname">Class name to add when condition is false</param>
        public TagBuilder AddClassIf(bool condition, string classname, string otherclassname)
        {
            if (condition)
            {
                AddClass(classname);
            }
            else if (string.IsNullOrEmpty(otherclassname) == false)
            {
                AddClass(otherclassname);
            }
            return this;
        }

        /// <summary>
        /// Adds a class(es) to the tag
        /// </summary>
        /// <param name="classes">List of tags to add</param>
        public TagBuilder AddClass(params string[] classes)
        {
            foreach (var c in classes)
            {
                MergeAttribute(Attributes.@class, c);
            }
            return this;
        }

        /// <summary>
        /// Adds a style attribute if not present to the tag and specifies the width in pixes
        /// </summary>
        /// <param name="value">Width in pixes</param>
        public TagBuilder width(int value)
        {
            return width(value.ToString(), value.ToString());
        }

        /// <summary>
        /// Adds a style attribute if not present to the tag and specifies the width in pixes for both the tag and its label
        /// </summary>
        /// <param name="inputWidth">Width in pixels for the input</param>
        /// <param name="labelWidth">Width in pixels for the label</param>
        public TagBuilder width(int inputWidth, int labelWidth)
        {
            return width(inputWidth.ToString(), labelWidth.ToString());
        }

        /// <summary>
        /// Adds a style attribute if not present to the tag and specifies the width
        /// </summary>
        /// <param name="value">Width value</param>
        public TagBuilder width(string value)
        {
            return width(value, value);
        }

        /// <summary>
        /// Adds a style attribute if not present to the tag and specifies the width for both the tag and its label
        /// </summary>
        /// <param name="inputWidth">Width for the input</param>
        /// <param name="labelWidth">Width for the label</param>
        public TagBuilder width(string inputWidth, string labelWidth)
        {
            AppendPixels(ref inputWidth);
            AppendPixels(ref labelWidth);
            _labelWidth = labelWidth;
            return MergeAttribute("style", "width: " + inputWidth);
        }

        private void AppendPixels(ref string value)
        {
            if (value.Trim().EndsWith("%") == false && value.Trim().EndsWith("px") == false)
            {
                value += "px";
            }
        }

        /// <summary>
        /// Adds a style attribute if not present to the tag and specifies the width for the label
        /// </summary>
        /// <param name="value">Width value</param>
        public TagBuilder labelwidth(string value)
        {
            if (value.Trim().EndsWith("%") == false && value.Trim().EndsWith("px") == false)
            {
                value += "px";
            }
            _labelWidth = value;
            return this;
        }

        /// <summary>
        /// Adds a style attribute if not present to the tag and specifies the height for the element
        /// </summary>
        /// <param name="value">Height value</param>
        public TagBuilder height(string value)
        {
            return MergeAttribute("style", "height: " + value);
        }

        /// <summary>
        /// Adds a style attribute if not present to the tag and specifies the width and height for the tag
        /// </summary>
        /// <param name="width">Width value in pixels</param>
        /// <param name="height">Height value in pixels</param>
        public TagBuilder size(int width, int height)
        {
            return addStyle(string.Format("width: {0}px; height: {1}px", width, height));
        }

        /// <summary>
        /// Adds a style attribute if not present to the tag and specifies the font size for the tag
        /// </summary>
        /// <param name="pixels">Font-size in pixels</param>
        public TagBuilder fontsize(int pixels)
        {
            return MergeAttribute("style", "font-size: " + pixels + "px");
        }

        /// <summary>
        /// Adds a style attribute if not present to the tag and specifies the font size for the tag
        /// </summary>
        /// <param name="value">Font-size value</param>
        public TagBuilder fontsize(string value)
        {
            return MergeAttribute("style", "font-size: " + value);
        }

        /// <summary>
        /// Adds a style attribute if not present to the tag and specifies the font-weight for the tag as bold
        /// </summary>
        public TagBuilder bold()
        {
            return MergeAttribute("style", "font-weight: bold");
        }

        /// <summary>
        /// Adds a style attribute if not present to the tag and specifies the padding-top for the tag
        /// </summary>
        /// <param name="pixels">Padding-top in pixels</param>
        public TagBuilder padTop(int pixels)
        {
            return addStyle(string.Format("padding-top: {0}px", pixels));
        }

        /// <summary>
        /// Adds a style attribute if not present to the tag and specifies the padding-left for the tag
        /// </summary>
        /// <param name="pixels">Padding-left in pixels</param>
        public TagBuilder padLeft(int pixels)
        {
            return addStyle(string.Format("padding-left: {0}px", pixels));
        }

        /// <summary>
        /// Adds a style attribute if not present to the tag and specifies the padding-right for the tag
        /// </summary>
        /// <param name="pixels">Padding-right in pixels</param>
        public TagBuilder padRight(int pixels)
        {
            return addStyle(string.Format("padding-right: {0}px", pixels));
        }

        /// <summary>
        /// Adds a style attribute if not present to the tag and specifies the padding-bottom for the tag
        /// </summary>
        /// <param name="pixels">Padding-bottom in pixels</param>
        public TagBuilder padBottom(int pixels)
        {
            return addStyle(string.Format("padding-bottom: {0}px", pixels));
        }

        /// <summary>
        /// Adds a style attribute if not present to the tag and specifies the padding for the tag in pixes
        /// </summary>
        public TagBuilder pad(int topPx, int rightPx, int bottomPx, int leftPix)
        {
            return addStyle(string.Format("padding: {0}px {1}px {2}px {3}px", topPx, rightPx, bottomPx, leftPix));
        }

        /// <summary>
        /// Adds a style attribute if not present to the tag and specifies the margin-top for the tag
        /// </summary>
        /// <param name="pixels">Margin-top in pixels</param>
        public TagBuilder marginTop(int pixels)
        {
            return addStyle(string.Format("margin-top: {0}px", pixels));
        }

        /// <summary>
        /// Adds a style attribute if not present to the tag and specifies the margin-left for the tag
        /// </summary>
        /// <param name="pixels">Margin-left in pixels</param>
        public TagBuilder marginLeft(int pixels)
        {
            return addStyle(string.Format("margin-left: {0}px", pixels));
        }

        /// <summary>
        /// Adds a style attribute if not present to the tag and specifies the margin-right for the tag
        /// </summary>
        /// <param name="pixels">Margin-right in pixels</param>
        public TagBuilder marginRight(int pixels)
        {
            return addStyle(string.Format("margin-right: {0}px", pixels));
        }

        /// <summary>
        /// Adds a style attribute if not present to the tag and specifies the margin-bottom for the tag
        /// </summary>
        /// <param name="pixels">Margin-bottom in pixels</param>
        public TagBuilder marginBottom(int pixels)
        {
            return addStyle(string.Format("margin-bottom: {0}px", pixels));
        }

        /// <summary>
        /// Adds a style attribute if not present to the tag and specifies the margin for the tag in pixels
        /// </summary>
        public TagBuilder margin(int topPx, int rightPx, int bottomPx, int leftPix)
        {
            return addStyle(string.Format("margin: {0}px {1}px {2}px {3}px", topPx, rightPx, bottomPx, leftPix));
        }

        /// <summary>
        /// Adds a style attribute if not present to the tag and specifies the display as inline-block
        /// </summary>
        public TagBuilder inlineblock()
        {
            return addStyle("display: inline-block");
        }

        /// <summary>
        /// Adds a style attribute if not present to the tag and specifies the text-decoration as none
        /// </summary>
        public TagBuilder noDecoration()
        {
            return MergeAttribute("style", "text-decoration: none");
        }

        /// <summary>
        /// Adds a style attribute if not present to the tag and specifies the color
        /// </summary>
        public TagBuilder color(string value)
        {
            return MergeAttribute("style", "color: " + value);
        }

        /// <summary>
        /// Adds the method attribte to the tag. Used in the form tag
        /// </summary>
        /// <param name="value">post, get etc.</param>
        public TagBuilder method(string value)
        {
            return AddAttribute("method", value);
        }

        /// <summary>
        /// Adds the multipart attribute to a form element
        /// </summary>
        public TagBuilder multipart()
        {
            return AddAttribute("enctype", "multipart/form-data");
        }

        /// <summary>
        /// Sets the action on a form element
        /// </summary>
        /// <param name="url">The url to submit the form to</param>
        public TagBuilder action(string url)
        {
            return AddAttribute("action", url);
        }

        /// <summary>
        /// Sets the alt tag on an image element
        /// </summary>
        /// <param name="value">The alt text</param>
        public TagBuilder Alt(string value)
        {
            return AddAttribute(Attributes.alt, value);
        }

        /// <summary>
        /// Sets the InnerText value of the element. I.E. TextArea inner text
        /// </summary>
        public TagBuilder text(string value)
        {
            return InnerText(value);
        }

        /// <summary>
        /// Sets the InnerText value of the element. I.E. TextArea inner text
        /// </summary>
        public TagBuilder InnerText(string value)
        {
            _innerText = value;
            return this;
        }

        /// <summary>
        /// Sets the InnerHtml value of the element
        /// </summary>
        public TagBuilder html(string value)
        {
            return InnerHtml(value);
        }

        /// <summary>
        /// Sets the InnerHtml value of the element
        /// </summary>
        public TagBuilder InnerHtml(string value)
        {
            _innerHtml = value;
            return this;
        }

        /// <summary>
        /// Sets the id attribute of the element
        /// </summary>
        public TagBuilder ID(string value)
        {
            return AddAttribute("id", value);
        }

        /// <summary>
        /// Sets the name attribute of the element
        /// </summary>
        public TagBuilder Name(string value)
        {
            return AddAttribute("name", value);
        }

        /// <summary>
        /// Sets the id and name attribute of the element
        /// </summary>
        public TagBuilder IDandName(string value)
        {
            ID(value);
            Name(value);
            return this;
        }

        /// <summary>
        /// Sets the for attribute of the label element
        /// </summary>
        public TagBuilder @for(string name)
        {
            return AddAttribute(Attributes.@for, name);
        }

        /// <summary>
        /// Sets the href attribute for an anchor tag
        /// </summary>
        public TagBuilder href(string value)
        {
            return AddAttribute(Attributes.href, value);
        }

        /// <summary>
        /// Sets the type attribute of input element
        /// </summary>
        public TagBuilder type(string inputtype)
        {
            return AddAttribute(Attributes.type, inputtype);
        }

        /// <summary>
        /// Sets the value attribute of an input element
        /// </summary>
        public TagBuilder value(object val)
        {
            return TagName.ToLower() == "textarea" ? InnerText(GetValue(val)) : AddAttribute(Attributes.value, GetValue(val));
        }

        /// <summary>
        /// Cleans up the value to add to the html
        /// </summary>
        private string GetValue(object val)
        {
            if (val == null)
            {
                return "";
            }
            if (ReferenceEquals(val.GetType(), typeof(int)) && (int)val == 0)
            {
                return "";
            }
            if (ReferenceEquals(val.GetType(), typeof(decimal)) && (decimal)val == 0)
            {
                return "";
            }
            if (ReferenceEquals(val.GetType(), typeof(double)) && (double)val == 0)
            {
                return "";
            }
            if (ReferenceEquals(val.GetType(), typeof(string)) && string.IsNullOrEmpty(Convert.ToString(val)))
            {
                return "";
            }
            if (ReferenceEquals(val.GetType(), typeof(int?)) && ((int?)val).HasValue == false)
            {
                return "";
            }
            if (ReferenceEquals(val.GetType(), typeof(decimal?)) && ((decimal?)val).HasValue == false)
            {
                return "";
            }
            if (ReferenceEquals(val.GetType(), typeof(double?)) && ((double?)val).HasValue == false)
            {
                return "";
            }

            // if a match was not found then shove the tostring in there
            return val.ToString();
        }

        /// <summary>
        /// Add an onclick event to the tag
        /// </summary>
        /// <param name="value">The javascript script either inline or function call</param>
        public TagBuilder onclick(string value)
        {
            return AddAttribute(Attributes.onclick, value);
        }

        /// <summary>
        /// Adds javascript to the tag to cancel the enter element
        /// </summary>
        public TagBuilder cancelenter()
        {
            return AddAttribute("onkeypress", "if (event.keyCode == 13) { event.cancelBubble = true; return false; }");
        }

        /// <summary>
        /// Add an onchange event to the tag
        /// </summary>
        /// <param name="value">The javascript script either inline or function call</param>
        public TagBuilder onchange(string value)
        {
            return AddAttribute("onchange", value);
        }

        /// <summary>
        /// Add an onmouseover event to the tag
        /// </summary>
        /// <param name="value">The javascript script either inline or function call</param>
        public TagBuilder onmouseover(string value)
        {
            return AddAttribute(Attributes.onmouseover, value);
        }

        /// <summary>
        /// Add an onmouseout event to the tag
        /// </summary>
        /// <param name="value">The javascript script either inline or function call</param>
        public TagBuilder onmouseout(string value)
        {
            return AddAttribute(Attributes.onmouseout, value);
        }

        /// <summary>
        /// Add the column count to a textarea tag
        /// </summary>
        /// <param name="value">Cols count</param>
        public TagBuilder cols(int value)
        {
            return AddAttribute(Attributes.cols, value.ToString());
        }

        /// <summary>
        /// Add the rows count to a textarea tag
        /// </summary>
        /// <param name="value">Rows count</param>
        public TagBuilder rows(int value)
        {
            return AddAttribute(Attributes.rows, value.ToString());
        }

        /// <summary>
        /// Sets the style attribute for the lement. Replaces any other styles previously added. 
        /// To progressively add styles you can use the addStyle method instead
        /// </summary>
        public TagBuilder style(string value)
        {
            return AddAttribute(Attributes.style, value);
        }

        /// <summary>
        /// Adds the style attribute to the class if not present and appends the current style to it following the correct syntax
        /// </summary>
        /// <param name="value">The style to add i.e. color: white</param>
        public TagBuilder addStyle(string value)
        {
            return MergeAttribute(Attributes.style, value);
        }
        
        /// <summary>
        /// Adds the title attribute the the class. Used for links
        /// </summary>
        public TagBuilder title(string value)
        {
            return AddAttribute("title", value);
        }

        /// <summary>
        /// Adds a label to the current tag. Useful when adding an input and you want to also add the label for it
        /// </summary>
        /// <param name="value">The text for the label</param>
        public TagBuilder Label(string value)
        {
            _label = value;
            return this;
        }

        /// <summary>
        /// Adds a label to the current tag. Useful when adding an input and you want to also add the label for it
        /// </summary>
        /// <param name="value">The text for the label</param>
        /// <param name="cssclass">Adds this class to the label</param>
        public TagBuilder Label(string value, string cssclass)
        {
            _label = value;
            _labelclass = cssclass;
            return this;
        }

        /// <summary>
        /// When creating a select list or a radio list etc. this adds a collection of ListItems as the InnerHtml of the 
        /// tag as option elements
        /// </summary>
        public TagBuilder options(SelectItem[] items)
        {
            return options(items, null);
        }

        /// <summary>
        /// When creating a select list or a radio list etc. this adds a collection of ListItems as the InnerHtml of the 
        /// tag as option elements. You also specify the value of the selected item in the list.
        /// </summary>
        public TagBuilder options(SelectItem[] items, object value)
        {
            this.value(value);
            return InnerHtml(GetOptionsString(items, value.ToString()));
        }

        /// <summary>
        /// Adds the checked attribute to the element. Used with checkbox elements.
        /// </summary>
        public TagBuilder @checked(bool value)
        {
            return value ? AddAttribute("checked", "checked") : RemoveAttribute("checked");
        }

        /// <summary>
        /// Adds the checked attribute to the element. Used with input elements.
        /// </summary>
        public TagBuilder @readonly()
        {
            return AddAttribute("ReadOnly", "readonly").AddClass("readonlyField");
        }

        /// <summary>
        /// Adds the src attribute to the element. Used with image elements.
        /// </summary>
        public TagBuilder src(string url)
        {
            return AddAttribute(Attributes.src, url);
        }

        /// <summary>
        /// Adds the target attribute to the element and sets to blank. Used with anchor elements.
        /// </summary>
        public TagBuilder Target_Blank()
        {
            return AddAttribute("target", "_blank");
        }

        /// <summary>
        /// This background to the element using a sprite image
        /// </summary>
        /// <param name="img">The sprite image</param>
        /// <param name="bgposition">The x y position of the image to be used in the sprite</param>
        /// <param name="width">The width of the image inside the sprite. sets the width of the element to this</param>
        /// <param name="height">The height of the image inside the sprite. sets the height of the element to this</param>
        public TagBuilder sprite(string img, string bgposition, int width, int height)
        {
            return spriteRollover(img, bgposition, width, height, string.Empty);
        }

        /// <summary>
        /// This adds a rollever effect to the element using a sprite image
        /// </summary>
        /// <param name="img">The sprite image</param>
        /// <param name="bgposition">The x y position of the image to be used in the sprite</param>
        /// <param name="width">The width of the image inside the sprite. sets the width of the element to this</param>
        /// <param name="height">The height of the image inside the sprite. sets the height of the element to this</param>
        /// <param name="rolloverposition">The x y position of the image to be used in the sprite for the rollover. Must be the same width and height as the initial image</param>
        public TagBuilder spriteRollover(string img, string bgposition, int width, int height, string rolloverposition)
        {
            addStyle(string.Format("background-image:url({0}); background-position: {1}; width: {2}px; height: {3}px;", img, bgposition, width, height));
            if (string.IsNullOrEmpty(rolloverposition) == false)
            {
                onmouseover("jQuery(this).css('background-position','" + rolloverposition + "');");
                onmouseout("jQuery(this).css('background-position','" + bgposition + "');");
            }
            return this;
        }

        /// <summary>
        /// Adds a style attribute if not present to the tag and specifies the position as absolute top left using pixels
        /// </summary>
        public TagBuilder absolute(int top, int left)
        {
            return addStyle(string.Format("position: absolute; top: {0}px; left: {1}px", top, left));
        }

        /// <summary>
        /// Adds a style attribute if not present to the tag and specifies the position as absolute top right using pixels
        /// </summary>
        public TagBuilder absoluteTR(int top, int right)
        {
            return addStyle(string.Format("position: absolute; top: {0}px; right: {1}px", top, right));
        }

        /// <summary>
        /// Adds a style attribute if not present to the tag and specifies the position as absolute bottom right using pixels
        /// </summary>
        public TagBuilder absoluteBR(int bottom, int right)
        {
            return addStyle(string.Format("position: absolute; bottom: {0}px; right: {1}px", bottom, right));
        }

        /// <summary>
        /// Adds a style attribute if not present to the tag and specifies the position as absolute bottom left using pixels
        /// </summary>
        public TagBuilder absoluteBL(int bottom, int left)
        {
            return addStyle(string.Format("position: absolute; bottom: {0}px; left: {1}px", bottom, left));
        }

        /// <summary>
        /// Adds a class to the element named causesValidation. This is used by the JSHelper to idenfity when to fire the validation routines.
        /// Usually added to links, buttons and other actionable elements to kick start validation when clicked
        /// </summary>
        public TagBuilder causesValidation()
        {
            return AddClass("causesValidation");
        }

        /// <summary>
        /// Adds validation to an input control. Makes the input required
        /// </summary>
        public TagBuilder ValRequired()
        {
            return AddClass("required");
        }

        /// <summary>
        /// Adds validation to an input control. Makes the input validate agains a remote url. The remore url must return true when valid
        /// </summary>
        public TagBuilder ValRemote(string remoteUrl)
        {
            return AddAttribute("remote", remoteUrl);
        }

        /// <summary>
        /// Adds validation to an input control. Makes the input value have a min length
        /// </summary>
        public TagBuilder ValMinLength(int minLength)
        {
            return AddAttribute("minlength", minLength.ToString());
        }

        /// <summary>
        /// Adds validation to an input control. Makes the input value have a max length
        /// </summary>
        public TagBuilder ValMaxLength(int maxLength)
        {
            return AddAttribute("maxlength", maxLength.ToString());
        }

        /// <summary>
        /// Adds validation to an input control. Makes the input value length be between two numbers
        /// </summary>
        public TagBuilder ValRangeLength(int RangeStart, int RangeEnd)
        {
            return AddAttribute("rangelength", string.Format("[{0},{1}]", RangeStart, RangeEnd));
        }

        /// <summary>
        /// Adds validation to an input control. Makes the input value me no less than than the min specified
        /// </summary>
        public TagBuilder ValMin(int min)
        {
            return AddAttribute("min", min.ToString());
        }

        /// <summary>
        /// Adds validation to an input control. Makes the input value not be greater than the max specified
        /// </summary>
        public TagBuilder ValMax(int max)
        {
            return AddAttribute("max", max.ToString());
        }

        /// <summary>
        /// Adds validation to an input control. Makes the input value be between two numbers
        /// </summary>
        public TagBuilder ValRange(int start, int end)
        {
            return AddAttribute("range", string.Format("[{0},{1}]", start, end));
        }

        /// <summary>
        /// Adds validation to an input control. Makes the input be a valid email address
        /// </summary>
        public TagBuilder ValEmail()
        {
            return AddClass("email");
        }

        /// <summary>
        /// Adds validation to an input control. Makes the input be a valid url address
        /// </summary>
        public TagBuilder ValUrl()
        {
            return AddClass("url");
        }

        /// <summary>
        /// Adds validation to an input control. Makes the input be a valid date
        /// </summary>
        public TagBuilder ValDate()
        {
            return AddClass("date");
        }

        /// <summary>
        /// Adds validation to an input control. Makes the input be a valid number
        /// </summary>
        public TagBuilder ValNumber()
        {
            return AddClass("number");
        }

        /// <summary>
        /// Adds validation to an input control. Makes the input be a valid credit card
        /// </summary>
        public TagBuilder ValCreditCard()
        {
            return AddClass("creditcard");
        }

        /// <summary>
        /// Adds validation to an input control. Makes the input be a valid digits only
        /// </summary>
        public TagBuilder ValDigits()
        {
            return AddClass("digits");
        }

        /// <summary>
        /// Adds validation to a file input control. Only allows certain extensions for files to upload
        /// </summary>
        public TagBuilder ValAcceptExt(params string[] extensions)
        {
            var extString = "";
            foreach (var e in extensions)
            {
                if (extString.Length > 0)
                    extString += "|";
                extString += e;
            }
            return AddAttribute("accept", extString);
        }

        /// <summary>
        /// Adds validation to an input control. Makes the input value be the same as the value of another input element
        /// </summary>
        public TagBuilder ValEqualTo(string fieldname)
        {
            if (fieldname.StartsWith("#") == false)
            {
                fieldname = "#" + fieldname;
            }
            return AddAttribute("equalTo", fieldname);
        }

        /// <summary>
        /// Adds validation to an input control. Makes the input be a valid password with a minimum length of 7 and incudes at least 1 non-alpha numeric character
        /// </summary>
        public TagBuilder ValPassword()
        {
            return ValRegExp("^.*(?=.{7,})((?=.*\\d)(?=.*[a-z])|(?=.*\\d)(?=.*[a-z])(?=.*[^a-zA-Z0-9])|(?=.*\\d)(?=.*[^a-zA-Z0-9])|(?=.*[a-z])(?=.*[^a-zA-Z0-9])).*$", "Password must be at least 7 characters, must contain a letter and a number, and must also contain one or more non-alphanumeric character.");
        }

        /// <summary>
        /// Adds validation to an input control. Makes the input value match the specified regular expression
        /// </summary>
        public TagBuilder ValRegExp(string expression, string message)
        {
            AddAttribute("regExp", expression);
            return ValMessage("regExp", message);
        }

        private readonly Dictionary<string, string> _valMessages = new Dictionary<string, string>();
        /// <summary>
        /// Adds validation messages to be used for validators assiged to this element
        /// </summary>
        /// <param name="validator">This is the validator i.e. required</param>
        /// <param name="message">This is the error message to display</param>
        public TagBuilder ValMessage(string validator, string message)
        {
            _valMessages.Add(validator, message);
            return this;
        }

        /// <summary>
        /// Adds an attribute to the element
        /// </summary>
        /// <param name="key">the name of the attribute i.e. href</param>
        /// <param name="value">the value to assign to the attribute i.e. http://google.com</param>
        public TagBuilder attr(string key, int value)
        {
            return AddAttribute(key, value.ToString());
        }

        /// <summary>
        /// Adds an attribute to the element
        /// </summary>
        /// <param name="key">the name of the attribute i.e. href</param>
        /// <param name="value">the value to assign to the attribute i.e. http://google.com</param>
        public TagBuilder attr(string key, string value)
        {
            return AddAttribute(key, value);
        }

        /// <summary>
        /// Adds an attribute to the element
        /// </summary>
        /// <param name="key">the name of the attribute i.e. href</param>
        /// <param name="value">the value to assign to the attribute i.e. http://google.com</param>
        public TagBuilder AddAttribute(string key, string value)
        {
            if (TagAttributes.ContainsKey(key))
            {
                TagAttributes[key] = value;
            }
            else
            {
                TagAttributes.Add(key, value);
            }
            return this;
        }

        /// <summary>
        /// Removes an attribute from the element
        /// </summary>
        /// <param name="key">the name of the attribute to remove i.e. href</param>
        public TagBuilder RemoveAttribute(string key)
        {
            if (TagAttributes.ContainsKey(key))
            {
                TagAttributes.Remove(key);
            }
            return this;
        }

        /// <summary>
        /// Adds an attribute to the element if the attribute has not already been added. If it has then it merges the contents 
        /// of the attribute by appending this value. In the case of the style attribute it knows to to properly merge
        /// styles using the right css syntax
        /// </summary>
        /// <param name="key">the name of the attribute style</param>
        /// <param name="value">the value to assign to the attribute color=white</param>
        public TagBuilder MergeAttribute(string key, string value)
        {

            if (TagAttributes.ContainsKey(key))
            {
                // Merge style attributes with a ;
                if (key == Attributes.style)
                {
                    var currentStyle = GetAttribute(Attributes.style).Trim();

                    if (currentStyle.Length > 0)
                    {
                        if (currentStyle.EndsWith(";") == false)
                            currentStyle += "; ";
                        currentStyle += value;
                    }
                    else
                    {
                        currentStyle = value;
                    }

                    TagAttributes[key] = currentStyle;

                    // add a space for all other merging operations
                }
                else
                {
                    TagAttributes[key] += " " + value;
                }
            }
            else
            {
                TagAttributes.Add(key, value);
            }
            return this;
        }

        /// <summary>
        /// Returns the value of the specified attribute
        /// </summary>
        /// <param name="attrName">The name of the attribute to retreive</param>
        /// <returns>The value of the attribute</returns>
        public string GetAttribute(string attrName)
        {
            return TagAttributes.ContainsKey(attrName) == false ? string.Empty : TagAttributes[attrName];
        }

        /// <summary>
        /// Format all the added attributes into a properly formatted string
        /// </summary>
        private string GetAttributeString()
        {
            const string attributeFormat = " {0}=\"{1}\"";
            var value = "";
            foreach (var key in TagAttributes.Keys)
            {
                value += string.Format(attributeFormat, key, TagAttributes[key]);
            }
            return value;
        }

        /// <summary>
        /// Generates the label string (only when specified) to use for the element
        /// </summary>
        /// <returns></returns>
        private string GetLabelString()
        {
            if (string.IsNullOrEmpty(_label)) return "";

            var lbl = new TagBuilder("label");
            if (string.IsNullOrEmpty(GetAttribute("name")) == false)
            {
                lbl.@for(GetAttribute("name"));
            }
            lbl.AddClass(string.IsNullOrEmpty(_labelclass) == false ? _labelclass : "fieldlabel");
            if (string.IsNullOrEmpty(_labelWidth) == false)
            {
                lbl.width(_labelWidth);
            }
            lbl.InnerText(_label);
            return "\r\n" + lbl + "\r\n";
        }

        /// <summary>
        /// Generates the html for all the options added to the element to be included as innerHtml
        /// </summary>
        private string GetOptionsString(IEnumerable<SelectItem> items, string val)
        {
            const string optionFormat = " <option value=\"{0}\" {2}>{1}</option>";
            const string selectedString = " selected=\"selected\"";
            var sb = new StringBuilder();
            var valueFound = false;
            foreach (var item in items)
            {
                var selected = false;
                if (item.Selected)
                {
                    selected = true;
                    valueFound = true;
                }
                else if (string.IsNullOrEmpty(val) == false && item.Value.ToUpper() == val.ToUpper())
                {
                    selected = true;
                    valueFound = true;
                }
                sb.AppendLine(string.Format(optionFormat, item.Value, item.Text, (selected ? selectedString : "")));
            }

            // If the value was not found in the options then add it.
            if (string.IsNullOrEmpty(val) == false && valueFound == false)
            {
                sb.AppendLine(string.Format(optionFormat, val, val, selectedString));
            }

            return sb.ToString();
        }

        /// <summary>
        /// Returns the element properly formated as html code. It also recursively renders all child elements
        /// </summary>
        public override string ToString()
        {
            var renderMode = Mode.HasValue ? Mode.Value : RenderMode.Normal;
            return ToString(renderMode);
        }

        /// <summary>
        /// Returns the element properly formated as html code. It also recursively renders all child elements.
        /// Here you spcify in the maner you want the tag to render (Self closing, no closing tag, or normal)
        /// </summary>
        public string ToString(RenderMode mode)
        {
            const string normalTagFormat = "<{0}{1}>{2}</{0}>";
            const string selfCloseTagFormat = "<{0}{1} />";
            const string openTagFormat = "<{0}{1}>{2}";

            // Create the validation messages string
            var valData = "";
            foreach (string m in _valMessages.Keys)
            {
                if (valData.Length > 0)
                    valData += ", ";
                valData += m + ": '" + _valMessages[m] + "'";
            }
            if (valData.Length > 0)
                AddClass("{messages: {" + valData + "}}");

            // Create the inner value
            var innerValue = (string.IsNullOrEmpty(_innerHtml) ? HttpUtility.HtmlEncode(_innerText) : _innerHtml);

            // Render the children into the inner html of the element
            foreach (var c in Children)
            {
                innerValue += c.ToString();
            }

            switch (TagName.ToLower())
            {
                case "input":
                    mode = RenderMode.SelfClose;
                    break;
                case "img":
                    mode = RenderMode.SelfClose;
                    break;
            }

            // Create the tag string
            string tagstring;
            if (mode == RenderMode.SelfClose && string.IsNullOrEmpty(innerValue))
            {
                tagstring = string.Format(selfCloseTagFormat, TagName, GetAttributeString());
            }
            else if (mode == RenderMode.openTagOnly)
            {
                tagstring = string.Format(openTagFormat, TagName, GetAttributeString(), innerValue);
            }
            else
            {
                tagstring = string.Format(normalTagFormat, TagName, GetAttributeString(), innerValue);
            }

            // Return the label (if found) and tag string
            return GetLabelString() + tagstring;
        }

    }
    
}
