﻿using System;
using System.IO;
using System.Net.Mail;
using System.Reflection;
using System.Security.Cryptography;
using System.Text;
using System.Web;
using System.Collections.Generic;
using System.Web.Security;
using System.Web.UI;
using BigfootWeb.Email;
using BigfootWeb.Helpers;

namespace BigfootWeb
{
    /// <summary>
    /// This is code derived from the opensource project BigfootMVC
    /// </summary>
    public class WebApp
    {
        /// <summary>
        /// Initializes the WebApp by specifying the root path
        /// </summary>
        /// <param name="appPath">The relative root path i.e. ~/ or ~/desktopmodules/module/</param>
        public WebApp(string appPath)
        {
            AppPath = appPath;
            ContextProvider = new ASPContext();
            EmailServer = new ASPEmail();
        }

        /// <summary>
        /// Initializes the WebApp by specifying the root path as well as the email and context provider. Particularly useful when testing
        /// </summary>
        /// <param name="appPath">The relative root path i.e. ~/ or ~/desktopmodules/module/</param>
        /// <param name="context">This is the the IContext for the current instance of this application</param>
        /// <param name="emailProvider">This is the email provider for this instance of the application</param>
        public WebApp(string appPath, IContext context, IEmailProvider emailProvider)
        {
            AppPath = appPath;
            ContextProvider = context;
            EmailServer = emailProvider;
        }

        /// <summary>
        /// This is the root virtual path of where your applications files reside. This is the point from where 
        /// all content paths start. Example if you have all the content folder in ~/app/content/ then
        /// the AppPath would be ~/app/ . Always includes the last forward slash
        /// </summary>
        public string AppPath = "~/";
        /// <summary>
        /// This is the data path of your applcation. This is the Data folder inside your application root
        /// </summary>
        public string DataPath { get { return AppPath + "Data/"; } }
        /// <summary>
        /// This is the path where the views of your applcation reside. They are in the Views folder inside your application root
        /// </summary>
        public string ViewsPath { get { return AppPath + "Views/"; } }
        /// <summary>
        /// This is the path where the content of your applcation reside like images, css, javascript, flash, and all other static files. 
        /// They are in the content folder folder inside your application root
        /// </summary>
        public string ContentPath { get { return AppPath + "content/"; } }
        /// <summary>
        /// This is the path to all your javascript files. AppPath/ContentPath/js/
        /// </summary>
        public string ScriptsPath { get { return AppPath + "content/js/"; } }
        /// <summary>
        /// This is the path to all your image files. AppPath/ContentPath/images/
        /// </summary>
        public string ImagesPath { get { return AppPath + "content/images/"; } }
        /// <summary>
        /// This is the path to all your CSS files. AppPath/ContentPath/css/
        /// </summary>
        public string CssPath { get { return AppPath + "content/css/"; } }
        /// <summary>
        /// This is the path to all your flash files. AppPath/ContentPath/FlashPath/
        /// </summary>
        public string FlashPath { get { return AppPath + "content/flash/"; } }
        /// <summary>
        /// This is the path to all your audio files. AppPath/ContentPath/AudioPath/
        /// </summary>
        public string AudioPath { get { return AppPath + "content/audio/"; } }

        ///// <summary>
        ///// This is the path to the global error view user control
        ///// </summary>
        //public static string ErrorViewPath { get { return AppPath + "error.ascx"; } }

        /// <summary>
        /// Flag that determines that the application is in testing mode
        /// </summary>
        public bool InTestingMode { get; set; }

        /// <summary>
        /// This is the email provider to be used by the application
        /// </summary>
        public IEmailProvider EmailServer;

        /// <summary>
        /// This provides the application functions with context information. 
        /// For testing you can simply overwrite the context provider with your own mocked 
        /// objects and all the functionality is re-routed to your own testing mock
        /// </summary>
        public IContext ContextProvider;
        
        /// <summary>
        /// Access to the HttpRequest contained within the ContextProvider. Aliased here for convinience
        /// </summary>
        public HttpRequest Request { get { return ContextProvider.GetRequest(); } }

        /// <summary>
        /// Access to the HttpResponse contained within the ContextProvider. Aliased here for convinience
        /// </summary>
        public HttpResponse Response { get { return ContextProvider.GetResponse(); } }

        /// <summary>
        /// Access to the HttpContext contained within the ContextProvider. Aliased here for convinience
        /// </summary>
        public HttpContext Context { get { return ContextProvider.GetContext(); } }

        /// <summary>
        /// Access to the HttpServerUtility contained within the ContextProvider. Aliased here for convinience
        /// </summary>
        public HttpServerUtility Server { get { return ContextProvider.GetServer(); } }

        /// <summary>
        /// The name of the currently logged in principle
        /// </summary>
        public string UserName { get { return (Request.IsAuthenticated) ? Context.User.Identity.Name : ""; } }

        /// <summary>
        /// Determines weather we are in debug mode at compile time
        /// </summary>
        public static bool IsInDebugMode { get {
            #if (DEBUG)
                return true;
            #else
                return false;
            #endif
        } }
        
        #region "Cache"

        /// <summary>
        /// This is a global application cache that persists accross requests while the application lives. Please takenote that it is server specific
        /// </summary>
        public static class Cache
        {
            static Dictionary<string, object> _cache = new Dictionary<string, object>();

            /// <summary>
            /// Determine weather a certain key is contained in the cache
            /// </summary>
            /// <param name="key">The key to check for</param>
            /// <returns>True if found</returns>
            public static bool Contains(String key) { return _cache.ContainsKey(key); }

            /// <summary>
            /// Returns the value stored in the cache
            /// </summary>
            /// <param name="key">The parameter key to the value</param>
            /// <returns>The cached value</returns>
            public static object GetValue(String key)
            {
                return (_cache.ContainsKey(key)) ? _cache[key] : null;
            }

            /// <summary>
            /// Adds a value to the cache. Replaces any excisting value if found
            /// </summary>
            /// <param name="key">The key of the value to add</param>
            /// <param name="data">The data to add to the cache</param>
            public static void Add(String key, object data)
            {
                if (_cache.ContainsKey(key))
                    _cache[key] = data;
                else
                    _cache.Add(key, data);
            }

            /// <summary>
            /// Removes a value from the cache safely. It does not throw an error if the value is not found.
            /// </summary>
            /// <param name="key">The key to the cahced value</param>
            public static void Remove(string key)
            {
                if (_cache.ContainsKey(key)) _cache.Remove(key);
            }
        }
        #endregion
        
        #region "Context.Items cache"

        public object ContextData(string key)
        {
            return Context.Items.Contains(key) ? Context.Items[key] : null;
        }

        public void ContextData(string key, object value)
        {
            if (Context.Items.Contains(key))
                Context.Items[key] = value;
            else
                Context.Items.Add(key, value);
        }

        #endregion

        /// <summary>
        /// Determines weather the current request is an ajax request
        /// </summary>
        public bool IsAjaxRequest
        {
            get
            {
                if (InTestingMode) return false;
                return Request.Headers["X-Requested-With"] == "XMLHttpRequest";
            }
        }

        /// <summary>
        /// Gets a reference to the HTML Helper
        /// </summary>
        public HtmlHelper Html
        {
            get { return _html ?? (_html = HtmlHelper.Create()); }
        }
        private HtmlHelper _html;

        /// <summary>
        /// Gets a reference to the JS Helper
        /// </summary>
        public JSHelper JS
        {
            get { return _js ?? (_js = JSHelper.Create()); }
        }
        private JSHelper _js;

        #region "Path Functions"

        /// <summary>
        /// Gets a browser usable path to a content file within the system. It uses the extention of the file to determine the location of the static file
        /// within the content folder.
        /// i.e. "image.jpg" will return "/desktopmodules/modulename/content/images/image.jpg"
        /// i.e. "menu/left.jpg" will return "/desktopmodules/modulename/content/images/menu/left.jpg"
        /// i.e. "test.js" will return "/desktopmodules/modulename/content/js/test.js"
        /// </summary>
        /// <param name="fileName">The name of the file relative to the content folder i.e. left.jpg or menu/left.jpg</param>
        /// <returns>A url that may be used from the browser to access the static resource</returns>
        public string StaticUrl(string fileName)
        {
            return StaticUrl(fileName, false);
        }

        /// <summary>
        /// Gets a browser usable path to a content file within the system. It uses the extention of the file to determine the location of the static file
        /// within the content folder.
        /// i.e. "image.jpg" will return "/desktopmodules/modulename/content/images/image.jpg"
        /// i.e. "menu/left.jpg" will return "/desktopmodules/modulename/content/images/menu/left.jpg"
        /// i.e. "test.js" will return "/desktopmodules/modulename/content/js/test.js"
        /// </summary>
        /// <param name="fileName">The name of the file relative to the content folder i.e. left.jpg or menu/left.jpg</param>
        /// <param name="includeDomain">Determines weather to include the domain name in the url. (i.e. http://domain.com/desktopmodules/modulename/content/image.jpg)</param>
        /// <returns>A url that may be used from the browser</returns>
        public string StaticUrl(string fileName, bool includeDomain)
        {
            return GetAbsoluteUrl(GetRelativeUrl(fileName), includeDomain);
        }

        /// <summary>
        /// Adds the authentication token to a url. Provided that you are doing cookieless authentication
        /// </summary>
        /// <param name="url">The url to use</param>
        /// <returns>Returns the modified url</returns>
        public string AddAuthTokenToUrl(string url)
        {
            return Response.ApplyAppPathModifier(url);
        }

        /// <summary>
        /// Retreives the relative url of a file based on its extention
        /// </summary>
        /// <param name="filename">filename: test.jpg returns ~/content/images/test.jpg</param>
        /// <returns>The server side relative url to the file</returns>
        public string GetRelativeUrl(string filename)
        {
            filename = filename.ToLower();
            if (filename.EndsWith("js"))
                return ScriptsPath + filename;
            else if (filename.EndsWith("css"))
                return CssPath + filename;
            else if (filename.EndsWith("jpg") || filename.EndsWith("gif") || filename.EndsWith("png"))
                return ImagesPath + filename;
            else if (filename.EndsWith("flv") || filename.EndsWith("swf"))
                return FlashPath + filename;
            else
                return filename;
        }

        /// <summary>
        /// Returns the server root application path. /mycampus/ or / when at the root of the domain. This is the path to the application after the domain name
        /// </summary>
        /// <returns>Returns / or /appname/ or /apps/appname/ etc. What is after the domain name</returns>
        public string GetApplicationPath() { return GetApplicationPath(false); }

        /// <summary>
        /// Root application path. /app/ or / when at the root of the domain. 
        /// </summary>
        /// <param name="includeDomain">Determines weather the domain name is included</param>
        /// <returns>Returns the full application path root including the domain name if asked.</returns>
        public string GetApplicationPath(bool includeDomain)
        {
            var apppath = Request.ApplicationPath;
            if (apppath.EndsWith("/") == false) apppath += "/";

            if (includeDomain)
            {
                var host = GetApplicationHost;
                // Remove the host last "/"
                host = host.Substring(0, host.Length - 1);
                // Combine the application path and the host
                apppath = host + apppath;
            }
            return apppath;
        }

        
        /// <summary>
        /// Gets the application host name of the current request. Returns something like this http://applicationhost.com:4454 if it is port 80 then
        /// the port number is not used
        /// </summary>
        public string GetApplicationHost
        {
            get
            {
                var port = Request.ServerVariables["SERVER_PORT"];
                if (String.IsNullOrEmpty(port) || port == "80" || port == "443")
                    port = "";
                else
                    port = ":" + port;

                var protocol = Request.ServerVariables["SERVER_PORT_SECURE"];
                if (string.IsNullOrEmpty(protocol) || protocol == "0")
                    protocol = "http://";
                else
                    protocol = "https://";

                var domain = Request.ServerVariables["SERVER_NAME"];

                return protocol + domain + port + "/";
            }
        }

        /// <summary>
        /// Returns the physical path to a relative url. 
        /// For example url: ~/content/media/xyz.ppt returns c:\inetpub\wwwroot\appdomain.com\internal\content\media\xyz.ppt
        /// </summary>
        /// <param name="url">The relative application path i.e. ~/content/media/xyz.ppt</param>
        /// <returns>Full file system path i.e. c:\inetpub\wwwroot\appdomain.com\internal\content\media\xyz.ppt</returns>
        public string GetPhysicalPath(string url)
        {
            return Context.Server.MapPath(url);
        }

        /// <summary>
        /// Returns the absolute path to a url in the appliation. 
        /// For example url: ~/content/media/xyz.ppt returns /applicationpath/content/media/xyz.ppt
        /// </summary>
        /// <param name="url">The relative application path i.e. ~/content/media/xyz.ppt</param>
        /// <returns>Returns abosolute path to the url without the domain name. i.e. /applicationpath/content/media/xyz.ppt</returns>
        public string GetAbsoluteUrl(string url) { return GetAbsoluteUrl(url, false); }

        /// <summary>
        /// Returns the absolute path to a url in the appliation. 
        /// For example url: ~/content/media/xyz.ppt returns /applicationpath/content/media/xyz.ppt or http://applicationhsot/applicationpath/content/media/xyz.ppt
        /// </summary>
        /// <param name="url">The relative application path i.e. ~/content/media/xyz.ppt</param>
        /// <param name="includeFullPath">Determines weather to include the domain name</param>
        /// <returns>Returns abosolute path to the url without the domain name. i.e. /applicationpath/content/media/xyz.ppt
        /// or http://applicationhsot/applicationpath/content/media/xyz.ppt</returns>
        public string GetAbsoluteUrl(string url, bool includeFullPath)
        {
            var apppath = GetApplicationPath(includeFullPath);
            if (url.StartsWith("~/"))
            {
                url = url.Substring(2);
            }
            if (url.StartsWith("/"))
                url = url.Substring(1);
            return apppath + url;
        }
        #endregion

        #region "Authentication"
        /// <summary>
        /// Gets the session token to be included in the url for sessionless authentication
        /// </summary>
        /// <returns>The token to be used in the url</returns>
        public string GetUrlAuthToken()
        {

            if (InTestingMode) return "TESTTOKEN";
            // Inserts the url token by calling this function
            var token = Response.ApplyAppPathModifier("~/");
            // The token uses the following format: /(F(*******)/
            return token.Substring(1, token.Substring(1).IndexOf("/"));
        }

        /// <summary>
        /// Authenticates the user agains the membership provider setup in the web.config
        /// </summary>
        /// <param name="userName">The name of the user</param>
        /// <param name="password">The password</param>
        /// <returns>True if correctly authenticated</returns>
        public bool AuthenticateUser(string userName, string password)
        {
            var status = Membership.Provider.ValidateUser(userName, password);
            // Set the authentication cookie
            if (status && InTestingMode == false) FormsAuthentication.SetAuthCookie(userName, true);
            return status;
        }

        #endregion

        #region "Encryption"

        /// <summary>
        /// Encrypts a value with the specified key using DES encryption
        /// </summary>
        /// <param name="key">The key to use in the encryption</param>
        /// <param name="data">The data to encrypt</param>
        /// <returns>Base64 encrypted string</returns>
        public static string Encrypt(string key, string data)
        {
            // don't do anything if the key is empty
            if (string.IsNullOrEmpty(key)) return data;

            // normalize the key to 16 characters
            if (key.Length < 16)
                key = key + "XXXXXXXXXXXXXXXX".Substring(0, 16 - key.Length);
            else
                key = key.Substring(0, 16);

            // create encryption key
            var bytekey = Encoding.UTF8.GetBytes(key.Substring(0, 8));
            var bytevector = Encoding.UTF8.GetBytes(key.Substring(8));

            // Convert the data
            var bytedata = Encoding.UTF8.GetBytes(data);

            // Encrypt
            var des = new DESCryptoServiceProvider();
            var memstream = new MemoryStream();
            var cryptostream = new CryptoStream(memstream, des.CreateEncryptor(bytekey, bytevector),
                                                CryptoStreamMode.Write);
            cryptostream.Write(bytedata, 0, bytedata.Length);
            cryptostream.FlushFinalBlock();

            // covert to string and base64 encode
            return Convert.ToBase64String(memstream.ToArray());
        }

        /// <summary>
        /// Decrypts a value with the specified key using DES encryption
        /// </summary>
        /// <param name="key">The key to use in the decryption</param>
        /// <param name="data">The data to decrypt base64 encoded</param>
        /// <returns>Decripted string</returns>
        public static string Decrypt(string key, string data)
        {
            // don't do anything if the key is empty
            if (string.IsNullOrEmpty(key)) return data;

            // normalize the key to 16 characters
            if (key.Length < 16)
                key = key + "XXXXXXXXXXXXXXXX".Substring(0, 16 - key.Length);
            else
                key = key.Substring(0, 16);

            // create encryption key
            var bytekey = Encoding.UTF8.GetBytes(key.Substring(0, 8));
            var bytevector = Encoding.UTF8.GetBytes(key.Substring(8));

            byte[] bytedata;
            try
            {
                bytedata = Convert.FromBase64String(data);
            }
            // return the data unchanged if there is some sort of error converting
            catch (Exception)
            {
                return data;
            }

            // Run the decryption
            try
            {
                var des = new DESCryptoServiceProvider();
                var ms = new MemoryStream();
                var cs = new CryptoStream(ms, des.CreateDecryptor(bytekey, bytevector), CryptoStreamMode.Write);
                cs.Write(bytedata, 0, bytedata.Length);
                cs.FlushFinalBlock();

                return Encoding.UTF8.GetString(ms.ToArray());
            }
            catch (Exception)
            {
                return "";
            }
        }

        #endregion

        #region "GetValue"

        /// <summary>
        /// Class used to aid in the collection of posted values from the request. First it tries the QueryString and second it tries the form collection
        /// </summary>
        public PostHelper Values
        {
            get { return _values ?? (_values = new PostHelper(Request, ContextProvider.ManualValues, InTestingMode)); }
        }
        private PostHelper _values;
        
        ///// <summary>
        ///// Gets a value from the request. First it tries the QueryString and second it tries the form collection
        ///// </summary>
        ///// <param name="key">The parameter to get</param>
        ///// <returns>The value of the parameter</returns>
        //public string RequestValue(string key)
        //{

        //    return Values.RequestValue(key);
        //}

        ///// <summary>
        ///// Requires that a certain parameter be present in the request
        ///// </summary>
        ///// <param name="name">paramater name</param>
        ///// <returns>The parameter value</returns>
        //public string GetValueOrError(string name)
        //{
        //    return GetValueOrError(name, false);
        //}

        ///// <summary>
        ///// Requires that a certain parameter be present in the request
        ///// </summary>
        ///// <param name="name">paramater name</param>
        ///// <returns>Convers the parameter value to an int</returns>
        //public int GetValueOrErrorAsInt(string name)
        //{
        //    RequireValue(name);
        //    return GetInt(name).Value;
        //}

        ///// <summary>
        ///// Requires that a certain parameter be present in the request
        ///// </summary>
        ///// <param name="name">paramater name</param>
        ///// <returns>Convers the parameter value to a double</returns>
        //public double GetValueOrErrorAsDouble(string name)
        //{
        //    RequireValue(name);
        //    return GetDouble(name).Value;
        //}

        ///// <summary>
        ///// Requires that a certain parameter be present in the request
        ///// </summary>
        ///// <param name="name">paramater name</param>
        ///// <returns>Convers the parameter value to a date</returns>
        //public DateTime GetValueOrErrorAsDate(string name)
        //{
        //    RequireValue(name);
        //    return GetDate(name).Value;
        //}

        ///// <summary>
        ///// Requires that a certain parameter be present in the request
        ///// </summary>
        ///// <param name="name">paramater name</param>
        ///// <param name="decode">indicates weather you want to UrlDecode the parameter</param>
        ///// <returns>The parameter value</returns>
        //public string GetValueOrError(string name, bool decode)
        //{
        //    string value;
        //    if (string.IsNullOrEmpty(RequestValue(name)))
        //        throw new ApplicationException("Expected value not found: " + name);
        //    value = RequestValue(name);
        //    if (decode) value = HttpUtility.UrlDecode(value);
        //    return value;
        //}

        ///// <summary>
        ///// Validates that a certain parameter be present is present in the request. Throws an error otherwise
        ///// </summary>
        ///// <param name="name">name of the parameter</param>
        //public void RequireValue(string name)
        //{
        //    if (string.IsNullOrEmpty(RequestValue(name)))
        //        throw new ApplicationException("Expected value not found: " + name);
        //}

        ///// <summary>
        ///// Returns the value for a certain parameter (QueryString or Form)
        ///// </summary>
        ///// <param name="key">parameter name</param>
        ///// <returns>value</returns>
        //public string GetValue(string key)
        //{
        //    return RequestValue(key);
        //}

        ///// <summary>
        ///// Returns a value for a certain parameter typed as an integer
        ///// </summary>
        //public int? GetInt(string key)
        //{
        //    var result = new int?();
        //    if (RequestValue(key) != null)
        //        result = int.Parse(GetValue(key));
        //    return result;
        //}

        ///// <summary>
        ///// Returns a value for a certain parameter typed as a Decimal
        ///// </summary>
        //public decimal? GetDecimal(string key)
        //{
        //    var result = new decimal?();
        //    if (RequestValue(key) != null)
        //        result = decimal.Parse(GetValue(key));
        //    return result;
        //}

        ///// <summary>
        ///// Returns a value for a certain parameter typed as an double
        ///// </summary>
        //public double? GetDouble(string key)
        //{
        //    var result = new double?();
        //    if (RequestValue(key) != null)
        //        result = double.Parse(GetValue(key));
        //    return result;
        //}

        ///// <summary>
        ///// Returns a value for a certain parameter typed as an date 
        ///// </summary>
        //public DateTime? GetDate(string key)
        //{
        //    var result = new DateTime?();
        //    DateTime value;
        //    if (RequestValue(key) != null && DateTime.TryParse(GetValue(key), out value))
        //    {
        //        result = value;
        //    }
        //    return result;
        //}

        ///// <summary>
        ///// Returns a value for a certain parameter typed as a bool. Interprets: true | yes | on | 1   AS True
        ///// </summary>
        //public bool GetBool(string key)
        //{
        //    return GetValue(key) != null &&
        //                (GetValue(key).ToLower() == "true" ||
        //                 GetValue(key).ToLower() == "yes" ||
        //                 GetValue(key).ToLower() == "on" ||
        //                 GetValue(key).ToLower() == "1");
        //}

        ///// <summary>
        ///// Returns a list of integers from 
        ///// </summary>
        ///// <param name="data">The string of ints i.e. "1,2,3,4"</param>
        ///// <returns></returns>
        //public List<int> GetIntArray(string data)
        //{
        //    var result = new List<int>();
        //    if (RequestValue(data) != null)
        //    {
        //        var value = RequestValue(data);
        //        // Identify the devider
        //        var devider = ',';
        //        if (value.Contains(";")) { devider = ';'; }
        //        else if (value.Contains("|")) { devider = '|'; }
        //        // Strip the first and last characters if they are deviders
        //        if (value.StartsWith(devider.ToString())) value = value.Substring(1);
        //        if (value.EndsWith(devider.ToString())) value = value.Substring(0, value.Length - 2);

        //        // Split the string and add to the list
        //        foreach (var v in value.Split(devider))
        //        {
        //            result.Add(int.Parse(v));
        //        }
        //    }
        //    return result;
        //}

        #endregion

        #region "Email"

        /// <summary>
        /// Sends an email using the email provider
        /// </summary>
        /// <param name="from">Who is the email from</param>
        /// <param name="to">Who is it going to. Can include multiple address separated by comma</param>
        /// <param name="subject">The subject line</param>
        /// <param name="body">The body of the html</param>
        /// <param name="isHtml">True if the body is using HTML</param>
        public void SendEmail(string from, string to, string subject, string body, bool isHtml)
        {
            var msg = new MailMessage(from, to, subject, body) { IsBodyHtml = isHtml };
            SendEmail(msg);
        }

        /// <summary>
        /// Sends an email using the email provider
        /// </summary>
        /// <param name="msg">The email message to send</param>
        public void SendEmail(MailMessage msg)
        {
            if (InTestingMode)
            {
                WebApp.Cache.Add("EmailSent", msg);
                return;
            }
            EmailServer.SendEmail(msg);
        }
        #endregion

        #region "Render"

        public void RenderViewToResponse(string viewRelativePath, object data)
        {
            WebApp.RenderViewToResponse(Server, Response, viewRelativePath, data, null, null);
        }

        public void RenderViewToResponse(string viewRelativePath, object data, string loadField1Name, object loadField1Data)
        {
            WebApp.RenderViewToResponse(Server, Response, viewRelativePath, data, loadField1Name, loadField1Data);
        }

        public static void RenderViewToResponse(HttpServerUtility server, HttpResponse response, string viewRelativePath, object data, string loadField1Name, object loadField1Data)
        {
            // Render the page to the output stream of the response provided
            server.Execute(RenderViewToPage(viewRelativePath, data, loadField1Name, loadField1Data), response.Output, true);
        }

        public string RenderViewToString(string viewRelativePath, object data)
        {
            return WebApp.RenderViewToString(Server, viewRelativePath, data, null, null);
        }

        public string RenderViewToString(string viewRelativePath, object data, string loadField1Name, object loadField1Data)
        {
            return WebApp.RenderViewToString(Server, viewRelativePath, data, loadField1Name, loadField1Data);
        }

        public static string RenderViewToString(HttpServerUtility server, string viewRelativePath, object data, string loadField1Name, object loadField1Data)
        {
            // Render the page
            var output = new StringWriter();

            // Render the page to the output stream
            server.Execute(RenderViewToPage(viewRelativePath, data, loadField1Name, loadField1Data), output, true);

            // Return the rendered output
            return output.ToString();
        }

        private static Page RenderViewToPage(string viewRelativePath, object data, string loadField1Name, object loadField1Data)
        {
            // Load the control
            var pageHolder = new Page();
            var viewControl = (UserControl)pageHolder.LoadControl(viewRelativePath);

            // Get the type of the control
            var t = viewControl.GetType();

            // Set the model data
            SetField(viewControl, t, "ModelData", data);

            // Flag the render view as being partially rendered
            SetField(viewControl, t, "IsRenderPartial", true);
            
            // Add additional data if specified
            SetField(viewControl, t, loadField1Name, loadField1Data);

            // Load the control on the page
            pageHolder.Controls.Add(viewControl);

            return pageHolder;
        }

        private static void SetField(object control, Type t, string fieldName, object data)
        {
            // Add additional data if specified
            if (string.IsNullOrEmpty(fieldName) || data == null) return;
            var field = t.GetField(fieldName);
            if (field != null)
            {
                field.SetValue(control, data);
            }
            else // Check if it is a property
            {
                var prop = t.GetProperty(fieldName);
                if (prop != null)
                    prop.SetValue(control, data, BindingFlags.SetProperty, null, null, null);
            }
        }

        public static UserControl LoadView(string viewPath, object data)
        {
            // Load the control
            var pageHolder = new Page();
            return LoadView(viewPath, data, pageHolder);
        }

        public static UserControl LoadView(string viewPath, object data, Page currentPage)
        {
            return LoadView(viewPath, data, currentPage, null, null);
        }

        public static UserControl LoadView(string viewPath, object data, Page currentPage, string loadField1Name, object loadField1Data)
        {
            var ctl = (UserControl)currentPage.LoadControl(viewPath);

            // Get the type of the control
            var t = ctl.GetType();

            // Set the model data
            SetField(ctl, t, "ModelData", data);

            // Flag the render view as being partially rendered
            SetField(ctl, t, "IsRenderPartial", true);

            // Add additional data if specified
            SetField(ctl, t, loadField1Name, loadField1Data);
            return (UserControl)ctl;
        }

        public void RenderJson(string data)
        {
            WebApp.RenderJson(Response, data);
        }

        public static void RenderJson(HttpResponse response, string data)
        {
            response.Clear();
            WebApp.SetNoCache(response);
            response.ContentType = "application/json";
            response.Write(data);
            response.End();
        }

        public void RenderLiteral(string data)
        {
            WebApp.RenderLiteral(Response, data);
        }

        public static void RenderLiteral(HttpResponse response, string data)
        {
            response.Clear();
            WebApp.SetNoCache(response);
            response.Write(data);
            response.End();
        }

        public void SetNoCache()
        {
            WebApp.SetNoCache(Response);
        }

        public static void SetNoCache(HttpResponse response)
        {
            response.Expires = 0;
            response.Cache.SetNoServerCaching();
            response.Cache.SetNoStore();
            response.AppendHeader("Pragma", "no-cache");
        }



        #endregion

    }

}
